/*	File:	mux_add.v	 			*/
/*                                                      */
/*	Description: input muxes for adder/subtractor 	*/
/*			uses cascade modules		*/

/*========================================================================*/


/* `include "/afs/ece.cmu.edu/usr/lowpower/dsp/alu/globals.v" */
`ifdef SIM
   `include "../cmudsp/structural/globals.v"
`else
   `include "./verilog/globals.v"
`endif


/****************************************************************************/
/*							*/
/*	module:						*/
/*							*/
/****************************************************************************/

module mux_add (
	logic_out,
	xy1, xy0,
	a2, a1, a0, b2, b1, b0,
	add_2a_sel,
	add_22c_sel, add_21c_sel, add_20c_sel,
	out
	);

/*============	I/O direction	===========================================*/

/* possible inputs to adder */
input [`databus]	logic_out;	/* in2 */
input [`databus]	a0, a1, b0, b1;	/* in2 */
input [`ext]		a2, b2;	/* in2 */
input [`databus]	xy1, xy0;	/* x0, x1, y0, y1, selected with mux_mult */
/* mux select signals */
input		add_2a_sel;
input [1:0]	add_22c_sel;
input [1:0]	add_21c_sel, add_20c_sel;
/* output to adder inputs */
output [`acc]	out;				

   
/*===================== internal =======================*/

/* in2 level 1: muxes to select a or b accumulator */
wire [`databus]	mux_21a_out, mux_20a_out;
wire [`ext]	mux_22a_out;
   


/*==========================================================================*/


/*==========================================================================*/
/*	processes							*/
/*==========================================================================*/


/* this is a cascade module */

/*---------- mux trees for in2 ------------------*/
/* first level: select among a, b */
`ifdef DW
   DW01_mux_any #(16, 1, 8) MUX_22A (
	{b2, a2}, add_2a_sel, mux_22a_out);
   DW01_mux_any #(48, 1, 24) MUX_21A (
	{b1, a1}, add_2a_sel, mux_21a_out);
   DW01_mux_any #(48, 1, 24) MUX_20A (
	{b0, a0}, add_2a_sel, mux_20a_out);
`else
   mux2 #(8,0) MUX_22A (
	a2, b2, add_2a_sel, mux_22a_out);
	
   mux2 #(24,1,"dp24") MUX_21A (
	a1, b1, add_2a_sel, mux_21a_out);
	
   mux2 #(24,1,"dp24") MUX_20A (
	a0, b0, add_2a_sel, mux_20a_out);
`endif	
/* first level: selecting among x and y */
/* this is done by muxes in mult_mux */


/* second level: select among first level, zero, or logic_unit */
`ifdef DW
   DW01_mux_any #(32, 2, 8) MUX_22C (
	{{8{xy1[23]}}, {8{xy1[23]}}, mux_22a_out, 8'h0}, add_22c_sel, out[55:48]);
   DW01_mux_any #(96, 2, 24) MUX_21C (
	{xy1, mux_21a_out, logic_out, 24'h0}, add_21c_sel, out[47:24]);
   DW01_mux_any #(96, 2, 24) MUX_20C (
	{xy0, xy0, mux_20a_out, 24'h0}, add_20c_sel, out[23:0]);
`else
   mux3 #(8, 0) MUX_22C (
	8'h0, mux_22a_out, {8{xy1[23]}},
        add_22c_sel[0], add_22c_sel[1], out[55:48]);
	
   mux4 #(24,1,"dp24") MUX_21C (
	24'h0, logic_out, mux_21a_out, xy1,
	add_21c_sel[0], add_21c_sel[1], out[47:24]);
   
   mux3 #(24,1,"dp24") MUX_20C (
	24'h0, mux_20a_out, xy0, 
	add_20c_sel[0], add_20c_sel[1], out[23:0]);
`endif

endmodule	/* end module */



