/*	File:	mux_logic.v	 			*/

/*	Description: input muxes for logic unit 	*/
/*			uses cascade modules		*/

/*========================================================================*/

/* `include "/afs/ece.cmu.edu/usr/lowpower/dsp/alu/globals.v" */
`ifdef SIM
   `include "../cmudsp/structural/globals.v"
`else
   `include "./verilog/globals.v"
`endif


/***************************************************************************/
/*							*/
/*	module:						*/
/*							*/
/***************************************************************************/

module mux_logic (
	x0, x1, y0, y1,
	a1, b1,
	logic_in1_sel,
	logic_in2_sel,
	logic_ena,
	in1, in2
	);

/*============	I/O direction	===========================================*/

   input [`databus]  x0, x1, y0, y1;
   input [`databus]  a1, b1;
   input	     logic_in1_sel;
   input [1:0]	     logic_in2_sel;
   input	     logic_ena;
   
   output [`databus] in1, in2;

/*=========================================================================*/


/*==================== I/O type ===========================================*/
`ifdef DW
   reg [`databus]            in1, in2;
`else
   wire [`databus]	     in1, in2;
`endif
/*===================== internal =======================*/

   wire [`databus]   mux_in1, mux_in2;

   
/*==========================================================================*/


/*==========================================================================*/
/*	processes							*/
/*==========================================================================*/


/* in1: select among a, b */
/* in1: select among x0, x1, y0, y1 */

`ifdef DW
   DW01_mux_any #(48, 1, 24) MUX_1 (
	{b1, a1}, logic_in1_sel, mux_in1);
   DW01_mux_any #(96, 2, 24) MUX_2 (
	{y1, x1, y0, x0}, logic_in2_sel, mux_in2);
`else   
   mux2 #(24,1,"dp24") MUX_1 (
	a1, b1, logic_in1_sel, mux_in1);

   mux4 #(24,1,"dp24") MUX_2 (
	x0, y0, x1, y1, logic_in2_sel[0], logic_in2_sel[1], mux_in2);
`endif 
   
/* Guard latches */
`ifdef DW
   always @(logic_ena or mux_in1 or mux_in2)
       if (logic_ena)
	begin
	   in1 <= mux_in1;
	   in2 <= mux_in2;
	end
`else
   latch #(24, 1, "dp24") LATCH_1
      (mux_in1, logic_ena, in1);
   
   latch #(24, 1, "dp24") LATCH_2
      (mux_in2, logic_ena, in2);
`endif   

endmodule	/* end module */



