/*	File:	mux_mult.v	 			*/

/*	Description: input muxes for multiplier 	*/
/*			uses cascade modules		*/
/*                   Includes guard latches             */

/*========================================================================*/


/* `include "/afs/ece.cmu.edu/usr/lowpower/dsp/alu/globals.v" */

`ifdef SIM
   `include "../cmudsp/structural/globals.v"
`else
   `include "./verilog/globals.v"
`endif


/***************************************************************************/
/*																			*/
/*	module:																	*/
/*																			*/
/***************************************************************************/

module mux_mult (
	x0, x1, y0, y1,
	decode_out,
	mult_aa_ctl, mult_bb_ctl,
	mult_a_ctl,
	mult_ena,
	aa, a, bb, b
	);

/*============	I/O direction	===========================================*/

   input [`databus] x0, x1, y0, y1;
   input [`databus] decode_out;
   input [1:0]	    mult_aa_ctl, mult_bb_ctl;
   input	    mult_a_ctl;
   input	    mult_ena;
   
   output [`databus] aa;    // feeds adder input
   output [`databus] bb;    // feeds adder input
   output [`databus] a;
   output [`databus] b;

/*=========================================================================*/


/*==================== I/O type ===========================================*/

   wire [`databus]   aa;
`ifdef DW
   reg [`databus]    a, b;
`else   
   wire [`databus]   a, b;
`endif
/*===================== internal =======================*/

   wire [`databus]   mux_aa_out;
   wire [`databus]   mux_a_out, mux_b_out;
   

/*==========================================================================*/


/*==========================================================================*/
/*	processes							*/
/*==========================================================================*/


/* this is a cascade module */

/* first level: select among x0, x1, y0, y1 */
`ifdef DW
   DW01_mux_any #(96, 2, 24) MUX_AA(
       {y1, x1, y0, x0}, mult_aa_ctl, mux_aa_out);
   DW01_mux_any #(96, 2, 24) MUX_BB(
       {x1, y1, y0, x0}, mult_bb_ctl, mux_b_out);
`else   
   mux4 #(24,1,"dp24") MUX_AA (
	x0, y0, x1, y1, mult_aa_ctl[0], mult_aa_ctl[1], mux_aa_out);

   mux4 #(24,1,"dp24") MUX_BB (
	x0, y0, y1, x1, mult_bb_ctl[0], mult_bb_ctl[1], mux_b_out);
`endif 
   
assign aa = mux_aa_out;
assign bb = mux_b_out;
   

/* second level, a: choose among decode or mult_aa */
`ifdef DW
   DW01_mux_any #(48, 1, 24) MUX_A(
       {mux_aa_out, decode_out}, mult_a_ctl, mux_a_out);
`else   
   mux2 #(24,1,"dp24") MUX_A (
	decode_out, mux_aa_out, mult_a_ctl, mux_a_out);
`endif
   
/* Guard latches */
`ifdef DW
   always @(mult_ena or mux_a_out or mux_b_out)
    if (mult_ena)
     begin
	a <= mux_a_out;
	b <= mux_b_out;
     end 
`else   
   latch #(24, 1, "dp24") LATCH_A
      (mux_a_out, mult_ena, a);
   
   latch #(24, 1, "dp24") LATCH_B
      (mux_b_out, mult_ena, b);
`endif   

endmodule	/* end module */



