/********************************************************************************/
/*																				*/
/*	module:	pcu_control																	*/
/*																				*/
/********************************************************************************/
`ifdef SIM
   `include "../cmudsp/structural/globals.v"
`else
   `include "./verilog/globals.v"
`endif 

module pcu_control (
	reset,
        REPEAT,
	LF,
        PC,
        LA,
        LC,
        SP,
        PCwrite_2,
        PCwrite_3,
        PC_in_2,
        PC_in_3,
	LF_out,
	PC_out,
	LC_out,
	LA_out,
        LF_in_3,
        LFwrite_3,
        LCwrite_repeat,
 	LC_repeat,
	LC_temp,
        LC_do,
	LCwrite_do,
	LAwrite_2,
        SP_in_2,
  	SP_in_3,
	SPwrite_2,
	SPwrite_3,
	SSL_in_2,
	SSL_in_3,
   	SSLwrite_2,
	SSLwrite_3,
  	SSH_in_2,
	SSH_in_3,
   	SSHwrite_2,
	SSHwrite_3,
        PDB,
	REPEATset,
	PABwrite_out,
	test,

        PCwrite,
	PC_in,
	LFwrite,
	LF_in,
	LCwrite,
	LC_in,
	LAwrite,
	LA_in,
	SPwrite,
	SP_in,
	SSLwrite,
	SSL_in,
	SSHwrite,
	SSH_in,
	REPEATreset,
        PABwrite,
	SSa,
	SSb,
	LC_temp_write,
	LC_temp_in,
	pab_tri_en
	);


/*==================================================================================*/
/*============						I/O direction						============*/
/*==================================================================================*/

input reset;
input REPEAT;					/* repeat last word instruction signal */
input [`addrbus] PC;			/* program counter REGISTER 	*/
input [`addrbus] PC_in_2;		/* the new value for the program counter from stage 2			*/
input [`addrbus] PC_in_3;		/* the new value for the program counter from stage 3			*/
input PCwrite_2;				/* program counter write enable for value coming from stage 2	*/
input PCwrite_3;				/* program counter write enable for value coming from stage 3	*/
input [`addrbus] SP;				/* Stack pointer REGISTER 					*/
input [`addrbus] SP_in_2;			/* the new value for the SP from stage 2	*/
input [`addrbus] SP_in_3;			/* the new value for the SP from stage 3	*/
input SPwrite_2;					/* SP write enable from stage 2				*/
input SPwrite_3;					/* SP write enable from stage 3				*/
input [`databus] PDB;
input [`addrbus] SSL_in_2;		/* System Stack Low input from stage 2		*/
input [`addrbus] SSL_in_3;		/* System Stack Low input from stage 3		*/
input SSLwrite_2;					/* SSL write enable from stage 2			*/
input  SSLwrite_3;					/* SSL write enable from stage 3			*/

input [`addrbus] SSH_in_2;		/* System Stack High input from stage 2		*/
input [`addrbus] SSH_in_3;		/* System Stack High input from stage 3		*/
input SSHwrite_2;					/* SSH write enable from stage 2			*/
input SSHwrite_3;					/* SSH write enable from stage 3			*/
input [`addrbus] LC;		   /* Loop counter REGISTER			*/
input [`addrbus] LC_repeat;		/* the initial number of repetitions for a REP inst				*/
input [`addrbus] LC_temp;    /* the value of LC before a REP inst */

input [`addrbus] LC_do;			/* the initial number of iterations for a DO loop, from stage 3	*/
input LCwrite_repeat;				/* the write enable for the loop counter's initial value		*/ 
input LCwrite_do;		/* the write enable for the loop counter from stage 3 (DO)		*/ 
input [`addrbus] LA;				/* Loop address REGISTER					*/
input LAwrite_2;					/* loop address write enable from stage 2	*/ 
input LF_in_3;							/* A new LF bit coming from stage 3 (DO instruction begins) */
input LFwrite_3;							/* write enable from stage 3 to start a DO loop */
input LF;
input [`addrbus] LF_out;
input [`addrbus] PC_out;
input [`addrbus] LC_out;
input [`addrbus] LA_out;
input REPEATset;				/* set the repeat signal (flip-flop) */


input PABwrite_out;
input test;
   

   
output SPwrite;					/* stack pointer write enable				*/
output [`addrbus] SP_in;			/* the input for the stack pointer register	*/
output [`addrbus] SSL_in;			/* System Stack Low input					*/
output SSLwrite;			/* SSL write enable							*/
output SSHwrite;			/* SSH write enable							*/
output [`addrbus] LC_in;		/* the input to the loop counter register	*/
output [`addrbus] SSH_in;			/* System Stack High input					*/
output LCwrite;					/* the write enable for the loop counter register*/ 
output LAwrite;					/* LA register write enable					*/ 
output [`addrbus] LA_in;			/* the new input to the loop address		*/
output LF_in;								/* The new LF bit before latching */
output REPEATreset;			/* reset the repeat signal (flip-flop) */

output [`addrbus] PC_in;		/* the input to the program counter register					*/
output PCwrite;				/* PC register write enable	*/
output LFwrite;							/* write enable */
output PABwrite;

output [3:0] SSa;
output [3:0] SSb;

output LC_temp_write;    /* write enable for the LC_temp register */
output [`addrbus] LC_temp_in;  /* input to the LC_temp register */

output pab_tri_en;
   

reg SPwrite;					/* stack pointer write enable				*/
reg [`addrbus] SP_in;			/* the input for the stack pointer register	*/
reg [`addrbus] SSL_in;			/* System Stack Low input					*/
reg SSLwrite;			/* SSL write enable							*/
reg SSHwrite;			/* SSH write enable							*/
reg [`addrbus] LC_in;		/* the input to the loop counter register	*/
reg [`addrbus] SSH_in;			/* System Stack High input					*/
reg LCwrite;					/* the write enable for the loop counter register*/ 
reg LAwrite;					/* LA register write enable					*/ 
reg [`addrbus] LA_in;			/* the new input to the loop address		*/
reg LF_in;								/* The new LF bit before latching */
reg REPEATreset;			/* reset the repeat signal (flip-flop) */
reg [`addrbus] PC_in;		/* the input to the program counter register					*/
reg PCwrite;				/* PC register write enable	*/
reg LFwrite;							/* write enable */
reg PABwrite;

reg [3:0] SSa;
reg [3:0] SSb;

reg LC_temp_write;    /* write enable for the LC_temp register */
reg [`addrbus] LC_temp_in;  /* input to the LC_temp register */


/*** tri-state buffer enable signal for PAB ***/   
reg pab_tri_en;   
/*==============================================================================================*/
/*==============================================================================================*/
/*																								*/
/*	Processes								Processes							Processes		*/
/*																								*/
/*==============================================================================================*/
/*==============================================================================================*/

/*	LF mode bit																	*/
/*																				*/
/*==============================================================================*/

/* select the input to the LF bit */

reg [15:0] tmp;

always @(	        LF_in_3 or
			LFwrite_3 or
			LF or
			LA or
			PC or
			LC or
                        SP or
                        tmp or
			LF_out
			)
	begin
		if ( LF && (LA == PC) && (LC == 16'h1) )	/* is this the end of the last iteration of a loop (DO inst)? */
			begin									/* executed while the last word in the loop is fetched */
				tmp = LF_out;
				//tmp = SSL[SP[3:0] - 1'h1];
				LF_in = tmp[15];					/* restore LF from SSL */
				LFwrite = `true;
			end

		else
			begin									/* change LF when a DO instruction starts, coming from stage 3 */
				LF_in = LF_in_3;
				LFwrite = LFwrite_3;
			end
	end



/*==============================================================================*/
/*																				*/
/*	Program Address Bus (PAB_out)												*/
/*																				*/
/*==============================================================================*/

/*---------------------------------------------------------------*/
/* select PAB write enable based on the PC and the REPEAT signal */
/*---------------------------------------------------------------*/

always @(	reset or
			PCwrite or
			REPEAT or
			REPEATset or 
			REPEATreset
			)
	begin
		if (reset)
			PABwrite = `true;  /* allow PAB set to 16'h0000 */

		/* stop driving the bus when REP starts, using REPEATset signal, and until REP starts its last repetition, when REPEATreset goes high */
		else if ( (REPEAT || REPEATset) && (~REPEATreset) )
			PABwrite = `false;

		else
			PABwrite = PCwrite;
	end



/*==============================================================================================*/
/*																								*/
/*	Program Counter (PC)																		*/
/*																								*/
/*==============================================================================================*/

/*--------------------------------------------------*/
/* select the new value for the PC and write enable */
/*--------------------------------------------------*/

always @(	reset or
			LF or
			PC or
			LA or
			SP or
			LC or
			PCwrite_2 or
			PCwrite_3 or
			PC_in_2 or
			PC_in_3 or
			REPEAT or
			REPEATreset or
			PC_out
			)
			
	begin
		if (reset)
			begin
				PCwrite = `false;
				PC_in = 16'h0;    /* for synthesis, has to specify PC_in in every control path */
			end
		else if (LF && (LA == PC) && (LC != 16'h1) )		/* at the end of a loop iteration (DO instruction) */
			begin											/* executed while the last word in the loop is fetched */
				PC_in = PC_out;
				//PC_in = SSH[SP[3:0] - 1'h1];				/* PC gets the address of the first inst in the loop */
				PCwrite = `true;
			end

		/* 
		 * stop counting when REPEAT signal goes high, which at that time PC 
		 * already contains the new PC for the instruction after the repeated 
		 * one. resume counting when the last repetition is starting in stage 2 
		 * and REPEATreset goes high.
		 */
		else if ( (REPEAT) && (~REPEATreset))	/* while REP is in progress, don't advance the PC */
			begin
				PCwrite = `false;		
				PC_in = 16'hx;    /* for synthesis, has to specify PC_in in every control path */
			end
		else if ( PCwrite_3 )								/* writing from stage 3 has priority over those from stage 2 */
			begin
				PC_in = PC_in_3;
				PCwrite = `true;
			end

		else if ( PCwrite_2 && (~PCwrite_3) )				/* write new PC from stage 2 */
			begin
				PC_in = PC_in_2;
				PCwrite = `true;
			end

		else
			begin
				PCwrite = `false;
				PC_in = 16'hx;    /* for synthesis, has to specify PC_in in every control path */
			end
	end



/*---------------------------------------*/
/* loop counting and repeat-signal reset */
/*---------------------------------------*/

always @(	reset or
			REPEAT or
			LC or
			LC_in or
			LC_repeat or
			LC_temp or
			LC_do or
			LCwrite_repeat or
			LCwrite_do or
			LF or
			LA or
			PC or
			SP or
			LC_out
			)
	begin
		REPEATreset = `false;   /* for the sake of synthesis */
		LC_in = 16'hx;   /* for the sake of synthesis */				
		LC_temp_write = `false;
		LC_temp_in = 16'hx;

                if (reset)
			begin
				LCwrite = `false;
				REPEATreset   = `false;
				
			end

			
		else if (REPEAT && LCwrite_repeat /*&& (~LF)*/)		/* REP first cycle - load initial repetitions and decrease by one */
			begin
				LC_in =  LC_repeat - 1'b1;				/* decrease the number of repetitions using the initial value */

				if (LC_in == 16'h1)						/* conclude REP execution? */
					begin
						REPEATreset = `true;			/* reset the repeat signal once repetition should cease */
						LCwrite   = `true;
						LC_in = LC_temp;  /* restore LC after a REP inst */
					end
				else
					begin
						REPEATreset   = `false;			/* REP is still in effect */
						LCwrite = `true;				/* keep counting down */
						



						LC_temp_write = `true;
						LC_temp_in = LC;     /* store current LC to LC_temp reg */
					end
			end		/* REP 1st cycle */
			

		else if (REPEAT && (~LCwrite_repeat) /*&& (~LF)*/)	/* REP non-first cycle */
			begin
				LC_in =  LC - 1'b1;						/* decrease the number of repetitions using the stored LC */

				if (LC_in == 16'h1)						/* conclude REP execution? */
					begin
						REPEATreset = `true;			/* reset the repeat signal when repetition should cease */
						LCwrite     = `true;			/* stop loop counting */
						LC_in = LC_temp; /* restore LC after a REP inst */
					end
				else
					begin
						REPEATreset = `false;			/* REP is still in effect */
						LCwrite     = `true;			/* keep counting down */
					end
			end		/* REP non-1st cycle */
			

		else if ((~REPEAT) && LCwrite_do )		/* DO instruction first iteration */
														/* Effective while the first instruction in the loop is fetched for the fisrt time. */
			begin
				LC_in = LC_do;							/* number of repetitions for the DO loop - coming from stage 3 */
				LCwrite = LCwrite_do;
			end


		else if ((~REPEAT) && LF && (LA == PC) && (LC != 16'h1) )
			begin										/* DO instruction is in progress - is not the last iteration */
														/* executed while the last word in the loop is fetched */
														/* update LC at the end of each iteration */
				LC_in = LC - 1'b1;						/* count down the number of remaining iterations */
				LCwrite = `true;
			end

		else if ((~REPEAT) && LF && (LA == PC) && (LC == 16'h1) )
			begin										/* conclude DO execution? */
														/* executed while the last word in the loop is fetched */
				LC_in = LC_out;
				//LC_in = SSL[SP[3:0] - 2'h2];			/* Restore LC from SSL */
				LCwrite = `true;
			end
		
		else
			begin
				REPEATreset = `false;
				LCwrite = `false;
				LC_in = 16'hx;    /* for the sake of synthesis */
			end

	end		/* LC */




/*==============================================================================================*/
/*																								*/
/*	Loop Address (LA)																			*/
/*																								*/
/*==============================================================================================*/

/* Latch new loop address which is always the second word in a DO instruction */
/* The Address given in the second word is of the instruction AFTER the last instruction inside the loop. */
/* That's why the last address is that address minus one. */

always @(	PDB or
			LF or
			LA or
			PC or
			LC or
			SP or
			LAwrite_2 or
			LA_out
			)
	begin
		if ( LF && (LA == PC) && (LC == 16'h1) )		/* is this the end of the last iteration of a loop (DO inst) */
			begin										/* executed while the last word in the loop is fetched */
				LA_in = LA_out;
				//LA_in = SSH[SP[3:0] - 2'h2];			/* restore LA from SSH */
				LAwrite = `true;
			end
		else
			begin
				LA_in = PDB[`addrbus]/* - 1'b1*/;			/* input to the LA register */
				LAwrite = LAwrite_2;					/* write enable from stage 2 (DO inst) */
			end
	end




/*==============================================================================================*/
/*																								*/
/*	Stack Pointer (SP)																			*/
/*																								*/
/*==============================================================================================*/

/* select the new value for the stack pointer */

always @(	SP_in_2 or
			SP_in_3 or
			SPwrite_2 or
			SPwrite_3 or
			LF or
			LA or
			PC or
			LC or
			SP
			)
	begin
		if ( LF && (LA == PC) && (LC == 16'h1) )		/* is this the end of the last iteration of a loop (DO inst) */
			begin										/* executed while the last word in the loop is fetched */
				SP_in = SP[5:0] - 2'h2;
				SPwrite = `true;
			end
		else if ( SPwrite_2 && (~SPwrite_3) )			/* from stage 2 */
			begin
				SP_in = SP_in_2;
				SPwrite = SPwrite_2;
			end
		else if ( (~SPwrite_2) && SPwrite_3 )			/* from stage 3 */
			begin
				SP_in = SP_in_3;
				SPwrite = SPwrite_3;
			end
		else
			begin
				SPwrite = `false;
				SP_in = 16'hx;		/* for synthesis */
			end
	end


/*==============================================================================================*/
/*																								*/
/*	System Stack (SSL & SSH)																	*/
/*																								*/
/*==============================================================================================*/

/* System Stack input and write enable selectors */

/*---------------*/
/* SSL selectors */
/*---------------*/

always @(	SSL_in_2 or
			SSL_in_3 or
			SSLwrite_2 or
			SSLwrite_3
			)
	begin
		if ( SSLwrite_2 && (~SSLwrite_3) )
			begin
				SSL_in = SSL_in_2;
				SSLwrite = SSLwrite_2;
			end
		else if ( (~SSLwrite_2) && SSLwrite_3 )
			begin
				SSL_in = SSL_in_3;
				SSLwrite = SSLwrite_3;
			end
		else
			begin
				SSLwrite = `false;
				SSL_in = 16'hx;		/* for synthesis */
			end
	end		/* SSL */

/*---------------*/
/* SSH selectors */
/*---------------*/

always @(	SSH_in_2 or
			SSH_in_3 or
			SSHwrite_2 or
			SSHwrite_3
			)
	begin
		if ( SSHwrite_2 && (~SSHwrite_3) )
			begin
				SSH_in = SSH_in_2;
				SSHwrite = SSHwrite_2;
			end
		else if ( (~SSHwrite_2) && SSHwrite_3 )
			begin
				SSH_in = SSH_in_3;
				SSHwrite = SSHwrite_3;
			end
		else
			begin
				SSHwrite = `false;
				SSH_in = 16'hx;      /* for synthesis */
			end
	end		/* SSH */

// Caculating the correct stack address
always @(SP or SSHwrite or SSLwrite)
begin
	if (SSHwrite || SSLwrite) /* if it's a write, write to sp+1 since sp hasn't been updated yet. if it's a read, SP is pointing to the top of the stack, that's right */
		SSa = SP[3:0] + 1'b1;
       else
		SSa = SP[3:0];
	SSb = SSa - 1'h1;
end

/*===========================================================================*/
/*=    Generate the tri-state buffer enable signal for PAB                  =*/
/*===========================================================================*/
always @(test or PABwrite_out)
	begin
		if (!test && PABwrite_out)
		 	pab_tri_en = 1'b0;
		else
			pab_tri_en = 1'b1;   // hi-Z
	 end   
endmodule	/* pcu_control */


