/* $Id: addr_overflow_detect.v,v 1.3 1997/10/13 21:34:15 inacio Exp $ */
/*****************************************************************************
 *addr_overflow_detect
 *
 *detects if the addresses on c_xab, c_pab, or c_yab have gone over the
 *SRAM size
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: inacio $
 *Date: $Date: 1997/10/13 21:34:15 $
 *Revision: $Revision: 1.3 $
 *
 *
 *Log: $Log: addr_overflow_detect.v,v $
 *Log: Revision 1.3  1997/10/13 21:34:15  inacio
 *Log: some small changes for synopsys and make the PAB part correct
 *Log:
# Revision 1.2  1997/10/13  20:38:41  inacio
# The is the logic to detect when an address is out of range to any of the
# internal memories
#
# Revision 1.1  1997/10/10  20:30:10  inacio
# some logic to detect if we overflow the internal SRAM sizes
#
 *****************************************************************************/

module addr_overflow_detect (C_XAB, C_PAB, C_YAB, c_xab_ov, c_pab_ov, c_yab_ov);

	parameter num_detect_bits = 4;
	parameter addr_bitwidth = 16;

	input	[(addr_bitwidth)-1:0]C_XAB;
	input	[(addr_bitwidth)-1:0]C_PAB;
	input	[(addr_bitwidth)-1:0]C_YAB;

	output c_xab_ov;
	output c_yab_ov;
	output c_pab_ov;

	reg c_xab_ov;
	reg c_yab_ov;
	reg c_pab_ov;


always @(C_XAB or C_YAB or C_PAB)
begin
	if ( C_XAB[((addr_bitwidth)-1):((addr_bitwidth)-num_detect_bits)] != {{num_detect_bits {1'b0}}} )
		c_xab_ov <= 1;
		
	if ( C_YAB[((addr_bitwidth)-1):((addr_bitwidth)-num_detect_bits)] != {{num_detect_bits {1'b0}}} )
		c_yab_ov <= 1;

	if ( C_PAB[((addr_bitwidth)-1):((addr_bitwidth)-(num_detect_bits-1))] != {{num_detect_bits {1'b0}}} )
		c_pab_ov <= 1;


end


endmodule
