/* $Id: ebus_drivers_control.v,v 3.4 1998/02/05 18:45:21 benk Exp $ */
/*****************************************************************************
 *ebus_drivers_control.v
 *
 *control logic for the ebus drivers
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: benk $
 *Date: $Date: 1998/02/05 18:45:21 $
 *Revision: $Revision: 3.4 $
 *
 *
 *Log: $Log: ebus_drivers_control.v,v $
 *Log: Revision 3.4  1998/02/05 18:45:21  benk
 *Log: Changed some connections from inv_cpustop tp cpu_stop
 *Log:
# Revision 3.3  1998/01/27  18:55:44  benk
# Changed EDB tristates
#
# Revision 3.3  1997/1/24  23:14:32  benk
# Added comments,  disabled edb in illegal mode
#
# Revision 3.2  1997/12/16  23:14:32  inacio
# forgot to remove mem_select from the parameter list
#
# Revision 3.1  1997/12/16  20:28:11  inacio
# changed the sensitivity list and added the e_outen parameter to the module
#
# Revision 3.0  1997/12/16  20:25:42  inacio
# moved up to version 3.0
#
# Revision 2.3  1997/12/16  20:24:38  inacio
# The new version with Logic By Comittee (TM) by Ben Herman and Jon
#
# Revision 2.2  1997/12/13  05:33:32  inacio
# changed the enable logic for the eab and edb i/o pins.  It is basically the
# same logic now, but it makes more sense to read it.
#
# Revision 2.1  1997/12/03  00:23:08  inacio
# added reg's to the file for some assignments, stupid verilog!
#
# Revision 2.0  1997/11/28  05:53:26  inacio
# changing the revision level for the new interface version
#
# Revision 1.6  1997/11/28  05:52:58  inacio
# completely changed for the new bus definition.  Now this module only
# generates control signals for the pads
#
# Revision 1.5  1997/11/18  22:11:03  inacio
# reformatted the source
#
# Revision 1.4  1997/11/09  08:51:46  inacio
# I changed the tabbing in the files to be spaces, now it looks the same in
# less as it does in my editor
#
# Revision 1.3  1997/10/29  23:19:51  inacio
# changed the sensitivety list
#
# Revision 1.2  1997/10/29  22:50:14  inacio
# exported a new signal edb_dir_bar for controling the pads for the edb
#
# Revision 1.1  1997/10/14  20:19:14  inacio
# moved the file into before_synth subdir
#
# Revision 1.3  1997/10/13  21:41:57  inacio
# some modifications and bug fixes for Synopsys
#
# Revision 1.2  1997/10/13  20:38:56  inacio
# this is the control logic for the external bus drivers
#
# Revision 1.1  1997/10/10  20:30:55  inacio
# control logic for the ebus drivers, not done yet
#
 *****************************************************************************/


module ebus_drivers_control (eab_dir, eab_en, edb_dir, edb_en, cpu_stop, 
                           e_write, e_outen);

output   eab_dir;
output   eab_en;
output   edb_dir;
output   edb_en;
input    cpu_stop;
input    e_write;
input    e_outen;

reg      eab_dir;
reg      eab_en;
reg      edb_dir;
reg      edb_en;

always @(cpu_stop or e_outen or e_write)
   begin
      
      /*
       * pad enables are active low
       *
       * direction, 0 -> output
       *            1 -> input
       */

      if (cpu_stop == 1'b1)
         begin
            
            /*
             *
             *external port is always in output mode, the core is active
             *
             */
            eab_en <= 1'b0;
            eab_dir <= 1'b0;
            edb_en <= 1'b0;
            edb_dir <= 1'b0;
         end
      else	/* RAM mode */
      /*
      	e_outen | e_write | edb_en | edb_dir | direction
      	--------+---------+--------+---------+----------
      	   0  	|   0	  |   1    |   x     |  no connect
      	   0  	|   1	  |   0    |   0     |  output
      	   1  	|   0	  |   0    |   1     |  input
      	   1  	|   1	  |   0    |   1     |  input
      	   */
         begin

            if (e_outen == 1'b0 && e_write == 1'b0)
               begin
                  /* illegal, nothing connected */
                  eab_en <= 1'b1;
                  edb_en <= 1'b1;
                  
                  eab_dir <= 1'bx;
                  edb_dir <= 1'bx;
               end
            else
               begin
                  /* eab set to input, edb set according to truth table */
                  eab_en <= 1'b0;
                  eab_dir <= 1'b1;
                  edb_en <= 1'b0;
                  
                  if (e_outen == 1'b1)
                     begin
                        edb_dir <= 1'b1;
                     end
                  else
                     begin
                        edb_dir <= 1'b0;
                     end
               end
         end
   end
endmodule



