/* $Id: clock_stop.v,v 1.26 1998/07/08 21:07:55 ar39 Exp $ */
/*****************************************************************************
 *clock_stop.v
 *
 *This unit stops the clock to the core of the processor
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: ar39 $
 *Date: $Date: 1998/07/08 21:07:55 $
 *Revision: $Revision: 1.26 $
 *
 *
 *Log: $Log: clock_stop.v,v $
 *Log: Revision 1.26  1998/07/08 21:07:55  ar39
 *Log: I don't know what I did to clock_stop...
 *Log:
 *Log: Revision 1.25  1998/06/15 20:05:19  ar39
 *Log: Lengthened ESTROBE delay from 8 ns to 13 ns.
 *Log:
 *Log: Revision 1.24  1998/06/15 17:02:27  ar39
 *Log: Split more AGU clocks--whoo-hoo!
 *Log:
 *Log: Revision 1.23  1998/06/10 21:33:10  ar39
 *Log: Split the PCU clock.
 *Log:
 *Log: Revision 1.22  1998/06/02 20:58:10  ar39
 *Log: Split clock nets.
 *Log:
 *Log: Revision 1.21  1998/02/25 22:04:45  ar39
 *Log: Moved the clock attribute stuff from clock_stop.v
 *Log:
 *Log: Revision 1.20  1998/02/18 23:35:18  ar39
 *Log: Split the clock into the AGU.
 *Log:
 *Log: Revision 1.19  1998/02/14 16:29:25  ar39
 *Log: Added a third clock net.
 *Log:
 *Log: Revision 1.18  1998/02/10 23:19:39  benk
 *Log: Changed logic for ESTROBE, E_XWRITE, and E_YWRITE for writing in DSP mode
 *Log:
# Revision 1.17  1998/02/06  19:54:29  ar39
# Just tweaked the stddelaybuf...
#
 *Log: Revision 1.16  1998/02/05 22:20:04  benk
 *Log: Changed timing of estrobe to account for memory timing
 *Log: Added assign satement for RTL simulation
 *Log:
# Revision 1.15  1998/02/04  22:38:24  ar39
# Oops, ESTROBE wrong polarity... fixed.
#
 *Log: Revision 1.14  1998/01/30 23:51:56  ar39
 *Log: Implemented better write enable pulse, don't know if it works...
 *Log:
 *Log: Revision 1.13  1998/01/29 22:50:54  ar39
 *Log: Took out delay buffer, took out spurious net attributes
 *Log:
 *Log: Revision 1.12  1998/01/19 17:42:00  inacio
 *Log: added a buffer driver to the en_reg2 which outputs the clock stop signal, so
 *Log: it can drive the load to say the core is off
 *Log:
# Revision 1.11  1998/01/16  21:23:22  inacio
# moved stuff for clocks for cascade to detect them
#
# Revision 1.10  1998/01/16  00:20:32  inacio
# moved the //epoch comments for it to recognize the clocks
#
# Revision 1.9  1998/01/15  22:22:40  inacio
# tweaked the delay again
#
# Revision 1.8  1998/01/15  22:21:31  inacio
# added information for cascade to make pretty clock trees; and did some
# reformatting
#
# Revision 1.7  1998/01/08  19:38:22  inacio
# increased some of the driver sizes, tuned the delay to sync up the two
# clocks some more.
#
# Revision 1.6  1997/12/13  05:32:15  inacio
# added buffers to the clock signals for scaling to much larger drivers.
#
# Revision 1.5  1997/11/09  08:51:29  inacio
# I changed the tabbing in the files to be spaces, now it looks the same in
# less as it does in my editor
#
# Revision 1.4  1997/11/06  06:41:21  inacio
# had to invert the reset signal to the flip flops....
#
# Revision 1.3  1997/10/24  19:31:42  inacio
# added epoch clock comments to this to maybe help clock routing??
#
# Revision 1.2  1997/10/10  20:30:28  inacio
# some updates on signals
#
# Revision 1.1  1997/10/08  23:41:17  inacio
# This module has the clock stop unit.  It generates to clock out signals, one
# which can be disabled and one which can't.
#
 *****************************************************************************/





module clock_stop(Clkin,
		  Clk_agu_0_rfm, Clk_agu_1_rfn, Clk_agu_2_rfr,
		  Clk_agu_3_ctrl, Clk_agu_4_ctrl, Clk_agu_5_misc, Clk_agu_6_misc,
		  Clk_alu_0_xy, Clk_alu_1_ab, Clk_alu_2_misc,
                  Clk_bus_0, Clk_pcu_0, Clk_pcu_1,
		  ESTROBE, EENABLE, cpu_stop, reset);

   input  EENABLE;    // enable the external port, turn off the 
          // CPU clock
   input  reset;    // reset the chip
   input  Clkin;    // input from external crystal
   output Clk_agu_0_rfm, Clk_agu_1_rfn, Clk_agu_2_rfr;
   output Clk_agu_3_ctrl, Clk_agu_4_ctrl, Clk_agu_5_misc, Clk_agu_6_misc;
   output Clk_alu_0_xy, Clk_alu_1_ab, Clk_alu_2_misc;
   output Clk_bus_0, Clk_pcu_0, Clk_pcu_1;
   output ESTROBE; // clock for the periphery (not core) of chip, DOES HALT
   output cpu_stop;   // indicates that the CPU has been halted safely

   wire  enreg1;
   wire  not_Clkin;
   wire  not_reset;
   wire  q;
   wire  unbuf_halt_clock;
   wire  unbuf_nonhalt_clock;
   wire  cpu_stop_buf;
   wire  Clk_agu_0_rfm, Clk_agu_1_rfn, Clk_agu_2_rfr;
   wire  Clk_agu_3_ctrl, Clk_agu_4_ctrl, Clk_agu_5_misc, Clk_agu_6_misc;
   wire  Clk_alu_0_xy, Clk_alu_1_ab, Clk_alu_2_misc;
   wire  Clk_bus_0, Clk_pcu_0, Clk_pcu_1, TEST, C_PAB;
   wire  Clk_bus_0_delay, Clk_bus_0_delay_tmp;

// epoch set_attribute net Clk_agu_0_rfm  : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_agu_0_rfm  : CLOCKGROUP = 1
// epoch set_attribute net Clk_agu_1_rfn  : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_agu_1_rfn  : CLOCKGROUP = 1
// epoch set_attribute net Clk_agu_2_rfr  : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_agu_2_rfr  : CLOCKGROUP = 1
// epoch set_attribute net Clk_agu_3_ctrl : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_agu_3_ctrl : CLOCKGROUP = 1
// epoch set_attribute net Clk_agu_4_ctrl : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_agu_4_ctrl : CLOCKGROUP = 1
// epoch set_attribute net Clk_agu_5_misc : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_agu_5_misc : CLOCKGROUP = 1
// epoch set_attribute net Clk_agu_6_misc : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_agu_6_misc : CLOCKGROUP = 1
// epoch set_attribute net Clk_alu_0_xy   : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_alu_0_xy   : CLOCKGROUP = 1
// epoch set_attribute net Clk_alu_1_ab   : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_alu_1_ab   : CLOCKGROUP = 1
// epoch set_attribute net Clk_alu_2_misc : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_alu_2_misc : CLOCKGROUP = 1
// epoch set_attribute net Clk_bus_0      : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_bus_0      : CLOCKGROUP = 1
// epoch set_attribute net Clk_pcu_0      : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_pcu_0      : CLOCKGROUP = 1
// epoch set_attribute net Clk_pcu_1      : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_pcu_1      : CLOCKGROUP = 1

   /*
    * This takes clockin and when the EENABLE line is asserted
    * turns off clockout halt
    */
   stdinv resetinv (reset, not_reset);
   stddff_c en_reg1 (Clkin, not_reset, EENABLE, enreg1);
   stdinv clkinv (Clkin, not_Clkin);
   stddff_cq en_reg2 (not_Clkin, not_reset, enreg1, q, cpu_stop_buf);
   stdbuf en_reg2_driver (cpu_stop_buf, cpu_stop);
   stdand2 clk_gate (cpu_stop, Clkin, unbuf_halt_clock);

   stdbuf clock_driver_agu_0_rfm  (unbuf_halt_clock, Clk_agu_0_rfm);
   stdbuf clock_driver_agu_1_rfn  (unbuf_halt_clock, Clk_agu_1_rfn);
   stdbuf clock_driver_agu_2_rfr  (unbuf_halt_clock, Clk_agu_2_rfr);
   stdbuf clock_driver_agu_3_ctrl (unbuf_halt_clock, Clk_agu_3_ctrl);
   stdbuf clock_driver_agu_4_ctrl (unbuf_halt_clock, Clk_agu_4_ctrl);
   stdbuf clock_driver_agu_5_misc (unbuf_halt_clock, Clk_agu_5_misc);
   stdbuf clock_driver_agu_6_misc (unbuf_halt_clock, Clk_agu_6_misc);
   stdbuf clock_driver_alu_0_xy   (unbuf_halt_clock, Clk_alu_0_xy);
   stdbuf clock_driver_alu_1_ab   (unbuf_halt_clock, Clk_alu_1_ab);
   stdbuf clock_driver_alu_2_misc (unbuf_halt_clock, Clk_alu_2_misc);
   stdbuf clock_driver_bus_0      (unbuf_halt_clock, Clk_bus_0);
   stdbuf clock_driver_pcu_0      (unbuf_halt_clock, Clk_pcu_0);
   stdbuf clock_driver_pcu_1      (unbuf_halt_clock, Clk_pcu_1);

   /*
    * Create a second clock that goes low 11 ns after 
    * Clkout goes high.  Same pulse width
    */
   stddelayps #(10000) clk_delay_buf (Clk_bus_0, Clk_bus_0_delay_tmp);
   // assign statment makes delay in rtl description
   assign #10 Clk_bus_0_delay = Clk_bus_0_delay_tmp;
   stdbuf ESTROBE_driver (Clk_bus_0_delay, ESTROBE);
   
endmodule
