/* $Id: ebus.v,v 2.1 1998/07/27 21:09:41 benk Exp $ */
/*****************************************************************************
 *ebus_drivers.v
 *
 *tribufs and drivers for the external bus interface
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: benk $
 *Date: $Date: 1998/07/27 21:09:41 $
 *Revision: $Revision: 2.1 $
 *
 *
 *Log: $Log: ebus.v,v $
 *Log: Revision 2.1  1998/07/27 21:09:41  benk
 *Log: Changed file names to match module names
 *Log:
# Revision 2.7  1998/02/11  16:13:41  ar39
# Took out the datapath parameters in the tribufs.
#
 *Log: Revision 2.6  1998/02/06 22:19:57  benk
 *Log: Removed rs_ab and rs_db.  Direct connection with EAB, EDB
 *Log:
# Revision 2.5  1998/02/05  18:45:07  benk
# Changed some connections from inv_cpustop tp cpu_stop
#
# Revision 2.4  1998/01/27  18:55:35  benk
# Changed EDB tristates
#
# Revision 2.3  1998/01/15  22:26:19  inacio
# changed a standard or from 3 to 4; forgot to increase when adding in scan
# chain stuff
#
# Revision 2.2  1998/01/15  19:24:52  inacio
# added support for the scan chain to disable the core to ebus connection;
# this completes the isolation of the core during scans
#
# Revision 2.1  1997/11/30  09:10:35  inacio
# added the necessary semicolons to the file
#
# Revision 2.0  1997/11/28  05:00:18  inacio
# updating the revision number (which I tried last time) to reflect that this
# is a completely new version
#
# Revision 1.6  1997/11/28  04:56:58  inacio
# The new version of the ebus with bus snooping instead of x bus overflow
#
# Revision 1.5  1997/11/09  08:51:30  inacio
# I changed the tabbing in the files to be spaces, now it looks the same in
# less as it does in my editor
#
# Revision 1.4  1997/10/21  17:19:25  inacio
# fixed some sizing and formatting
#
# Revision 1.3  1997/10/16  19:56:53  inacio
# fixed sending and receiving info from XDB, which doesn't exist to C_XDB
# which does exist
#
# Revision 1.2  1997/10/15  22:17:51  inacio
# I don't know what I changed, but I must have when I did it....
#
# Revision 1.1  1997/10/10  20:30:43  inacio
# drivers and some control logic for the ebus
#
 *****************************************************************************/


module ebus (e_write, mem_select, e_enable, scan, eab, edb, c_xab, c_xdb,
            c_yab, c_ydb, c_pab, c_pdb);

 parameter address_bitwidth = 16;
 parameter databus_bitwidth = 24;
 parameter datapath = 0;



input    e_write;
input    [1:0]mem_select;
input    e_enable;
input    scan;
inout    [(address_bitwidth)-1:0]eab;
inout    [(databus_bitwidth)-1:0]edb;
input    [(address_bitwidth)-1:0]c_xab;
input    [(databus_bitwidth)-1:0]c_xdb;
input    [(address_bitwidth)-1:0]c_yab;
input    [(databus_bitwidth)-1:0]c_ydb;
input    [(address_bitwidth)-1:0]c_pab;
input    [(databus_bitwidth)-1:0]c_pdb;
          
wire     xbus_enable;
wire     not_mem_select0;
wire     ybus_enable;
wire     partial_pbus_enable;
wire     pbus_enable;



   /*
    * this creates the control logic for the ebus switch
    * all of the control signals are active low
    *
    * mem_select                 bus
    * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    *    00                      X
    *    01                      Y
    *    10                     (UNDEFINED)
    *    11                      P
    *
    * so based on mem_select and e_enable being low, (deactivating the
    * external port access to internal ram) snoop a bus determined by
    * mem_select
    *
    */
   stdor4 xb_or (mem_select[0], mem_select[1], e_enable, scan, xbus_enable);
   stdinv ms0_inv (mem_select[0], not_mem_select0);
   stdor4 yb_or (not_mem_select0, mem_select[1], e_enable, scan, ybus_enable);
   stdnand2 pb_nand (mem_select[0], mem_select[1], partial_pbus_enable);
   stdor3 pb_or (partial_pbus_enable, e_enable, scan, pbus_enable);


   /*
    *
    * tribuf #(num_bits, datapath_flag, "group_name", "buf_size", "optimize") 
    *       name (enable, input, output);
    *
    */

   /*
    * Address bus - core to external
    */
   tribuf #(address_bitwidth, datapath) cxab2eab_tribuf 
      (xbus_enable, c_xab, eab);
   tribuf #(address_bitwidth, datapath) cyab2eab_tribuf 
      (ybus_enable, c_yab, eab);
   tribuf #(address_bitwidth, datapath) cpab2eab_tribuf 
      (pbus_enable, c_pab, eab);
   
   /*
    * Data bus - core to external
    */
   tribuf #(databus_bitwidth, datapath) cxdb2edb_tribuf 
      (xbus_enable, c_xdb, edb);
   tribuf #(databus_bitwidth, datapath) cydb2edb_tribuf 
      (ybus_enable, c_ydb, edb);
   tribuf #(databus_bitwidth, datapath) cpdb2edb_tribuf 
      (pbus_enable, c_pdb, edb);
   
   
endmodule




