/* $Id: ram_switch.v,v 1.36 1998/06/16 15:19:13 ar39 Exp $ */
/*****************************************************************************
 *ram_switch.v
 *
 *Interface from the memory bus to the outside RAM units.  Also from the 
 *outside bus to the internal RAMs.
 *
 *
 *Author: $Author: ar39 $
 *Date: $Date: 1998/06/16 15:19:13 $
 *Revision: $Revision: 1.36 $
 *
 *
 *Log: $Log: ram_switch.v,v $
 *Log: Revision 1.36  1998/06/16 15:19:13  ar39
 *Log: Made PAB a true bi-directional bus.
 *Log:
 *Log: Revision 1.35  1998/06/15 17:02:29  ar39
 *Log: Split more AGU clocks--whoo-hoo!
 *Log:
 *Log: Revision 1.34  1998/06/10 21:33:12  ar39
 *Log: Split the PCU clock.
 *Log:
 *Log: Revision 1.33  1998/06/02 20:58:11  ar39
 *Log: Split clock nets.
 *Log:
 *Log: Revision 1.32  1998/02/24 22:45:19  inacio
 *Log: deleted commented out code
 *Log:
# Revision 1.31  1998/02/19  20:03:41  benk
# Removed x overflow detect from port of thingy_control
#
# Revision 1.30  1998/02/18  23:35:19  ar39
# Split the clock into the AGU.
#
 *Log: Revision 1.29  1998/02/18 23:01:54  benk
 *Log: Removed unnecessary logic
 *Log:
# Revision 1.28  1998/02/14  16:29:28  ar39
# Added a third clock net.
#
 *Log: Revision 1.27  1998/02/10 04:40:28  inacio
 *Log: swapped out the old ram_enable_control that was behavioral for the new one
 *Log: which is structural and supposed to be glitch free.
 *Log:
# Revision 1.26  1998/02/06  22:19:58  benk
# Removed rs_ab and rs_db.  Direct connection with EAB, EDB
#
# Revision 1.25  1998/02/05  18:45:08  benk
# Changed some connections from inv_cpustop tp cpu_stop
#
# Revision 1.24  1998/01/27  18:55:37  benk
# Changed EDB tristates
#
# Revision 1.23  1998/01/24  19:25:38  benk
# Replaces rs_db with EDB.  Direct connection.
#
# Revision 1.23  1998/01/15  19:25:38  inacio
# added signals and support for scan chain stuff to the ebus, passing the
# signals along in here
#
# Revision 1.22  1997/12/17  19:28:23  inacio
# changed the number of detection bits so that it's using 10-bits insteazd of
# 9-bits.
#
# Revision 1.21  1997/12/16  21:08:02  inacio
# changed the parameters to the ebus_drivers_control module so that it matches
# with the new logic by herman/jon/ben
#
# Revision 1.20  1997/12/14  23:20:18  benk
# Rebuilt control for thingy.  All logic in thingy_control.
# Added X_READ, Y_READ to enable signals.
#
# Revision 1.19  1997/12/13  06:26:04  inacio
# changed some formatting, that's all
#
# Revision 1.18  1997/11/30  09:09:15  inacio
# created an inverted cpu_stop signal for use in the ebus controller and
# replaced EENABLE with it.
#
# Revision 1.17  1997/11/28  05:59:44  inacio
# changed the modules to handle the new version of the ebus definition
#
# Revision 1.16  1997/11/18  23:37:18  inacio
# added parameters to the module whcih flow down into the addr_overflow_detect
# module
#
# Revision 1.15  1997/11/09  09:57:19  inacio
# added overflow detection to the rams so that addresses don't rap around, the
# rams turn off and you read or get high impedance back
#
# Revision 1.14  1997/11/09  08:51:31  inacio
# I changed the tabbing in the files to be spaces, now it looks the same in
# less as it does in my editor
#
# Revision 1.13  1997/11/07  22:39:36  inacio
# reformatted the parameter list so its more compact and I can read it now
#
# Revision 1.12  1997/10/29  22:48:38  inacio
# added changes to support control lines for bidirectional pads and the new
# test signal for the scan chain
#
# Revision 1.11  1997/10/21  17:20:29  inacio
# added some necessary wires and fixed some sizing problems
#
# Revision 1.10  1997/10/17  18:58:06  inacio
# finished the changes for parameterizing the ram_switch module
#
# Revision 1.9  1997/10/16  22:10:46  inacio
# added some parameters to the model
#
# Revision 1.8  1997/10/16  19:56:13  inacio
# cleaned up some missing parameters to some modules, makes YAB work.
#
# Revision 1.7  1997/10/15  22:15:43  inacio
# Added the rest of the modules to the ram_switch module
#
# Revision 1.6  1997/10/13  21:49:56  inacio
# added the necessary signals to the different units
#
# Revision 1.5  1997/10/13  20:39:23  inacio
# added a bunch more stuff-- a lot more complete
#
# Revision 1.4  1997/10/10  20:31:05  inacio
# updated some stuff in here too
#
# Revision 1.3  1997/10/08  23:38:58  inacio
# Major changes have been made.  I'm more modular and there's a lot less in
# this file.
#
# Revision 1.2  1997/10/06  22:35:29  inacio
# some updates, about to redo the file!
#
# Revision 1.1  1997/10/01  21:14:30  inacio
# beginnings of the ram switch
#
 *****************************************************************************/

/*
 * control signal truth tables....
 *
 *
 *  MEM_SLCT     |
 *  -------------+-----------------------
 *    00         |   X memory
 *    01         |   Y memory
 *    10         |   (reserved)
 *    11         |   P memory
 *
 *
 *
 *  EENABLE      |
 *  -------------+-----------------------
 *     0         |   cpu active
 *               |   (external port is x mem extension) 
 *               |
 *     1         |   cpu halt
 *               |   (external port controls internal SRAMs)
 *
 */




/*****************************************************************************
 *module: ram_switch
 *
 *
 *****************************************************************************/
module ram_switch (reset, MEM_SLCT, ESTROBE, EACCESS, EENABLE, Clkin,
 		   Clk_agu_0_rfm, Clk_agu_1_rfn, Clk_agu_2_rfr,
	 	   Clk_agu_3_ctrl, Clk_agu_4_ctrl, Clk_agu_5_misc, Clk_agu_6_misc,
		   Clk_alu_0_xy, Clk_alu_1_ab, Clk_alu_2_misc,
                   Clk_bus_0, Clk_pcu_0, Clk_pcu_1, 
		   test, C_PAB, C_PDB, C_XAB, C_XDB, CX_READ,
		   CX_WRITE, C_YAB, C_YDB, CY_READ, CY_WRITE, E_PAB, E_PDB_IN,
		   E_PDB_OUT, EP_OUTEN, EP_WRITE, E_XAB, E_XDB_IN, E_XDB_OUT,
		   EX_OUTEN, EX_WRITE, E_YAB, E_YDB_IN, E_YDB_OUT, EY_OUTEN, EY_WRITE,
		   EAB, EDB, E_OUTEN, E_WRITE, edb_dir, edb_en, eab_dir, eab_en);

/***************************************************************************** 
 *                           Signal descriptions
 *
 *
	reset		- reset the module 
				 

	MEM_SLCT	- select which memory to access when turning accessing from
	           external memory

	ESTROBE	- clock signal to external devices
				  

	EACCESS	- the port is active from an internal access request

	EENABLE	- turn on access from the external port and stop the CPU
	           clock

	Clkin		- clkin is from the xtal, clkout goes to the cpu, which can
   Clkout	- be deactivated

	test		- the core is getting scanned signal 
	
	
	**** Core side bus signals

	C_PAB		- CPU P bus 
	C_PDB


	CP_READ	- these two don't exist....
	CP_WRITE

	C_XAB		- CPU X bus
	C_XDB
	CX_READ
	CX_WRITE

	C_YAB		- CPU Y bus
	C_YDB
	CY_READ
	CY_WRITE


	***** Internal RAM side bus signals

	E_PAB		- RAM P bus
	E_PDB_IN
	E_PDB_OUT
	EP_OUTEN
	EP_WRITE

	E_XAB		- RAM X bus
	E_XDB_IN
	E_XDB_OUT
	EX_OUTEN
	EX_WRITE

	E_YAB		- RAM Y bus
	E_YDB_IN
	E_YDB_OUT
	EY_OUTEN
	EY_WRITE

	**** External bus side signals

	EAB		- External address and data bus
	EDB
	E_OUTEN
	E_WRITE	- (Active low)
	
	***** Control signals for I/O pads

	edb_dir
	edb_en
	eab_dir
	eab_en

*****************************************************************************/

//*****************************************************************************
// *  I/O port definition                _______    _______   _______
// *	                                   | X RAM |  | Y RAM | | P RAM |
// *  ___                        ___       ^  ^      ^   ^
// * | C | ===P Addr Bus====>   | R |E_XAB "  "      "   "
// * |   |                      | A |======"  "      "   "
// * | O | ===X Addr Bus====>   | M |E_XDB    "      "   "
// * |   |                      |   |<========"      "   "
// * |   | ===Y Addr Bus====>   | S |E_YAB           "   "
// * | R |                      | W |================"   "
// * |   | <===P Data Bus===>   | T |E_YDB               "
// * |   |                      | I |<==================="
// * | E | <===X Data Bus===>   | C |                            ______________
// * |   |                      | H |===EAB===================> | External I/O |
// * |___| <===Y Data Bus===>   |___|<==EDB===================> |______________|
// *
// *****************************************************************************/

	/*
	 * parameters for this module
	 *
	 */
   parameter		    data_num_detect_bits = 6;
   parameter		    addr_num_detect_bits = 6;
   parameter		    abus_size = 16;
   parameter		    dbus_size = 24;
   parameter		    dmem_size = 512;
   parameter		    dmem_bits = 9;
   parameter		    pmem_size = 1024;
   parameter		    pmem_bits = 10;



	/*
	 *unit control signals
	 */
   input		    reset;
   input [1:0]		    MEM_SLCT;
   output		    ESTROBE;
   output		    EACCESS;
   input		    EENABLE;
   input		    Clkin;
   output 		    Clk_agu_0_rfm;
   output 		    Clk_agu_1_rfn;
   output 		    Clk_agu_2_rfr;
   output 		    Clk_agu_3_ctrl;
   output 		    Clk_agu_4_ctrl;
   output 		    Clk_agu_5_misc;
   output 		    Clk_agu_6_misc;
   output 		    Clk_alu_0_xy;
   output 		    Clk_alu_1_ab;
   output 		    Clk_alu_2_misc;
   output 		    Clk_bus_0;
   output 		    Clk_pcu_0;
   output 		    Clk_pcu_1;
   input 		    test;
   
	

	/*
	 * Core side bus definitions
	 */
	 
   input [(abus_size-1):0]  C_PAB;
   inout [(dbus_size-1):0]  C_PDB;

   input [(abus_size-1):0]  C_XAB;
   inout [(dbus_size-1):0]  C_XDB;
   input		    CX_READ;
   input		    CX_WRITE;

   input [(abus_size-1):0]  C_YAB;
   inout [(dbus_size-1):0]  C_YDB;
   input		    CY_READ;
   input		    CY_WRITE;

	/*
	 * RAM side bus definitions
	 */

   output [(abus_size-1):0] E_PAB;
   output [(dbus_size-1):0] E_PDB_OUT;
   input [(dbus_size-1):0]  E_PDB_IN;
   output		    EP_OUTEN;
   output		    EP_WRITE;

   output [(abus_size-1):0] E_XAB;
   output [(dbus_size-1):0] E_XDB_OUT;
   input [(dbus_size-1):0]  E_XDB_IN;
   output		    EX_OUTEN;
   output		    EX_WRITE;

   output [(abus_size-1):0] E_YAB;
   output [(dbus_size-1):0] E_YDB_OUT;
   input [(dbus_size-1):0]  E_YDB_IN;
   output		    EY_OUTEN;
   output		    EY_WRITE;


	/*
	 *External side bus definitions
	 */
   inout [(abus_size-1):0]  EAB;
   inout [(dbus_size-1):0]  EDB;
   inout		    E_OUTEN;
   inout		    E_WRITE;
	
	/*
	 *External side bus control signals for bidirectional pads
	 */
   output		    edb_dir;
   output		    edb_en;
   output		    eab_dir;
   output		    eab_en;
	

	/*
	 * Some wires and internal "registers"....
	 */
   wire [1:0]		    ltch_mem_slct;
   wire			    inv_cpustop;
	
	/*
	 * control signals for ram interface (thingy)
	 */	
   wire			    nC_to_xram;	// from core to memory
   wire			    nC_to_yram;
   wire			    nC_to_pram;
   wire			    nRS_to_xram;	// from pins to memory
   wire			    nRS_to_yram;
   wire			    nRS_to_pram;
   wire			    nxram_to_C;	// from memory to core
   wire			    nyram_to_C;
   wire			    npram_to_C;
   wire			    nxram_to_RS;
   wire			    nyram_to_RS;
   wire			    npram_to_RS;


   /*
    * create the clock stopper unit
    *
    */

   wire 		    cpu_stop;
 		    
   clock_stop clock_controller (Clkin,
		  Clk_agu_0_rfm, Clk_agu_1_rfn, Clk_agu_2_rfr,
		  Clk_agu_3_ctrl, Clk_agu_4_ctrl, Clk_agu_5_misc, Clk_agu_6_misc,
		  Clk_alu_0_xy, Clk_alu_1_ab, Clk_alu_2_misc,
                  Clk_bus_0, Clk_pcu_0, Clk_pcu_1, 
		  ESTROBE, EENABLE, cpu_stop, reset);
   
   
   /*
    *
    * instantiate latch for mem_selsect and invert cpu_stop
    *
    */
   misc_signals m_signal (ESTROBE, reset, MEM_SLCT, ltch_mem_slct, cpu_stop, 
			  inv_cpustop);
   
   
   /*
    *
    * instantiate the behavioral synthesized miscellaneous logic stuff
    *
    */
   assign EACCESS = cpu_stop;
   
   
   /*
    *
    * instantiate the address bus control unit
    * REPLACE WITH mux2
    *
    */
   addr_bus_control ab_control (cpu_stop, ltch_mem_slct, C_PAB, C_XAB, C_YAB, 
				EAB, E_PAB, E_XAB, E_YAB);
   
   
   /*
    *
    * external bus pad control signals
    *
    */
   ebus_drivers_control eb_control (eab_dir, eab_en, edb_dir, edb_en, 
				    cpu_stop,    // changed from EENABLE
				    E_WRITE, E_OUTEN);
   
   /*
    * external bus interface
    *
    */
   ebus ebus_interface (E_WRITE, ltch_mem_slct, inv_cpustop, test, EAB, EDB,
			C_XAB, C_XDB, C_YAB, C_YDB, C_PAB, C_PDB);
   
   
   /*
    *
    * instaniate the SRAM enable control logic
    *
    */
   ram_control_signals re_control (cpu_stop, ltch_mem_slct, ESTROBE,
                        CX_WRITE, CX_READ, CY_READ, CY_WRITE, E_OUTEN,
                        E_WRITE, EP_OUTEN, EP_WRITE, EX_OUTEN, EX_WRITE,
                        EY_OUTEN, EY_WRITE);
   
   /*
    * create the thingy control logic
    *
    */
   thingy_control thing_control_logic (.cpu_stop(cpu_stop), 
				       .MEM_SLCT(ltch_mem_slct), 
				       .CX_WRITE(CX_WRITE), 
				       .CY_WRITE(CY_WRITE),
				       .CX_READ(CX_READ), 
				       .CY_READ(CY_READ),
				       .E_WRITE(E_WRITE), 
				       .E_OUTEN(E_OUTEN), 
				       .nC_to_xram(nC_to_xram), 
				       .nC_to_yram(nC_to_yram), 
				       .nC_to_pram(nC_to_pram), 
				       .nRS_to_xram(nRS_to_xram), 
				       .nRS_to_yram(nRS_to_yram), 
				       .nRS_to_pram(nRS_to_pram), 
				       .nxram_to_C(nxram_to_C), 
				       .nyram_to_C(nyram_to_C), 
				       .npram_to_C(npram_to_C), 
				       .nxram_to_RS(nxram_to_RS), 
				       .nyram_to_RS(nyram_to_RS), 
				       .npram_to_RS(npram_to_RS), 
				       .test(test)
				       );
   
   
   /*
    *
    * create the thingy elements (SRAM controller blocks)
    * Tri-states that control which bus has access to SRAM
    * and splits bidirectional bus into unidirectional busses
    *
    */
   thingy #(dbus_size) pbus_thing (C_PDB, EDB, E_PDB_IN, E_PDB_OUT, 
				   nC_to_pram, nRS_to_pram, 
				   npram_to_C, npram_to_RS );
   thingy #(dbus_size) xbus_thing (C_XDB, EDB, E_XDB_IN, E_XDB_OUT, 
				   nC_to_xram, nRS_to_xram, nxram_to_C, 
				   nxram_to_RS );
   thingy #(dbus_size) ybus_thing (C_YDB, EDB, E_YDB_IN, E_YDB_OUT, 
				   nC_to_yram, nRS_to_yram, nyram_to_C, 
				   nyram_to_RS );


endmodule



/*****************************************************************************
 *****************************************************************************/
