/* $Id: thingy_control.v,v 1.4 1997/10/13 21:44:42 inacio Exp $ */
/*****************************************************************************
 *thingy_control.v
 *
 *logic to control the bus thingy's.  Mux/tribuf/demultiplexers...
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: inacio $
 *Date: $Date: 1997/10/13 21:44:42 $
 *Revision: $Revision: 1.4 $
 *
 *
 *Log: $Log: thingy_control.v,v $
 *Log: Revision 1.4  1997/10/13 21:44:42  inacio
 *Log: updated the sensitivity list
 *Log:
# Revision 1.2  1997/10/10  20:32:19  inacio
# control logic for the thingies, now relies on the clock_stop signal instead
# of EENABLE
#
# Revision 1.1  1997/10/08  23:40:23  inacio
# The control logic for the thingy module.  Eventually the mem_slct lines will
# have to be latched and clocked, making them obsolete in here, but I javen't
# done that yet...
#
 *****************************************************************************/

module thingy_control (	cpu_stop, 
								c_xab_ov,
								MEM_SLCT,
								CX_WRITE,
								CY_WRITE,
								E_WRITE,
							  	p_bus_thing_en,
							  	x_bus_thing_en,
							  	y_bus_thing_en,
							  	p_bus_thing_dir,
							  	x_bus_thing_dir,
							  	y_bus_thing_dir,
							  	p_bus_thing_bussel,
							  	x_bus_thing_bussel,
							  	y_bus_thing_bussel
							  );


input		cpu_stop;					// cpu is stopped, external bus is enabled
input		MEM_SLCT;					// selects which mem bank to access
											// from external bus
input		CX_WRITE;					// memory write control signals (C's from
input		CY_WRITE;					// core, E from external)
input		E_WRITE;
input		c_xab_ov;					// asserted when address is out of SRAM range
output	p_bus_thing_en;			// enable signals for SRAM controler things
output	x_bus_thing_en;
output	y_bus_thing_en;
output	p_bus_thing_dir;			// direction signals, (write/read) for SRAM
output	x_bus_thing_dir;
output	y_bus_thing_dir;
output	p_bus_thing_bussel;		// selects which bus core or external to use
output	x_bus_thing_bussel;		// for the SRAM bus thingies
output	y_bus_thing_bussel;



/*
 * some definitions to make Synopsys happy...
 */
reg	p_bus_thing_en;	
reg	x_bus_thing_en;
reg	y_bus_thing_en;
reg	p_bus_thing_dir;	
reg	x_bus_thing_dir;
reg	y_bus_thing_dir;
reg	p_bus_thing_bussel;
reg	x_bus_thing_bussel;
reg	y_bus_thing_bussel;





always @(cpu_stop or MEM_SLCT or E_WRITE or c_xab_ov or CX_WRITE or CY_WRITE)
begin


	/*
	 *
	 * thingy enable control signals....
	 *
	 */
	if (cpu_stop==0)
		begin
		/*
		 * cpu active
		 */
		p_bus_thing_en <= 1'b0;
		y_bus_thing_en <= 1'b0;
		/*
		 *make sure the address is internal to enable cpu-to-SRAM
		 *otherwise disable and route to the external bus
		 *
		 */
		if (c_xab_ov == 1'b1)
			x_bus_thing_en <= 1'b0;
		else
			x_bus_thing_en <= 1'b1;
		end
	else
		case (MEM_SLCT)
			2'b00 :
				begin
      		p_bus_thing_en <= 1'b1;
      		x_bus_thing_en <= 1'b0;
      		y_bus_thing_en <= 1'b1;
				end
			2'b01 :
				begin
      		p_bus_thing_en <= 1'b1;
      		x_bus_thing_en <= 1'b1;
      		y_bus_thing_en <= 1'b0;
				end
			2'b10 :
				begin
      		p_bus_thing_en <= 1'b1;
      		x_bus_thing_en <= 1'b1;
      		y_bus_thing_en <= 1'b1;
				end
			2'b11	:
				begin
      		p_bus_thing_en <= 1'b0;
      		x_bus_thing_en <= 1'b1;
      		y_bus_thing_en <= 1'b1;
				end
		endcase


	/*
	 *
	 * thingy direction control signals
	 *
	 */
	if (cpu_stop==0)
		begin
			p_bus_thing_dir <= 1'b0;
			x_bus_thing_dir <= CX_WRITE;
			y_bus_thing_dir <= CY_WRITE;
		end
	else
		case (MEM_SLCT)
			2'b00:
				begin
					p_bus_thing_dir <= 1'bx;
					x_bus_thing_dir <= ~E_WRITE;
					y_bus_thing_dir <= 1'bx;
				end
			2'b01:
				begin
					p_bus_thing_dir <= 1'bx;
					x_bus_thing_dir <= 1'bx;
					y_bus_thing_dir <= ~E_WRITE;
				end
			2'b10:
				begin
					p_bus_thing_dir <= 1'bx;
					x_bus_thing_dir <= 1'bx;
					y_bus_thing_dir <= 1'bx;
				end
			2'b11:
				begin
					p_bus_thing_dir <= ~E_WRITE;
					x_bus_thing_dir <= 1'bx;
					y_bus_thing_dir <= 1'bx;
				end
		endcase
			

	/*
	 *
	 * thingy bussel control signals
	 *
	 */
	if (cpu_stop==0)
		begin
			p_bus_thing_bussel <= 1'b0;
			x_bus_thing_bussel <= 1'b0;
			y_bus_thing_bussel <= 1'b0;
		end
	else
		begin
			p_bus_thing_bussel <= 1'b1;
			x_bus_thing_bussel <= 1'b1;
			y_bus_thing_bussel <= 1'b1;
		end

end /*always block*/

endmodule

