/*	File:	cr_test.v	  					*/

/*	module name: top						*/

/*	Description: TEST FILE						*/

/*	Tested Module name:	core_ram.v				*/


/*=======================================================================*/

/*************************************************************************/
/*									*/
/*	module:	top							*/
/*									*/
/*************************************************************************/

module top;



/*============		internal nets	=================================*/

reg reset;

reg Clk;

integer dumpfile, dumpfile_and_stdout;


/*=========================================================================*/



/*===========	Module Instantiations	 ===================================*/

core_ram_s coreR(
	.reset (reset),
	.Clk (Clk)
	);

/*========================================================================*/


/*======================================================================*/
/*	initialize							*/
/*======================================================================*/

initial
	begin
	
		/* generates a dump file for Cwaves to use */
//		$shm_open("wavesCR.shm");		
		/* keep track of all signals in the hierarcy */
//		$shm_probe("AS");

		/* open text file for dumping results */
		dumpfile = $fopen("verilogSim.dump");
		/* also send results to standard output (Palnitkar p. 180) */
		dumpfile_and_stdout = $fopen("verilogSim.dump") | 1;
	
		
		/* load data into RAM modules 		*/
		/* start at RAMData[00]             */
		$readmemh( "pmem_file", coreR.PRAM.RAMData, 8'h00);
		$readmemh( "xmem_file", coreR.XRAM.RAMData, 12'h00);
		$readmemh( "ymem_file", coreR.YRAM.RAMData, 12'h00);
		

		/* Start simulation with reset signal high */
		Clk = 0;
		reset = 1;	
	
		/* end of reset cycle */
		#100 reset = 0;
		

		#4000 $finish;		/* end simulation */
		
		$fclose(dumpfile);		

//		$shm_close;		 /* close the dump file */
	end



/*======================================================================*/
/*	clock	50 units High, 50 units Low				*/
/* 		on each cycle, dump register contents			*/
/*======================================================================*/

always
	begin
		#50 Clk = 1;
		#50 Clk = 0;

		/* dump contents of architected registers */
		/* to send only to a file, use dumpfile instead of */
		/* dumpfile_and_stdout */
		$fdisplay(dumpfile_and_stdout,
			"\nCore signals: \n\t PDB = %h \t XDB = %h \t YDB = %h \n",
			 coreR.PDB, coreR.XDB, coreR.YDB, 

			"PCU Registers: \n",
			"\t PC = %h \n",
			coreR.CORE.PCU.PC, 	

			"ALU Registers: \n",
			"\t x = %h  \ty = %h\n",
			 {coreR.CORE.ALU.x1, coreR.CORE.ALU.x0},
			 {coreR.CORE.ALU.y1, coreR.CORE.ALU.y0},
			"\t A = %h \t B = %h \n",
			 {coreR.CORE.ALU.a2, coreR.CORE.ALU.a1, coreR.CORE.ALU.a0},
			 {coreR.CORE.ALU.b2, coreR.CORE.ALU.b1, coreR.CORE.ALU.b0},

			"AGU Registers; \n",
			"\t r7 = %h \t n7 = %h \t m7 = %h \n",
			coreR.CORE.agu.Rout3_H, coreR.CORE.agu.Nout3_H, coreR.CORE.agu.Mout3_H, 
			"\t r6 = %h \t n6 = %h \t m6 = %h \n",
			coreR.CORE.agu.Rout2_H, coreR.CORE.agu.Nout2_H, coreR.CORE.agu.Mout2_H, 
			"\t r5 = %h \t n5 = %h \t m5 = %h \n",
			coreR.CORE.agu.Rout1_H, coreR.CORE.agu.Nout1_H, coreR.CORE.agu.Mout1_H, 
/* can't fine these signals
			"\t r4 = %h \t n4 = %h \t m4 = %h \n",
*/
			"\t r3 = %h \t n3 = %h \t m3 = %h \n",
			coreR.CORE.agu.Rout3_L, coreR.CORE.agu.Nout3_L, coreR.CORE.agu.Mout3_L, 
			"\t r2 = %h \t n2 = %h \t m2 = %h \n",
			coreR.CORE.agu.Rout2_L, coreR.CORE.agu.Nout2_L, coreR.CORE.agu.Mout2_L, 
			"\t r1 = %h \t n1 = %h \t m1 = %h \n",
			coreR.CORE.agu.Rout1_L, coreR.CORE.agu.Nout1_L, coreR.CORE.agu.Mout1_L, 
/* can't find these signals
			"\t r3 = %h \t n3 = %h \t m3 = %h \n",
*/
			"next",$time,
			"\n\n"
			);

	end
	
endmodule	/* end module top */
