/* $Id: whole.v,v 1.14 1997/12/15 18:02:33 benk Exp $ */
/*****************************************************************************
 *
 *
 *
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: benk $
 *Date: $Date: 1997/12/15 18:02:33 $
 *Revision: $Revision: 1.14 $
 *
 *
 *Log: $Log: whole.v,v $
 *Log: Revision 1.14  1997/12/15 18:02:33  benk
 *Log: Changed logic to enable ead and edb in test bench
 *Log:
# Revision 1.11  1997/12/09  22:07:29  inacio
# got rid of extra precision warnings
#
# Revision 1.10  1997/12/09  21:31:36  inacio
# added synch stuff at the start of mem loading
#
# Revision 1.9  1997/12/09  21:29:07  inacio
# fixed zeroing out the ram
#
# Revision 1.8  1997/12/09  05:32:08  inacio
# removed some display messages, wasn't using them anymore
#
# Revision 1.7  1997/12/09  00:45:24  inacio
# added lots more stuff, like mem dumping to the file, plus the ability to
# easily change the number of cycles for program execution, etc.
#
# Revision 1.6  1997/12/07  04:39:55  inacio
# added some code to initialize the fake memories to 0x0 to be able to make
# sure the writes are working correctly
#
# Revision 1.5  1997/12/05  22:02:42  inacio
# corrected some the parameter typos
#
# Revision 1.4  1997/12/05  19:06:58  inacio
# cleaned up timing to load the chip from the external world, added the new
# extra power and gnd pins to the package
#
# Revision 1.3  1997/11/18  22:48:03  inacio
# fixed when I read the data from the fake memories
#
# Revision 1.2  1997/11/18  22:25:50  inacio
# Added better timing to the reset line; I turn off the write enable line more
# often now.
#
# Revision 1.1  1997/11/18  18:08:01  inacio
# test file for the whole chip behavioral model
#
 *****************************************************************************/


`timescale 1ns / 1ps
`define CLOCK_CYCLE 50

/*
 *
 * set PROG_CYCLES to how many cycles you want to simulate for
 *
 *
 * FIR - 2062
 * FFT - 7200
 * LMS - 51990
 */
`define PROG_CYCLES 25


module thing;


/*
 *fake memories to load up the real Cascade memories
 */
   reg [23:0]	temp_P_ram[1023:0];
   reg [23:0]	temp_X_ram[1023:0];
   reg [23:0]	temp_Y_ram[1023:0];

/*
 * variables to do the writing and bookkeeping
 */
   reg [32:0]	clock_count;
   reg [15:0]	loop_count;
   reg [23:0]	dummy_word;
   reg [15:0]	dummy_addr;
   reg [16:0]	dummy_data1;
   reg [23:17]	dummy_data2;
   reg		dummy_ram_enable;
   reg		dummy_ram_write;
   reg [2:0]	vil;
   integer	mem_dump;


/*
 * Connections to hook up to the chip, uncomment the reg version if you need
 * to make an assignment to it.
 */
   reg		Clk;
   reg		reset;
   reg		e_enable;
   reg [1:0]	mem_select;
   reg		e_outen;
   reg		e_write;
   reg		test;
/*
reg	[15:0]eab;
reg	[16:0]edb1;
reg	[23:17]edb2;
reg	scanin1;
reg	scanin2;
reg	scanout1;
reg	scanout2;
reg	estrobe;
reg	e_access;
*/



/*
 * Connections to hook up to the chip, uncomment the reg version if you need
 * to make an assignment to it.  Comment out the wire version if you need to
 * make an assignment to it
 */
/*
wire	Clk;
wire 	reset;
wire	e_enable;
wire	[1:0]mem_select;
wire	e_outen;
wire	e_write;
wire	test;
*/
   wire [15:0]	eab;
   wire [16:0]	edb1;
   wire [23:17]	edb2;
   wire [15:0]	eab_in;
   wire [16:0]	edb1_in;
   wire [23:17]	edb2_in;
   wire		scanin1;
   wire		scanin2;
   wire		scanout1;
   wire		scanout2;
   wire		estrobe;
   wire		e_access;

   reg		edb_input, eab_input;   // control for "input" to edb, eab bus
   



/*****************************************************************************
 *instance the chip
 *
 *****************************************************************************/
pads fullchip (Clk, reset, mem_select, estrobe, e_access, e_enable, eab,
	       edb1, edb2, e_outen, e_write, test, scanin1, scanin2, scanout1,
	       scanout2, 1'b1, 1'b1, 1'b1, 1'b1, 1'b1, 1'b1, 1'b1, 1'b0, 1'b0,
	       1'b0, 1'b0, 1'b0, 1'b0, 1'b0, 1'b1, 1'b1, 1'b1, 1'b1, 1'b1,
	       1'b1, 1'b1, 1'b0, 1'b0, 1'b0, 1'b0, 1'b0, 1'b0, 1'b0);
            

/*****************************************************************************
 *build the assign statements to drive the busses
 *
 *****************************************************************************/
assign
	eab = (eab_input) ? dummy_addr : 16'bz,
	edb1 = (edb_input) ? dummy_data1 : 17'bz,
	edb2 = (edb_input) ? dummy_data2 : 7'bz,
   	eab_in = (eab_input) ? dummy_addr : 16'bz,
	edb1_in = (edb_input) ? dummy_data1 : 17'bz,
	edb2_in = (edb_input) ? dummy_data2 : 7'bz;
//	e_write = (e_enable) ? dummy_ram_write : 1'bz,
//	e_outen = (e_enable) ? dummy_ram_enable : 1'bz;
	

/*****************************************************************************
 *reset the chip and get it ready to load the memories
 *
 *****************************************************************************/
initial
   begin


      /*
       *
       * Probe the chip and build wave forms to view
       *
       */
      $shm_open("wavesCR.shm");
      $shm_probe("AS");
      
      /*
       * reset the clock counter
       */
      clock_count <= 0;
      
      /*
       * clear the fake memories
       */
      for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 1)
	 begin
	    temp_P_ram[loop_count] = 24'b0;
	    temp_X_ram[loop_count] = 24'b0;
	    temp_Y_ram[loop_count] = 24'b0;
	 end
      
      
      /*
       * read from files into the fake memories
       */
      $readmemh("pmem_file", temp_P_ram, 16'h0000);
      $readmemh("xmem_file", temp_X_ram, 16'h0000);
      $readmemh("ymem_file", temp_Y_ram, 16'h0000);
      
      

      /*
       * signals that I want to have already set when I get ready
       * to run after reset....  Initializes the E bus to load the internal
       * RAMs
       */
      e_enable <= 1;
      mem_select <= 2'b11;
      dummy_addr <= 16'b0;
      dummy_data1 <= 17'b0;
      dummy_data2 <= 7'b0;
      e_write <= 1'b1;
      e_outen <= 1'b1;
      test <= 1'b0;
      vil = 1'b10;
      dummy_word = 24'hffffff;
      edb_input = 1;
      eab_input = 1;
      
      /*
       * reset the chip
       */		
      reset <= 1;
      @(posedge estrobe);
      @(posedge estrobe);
      @(posedge estrobe);
      @(posedge estrobe);
      reset <= 0;


      /*
       * write the values into the memory
       *
       */
      $display ("Loading the memory . . .\n");
      wait (reset == 1'b0);
      wait (e_access == 1'b1);
      @(posedge estrobe);
      for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 1)
	 begin
	    // $display ("Loop count is: %x\n",loop_count);
	    dummy_addr = loop_count;
	    for (vil = 0; vil < 3'b100; vil = vil + 2'b01)
	       begin
		  // $display ("vil is %x\n",vil);
		  mem_select = vil;
		  case (vil)
		    2'b00 : 
		       begin
			  dummy_word = temp_X_ram[loop_count];
			  dummy_data1 = dummy_word[16:0];
			  dummy_data2 = dummy_word[23:17];
			  #(`CLOCK_CYCLE/4 + 1) e_write = 1'b0;
			  #(`CLOCK_CYCLE - (`CLOCK_CYCLE/4) - 1) e_write <= 1'b1;
		       end
		    2'b01 : 
		       begin
			  dummy_word = temp_Y_ram[loop_count];
			  dummy_data1 = dummy_word[16:0];
			  dummy_data2 = dummy_word[23:17];
			  #(`CLOCK_CYCLE/4 + 1) e_write = 1'b0;
			  #(`CLOCK_CYCLE - (`CLOCK_CYCLE/4) - 1) e_write <= 1'b1;
		       end
		    2'b11 : 
		       begin
			  dummy_word = temp_P_ram[loop_count];
			  dummy_data1 = dummy_word[16:0];
			  dummy_data2 = dummy_word[23:17];
			  #(`CLOCK_CYCLE/4 + 1) e_write = 1'b0;
			  #(`CLOCK_CYCLE - (`CLOCK_CYCLE/4) - 1) e_write <= 1'b1;
		       end
		    default : 
		       begin
			  dummy_word = 256'b0;
			  e_write = 1'b1;
		       end
		  endcase
		  @(posedge estrobe);
	       end
	    @(posedge estrobe);
	 end

      
      /*
       * Leave test mode
       */
      @(posedge estrobe);
      e_write <= 1'b1;
      e_enable <= 0;
      edb_input = 0;
      eab_input = 0;
      wait (e_access == 1'b0);


      /*
       * reset the chip
       */		
      reset <= 1;
      @(posedge estrobe);
      @(posedge estrobe);
      @(posedge estrobe);
      @(posedge estrobe);
      reset <= 0;


      /*
       *
       * Simulate the program
       *
       */
      $display ("Running the program . . . \n");
      #(`CLOCK_CYCLE * `PROG_CYCLES);
      

		

		
      /*
       *
       * read out the physical memories and dump them to a file
       *
       */
      $display ("Dumping the memory . . .\n");
      e_enable <= 1'b1;
      e_write <= 1'b1;
      e_outen <= 1'b0;
      edb_input = 0;
      eab_input = 1;
      wait (e_access == 1'b1);
      for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 1)
         begin
            dummy_addr = loop_count;
            for (vil = 0; vil < 3'b100; vil = vil + 1)
               begin
                  case (vil)
                     2'b00:
                        begin
                           mem_select = vil;
                           #(`CLOCK_CYCLE - (`CLOCK_CYCLE/5)) dummy_word[16:0] = edb1;
                           dummy_word[23:17] = edb2;
                           temp_X_ram[loop_count] = dummy_word;
                        end
                     2'b01:
                        begin
                           mem_select = vil;
                           #(`CLOCK_CYCLE - (`CLOCK_CYCLE/5)) dummy_word[16:0] = edb1;
                           dummy_word[23:17] = edb2;
                           temp_Y_ram[loop_count] = dummy_word;
                        end
                     2'b11:
                        begin
                           mem_select = vil;
                           #(`CLOCK_CYCLE - (`CLOCK_CYCLE/5)) dummy_word[16:0] = edb1;
                           dummy_word[23:17] = edb2;
                           temp_P_ram[loop_count] = dummy_word;
                        end
                     default:
                        begin
//                           e_outen <= 1'b1;
//                           #(`CLOCK_CYCLE - (`CLOCK_CYCLE/5)) e_outen <= 1'b0;
                        end
                  endcase
                  @(posedge estrobe);
               end
         end


         
      /*
       *
       * Dump the memories to a file
       *
       */
      mem_dump = $fopen ("verilog_xmem_file");
      for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 4)
         begin
            $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",loop_count,temp_X_ram[loop_count],temp_X_ram[loop_count+1],temp_X_ram[loop_count+2],temp_X_ram[loop_count+3]);
         end
      $fclose(mem_dump);

      mem_dump = $fopen ("verilog_ymem_file");
      for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 4)
         begin
            $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",loop_count,temp_Y_ram[loop_count],temp_Y_ram[loop_count+1],temp_Y_ram[loop_count+2],temp_Y_ram[loop_count+3]);
         end
      $fclose(mem_dump);

      mem_dump = $fopen ("verilog_pmem_file");
      for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 4)
         begin
            $fdisplay (mem_dump, "p:$%x=\t$%x\t$%x\t$%x\t$%x\n",loop_count,temp_P_ram[loop_count],temp_P_ram[loop_count+1],temp_P_ram[loop_count+2],temp_P_ram[loop_count+3]);
         end
      $fclose(mem_dump);


      $shm_close;     /* close the dump file */
      
      $finish;


	end


/*****************************************************************************
 *build the clock signal
 *
 *****************************************************************************/
always
	begin
		#(`CLOCK_CYCLE/2) Clk <= 1;
		#(`CLOCK_CYCLE/2) Clk <= 0;
		clock_count = clock_count + 1;
		// $display ("Clock is now: %x\n",clock_count);
	end
			


endmodule	



/*****************************************************************************
 *****************************************************************************/
