/* $Id: whole_chip_cascade.v,v 1.11 1998/01/15 18:58:55 inacio Exp $ */
/*****************************************************************************
 *whole_chip_cascade.v
 *
 *stimulus file to test the entire chip design
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: inacio $
 *Date: $Date: 1998/01/15 18:58:55 $
 *Revision: $Revision: 1.11 $
 *
 *
 *Log: $Log: whole_chip_cascade.v,v $
 *Log: Revision 1.11  1998/01/15 18:58:55  inacio
 *Log: added setting the test signal to the chip, should solve some problems with
 *Log: unknown values in the simulations
 *Log:
# Revision 1.10  1997/12/10  20:55:25  benk
# Fixed bugs in adm.vec file that is output
#
# Revision 1.8  1997/12/09  05:31:02  inacio
# the memory writes now completely work, the memory reads I'm not compeltely
# sure about.  Should be good for generating test vectors
#
# Revision 1.7  1997/12/05  19:07:42  inacio
# added the new pwr and gnd pins, cleaned up loading the external interface
# and made everything work based on the defined CLOCK_CYCLE
#
# Revision 1.6  1997/11/18  17:56:57  inacio
# changed the estrobe code so that it works.  Changed some places where I was
# still using estrobe and not dummy_estrobe.
#
# Revision 1.5  1997/11/06  23:07:58  inacio
# got rid of the commented out version of the RAM loading code
#
# Revision 1.4  1997/11/06  23:06:59  inacio
# this version actually works, if the estrobe signal is clean enough!
#
# Revision 1.3  1997/11/05  22:32:12  inacio
# Many many changes to load the memories and simulate the whole design.
#
# Revision 1.2  1997/11/04  07:00:57  inacio
# yadda yadda yadda
#
# Revision 1.1  1997/11/04  06:44:30  inacio
# file to test the entire chip, stupid tests for chris
#
 *****************************************************************************/
              
`define CLOCK_CYCLE 50

/*
 *
 * set PROG_CYCLES to the number of cycles to execute the program for
 *
 * FIR - 2062
 * FFT - 7200
 * LMS - 51990
 */
`define PROG_CYCLES 25

module thing;


/*
 *fake memories to load up the real Cascade memories
 */
   reg [23:0]	temp_P_ram[1023:0];
   reg [23:0]	temp_X_ram[1023:0];
   reg [23:0]	temp_Y_ram[1023:0];


/*
 * variables to do the writing and bookkeeping
 */
   reg [32:0]	clock_count;
   reg [15:0]	loop_count;
   reg [23:0]	dummy_word;
   reg [15:0]	dummy_addr;
   reg [16:0]	dummy_data1;
   reg [23:17]	dummy_data2;
   reg		dummy_ram_enable;
   reg		dummy_ram_write;
   reg [2:0]	vil;
   reg		old_dummy_estrobe;
   reg		dummy_estrobe;
   integer	mem_dump;
   integer	dumpfile;
   

/*
 * Connections to hook up to the chip, uncomment the reg version if you need
 * to make an assignment to it.
 */
   reg		Clk;
   reg		reset;
   reg		e_enable;
   reg [1:0]	mem_select;
   reg		e_outen;
   reg		e_write;
   reg	test;
/*
   reg	[15:0]eab;
   reg	[16:0]edb1;
   reg	[23:17]edb2;
   reg	scanin1;
   reg	scanin2;
   reg	scanout1;
   reg	scanout2;
   reg	estrobe;
   reg	e_access;
*/



/*
 * Connections to hook up to the chip, uncomment the reg version if you need
 * to make an assignment to it.  Comment out the wire version if you need to
 * make an assignment to it
 */
/*
   wire	Clk;
   wire 	reset;
   wire	e_enable;
   wire	[1:0]mem_select;
   wire	e_outen;
   wire	e_write;
   wire		test;
*/
   wire [15:0]	eab;
   wire [16:0]	edb1;
   wire [23:17]	edb2;
   wire [15:0]	eab_in;           // input only for vec file input
   wire [16:0]	edb1_in;          // when pin is output, xxx_in = Z
   wire [23:17]	edb2_in;
   wire		scanin1;
   wire		scanin2;
   wire		scanout1;
   wire		scanout2;
   wire		estrobe;
   wire		e_access;
   wire		pwr1, pwr2, pwr3, pwr4, pwr5, pwr6, pwr7;
   wire		gnd1, gnd2, gnd3, gnd4, gnd5, gnd6, gnd7;
   wire		pwr8, pwr9, pwr10, pwr11, pwr12, pwr13, pwr14;
   wire		gnd8, gnd9, gnd10, gnd11, gnd12, gnd13, gnd14;

/*****************************************************************************
 *instance the chip
 *
 *****************************************************************************/
pads fullchip (Clk, reset, mem_select, estrobe, e_access, e_enable, eab,
    			edb1, edb2, e_outen, e_write, test, scanin1, scanin2, scanout1,
    			scanout2, pwr1, pwr2, pwr3, pwr4, pwr5, pwr6, pwr7, gnd1, gnd2,
    			gnd3, gnd4, gnd5, gnd6, gnd7, pwr8, pwr9, pwr10, pwr11, pwr12,
    			pwr13, pwr14, gnd8, gnd9, gnd10, gnd11, gnd12, gnd13, gnd14);
            

/*****************************************************************************
 *build the assign statements to drive the busses
 *
 *****************************************************************************/
assign
	eab = (e_enable) ? dummy_addr : 16'bz,
	edb1 = (e_enable) ? dummy_data1 : 17'bz,
	edb2 = (e_enable) ? dummy_data2 : 7'bz,
	eab_in = (e_enable) ? dummy_addr : 16'bz,
	edb1_in = (e_enable) ? dummy_data1 : 17'bz,
	edb2_in = (e_enable) ? dummy_data2 : 7'bz;
//	e_write = (e_enable) ? dummy_ram_write : 1'bz,
//	e_outen = (e_enable) ? dummy_ram_enable : 1'bz;
	

/*****************************************************************************
 *
 *clean up the estrobe signal
 *
 *****************************************************************************/
always
   begin
      @(estrobe)
         begin
            case (estrobe)
               1'b1:
                  begin
                     dummy_estrobe <= 1'b1;
                     old_dummy_estrobe <= 1'b1;
                  end
               1'b0:
                  begin
                     dummy_estrobe <= 1'b0;
                     old_dummy_estrobe <= 1'b0;
                  end
               1'bz:
                  begin
                     dummy_estrobe <= old_dummy_estrobe;
                  end
               1'bx:
                  begin
                     dummy_estrobe <= old_dummy_estrobe;
                  end
            endcase
         end
   end
   


/*****************************************************************************
 *reset the chip and get it ready to load the memories
 *
 *****************************************************************************/
initial
   begin
      
      /*
       *
       * Initialize memory to be all zeros to make sure things work and I
       * shouldn't get an X's that way
       *
       */
      for (loop_count=0; loop_count < 1024; loop_count = loop_count + 1)
	 begin
	    temp_P_ram[loop_count] = 0;
	    temp_X_ram[loop_count] = 0;
	    temp_Y_ram[loop_count] = 0;
	 end


      
      /*
       *
       * Probe the chip and build wave forms to view
       *
       */
      $shm_open("wavesCR.shm");
      $shm_probe("AS");
      
      /*
       * Set up dump file to hold input vectors for ADM
       */
      dumpfile = $fopen("adm.vec");
      /* Vector pattern definition section for ADM vector file */
      $fdisplay(dumpfile,
		"radix 1 1 11 1 1 1 4444 11111111111111111 1111111 1 1 1 1 \n",
		"+1 1 1");
      $fdisplay(dumpfile,
		"io    i i ii o o i bbbb bbbbbbbbbbbbbbbbb bbbbbbb i i i i \n",
		"+i o o");
      $fdisplay(dumpfile,
		"vname CLOCK_PAD RESET_PAD MEM_SLCT_PAD MEM_SLCT_PN5 \n",
		"+ESTROBE_PAD EACCESS_PAD EENABLE_PAD EAB_PAD[15:0] \n",
		"+EDB_PAD1[16:0] EDB_PAD2[23:17] E_OUTEN_PAD E_WRITE_PAD \n",
		"+TEST_PAD SCAN_IN1_PAD SCAN_IN2_PAD SCAN_OUT1_PA SCAN_OUT2_PA");
      $fdisplay(dumpfile,
		"enable ~EENABLE_PAD 0 0 00 0 0 0 FFFF \n",
		"+11111111111111111 1111111 0 0 0 0 0 0 0");
      $fdisplay(dumpfile,
		"tunit ns");
      $fdisplay(dumpfile,
		"tdelay 3 0 1 11 0 0 1 0000 00000000000000000 0000000 \n",
		"+1 1 1 1 1 0 0  ; delay data inputs by 3 ns");
      $fdisplay(dumpfile,
		"idelay 3 0 0 00 0 0 0 FFFF 11111111111111111 1111111 \n",
		"+0 0 0 0 0 0 0  ; delay data inputs by 3 ns");
      $fdisplay(dumpfile,
		"slope 0.2");
      $fdisplay(dumpfile,
		"vih 3.3");
      $fdisplay(dumpfile,
		"vil 0");
      $fdisplay(dumpfile,
		"vth 1.6  ;Threshold voltage is a guess");
      $fdisplay(dumpfile,
		"; Start vectors");

      /*
       * reset the clock counter
       */
      clock_count = 0;
      
      
      /*
       * read from files into the fake memories
       */
      $readmemh("pmem_file", temp_P_ram, 16'h0000);
      $readmemh("xmem_file", temp_X_ram, 16'h0000);
      $readmemh("ymem_file", temp_Y_ram, 16'h0000);
      
      

      /*
       * signals that I want to have already set when I get ready
       * to run after reset....  Initializes the E bus to load the internal
       * RAMs
       */
      dummy_estrobe <= 0;
      old_dummy_estrobe <= 0;
      e_enable <= 1;
      mem_select <= 2'b11;
      dummy_addr <= 16'b0;
      dummy_data1 <= 17'b0;
      dummy_data2 <= 7'b0;
      e_write <= 1'b1;
      e_outen <= 1'b1;
      test <= 1'b0;


      /*
       * reset the chip
       */		
      reset <= 1;
      #100 reset <= 0;


      /*
       * write the values into the memory
       *
       */
      $display ("Loading the memory (clock is %d) . . .\n",clock_count);
      wait (reset == 1'b0);
      wait (e_access == 1'b1);
      @(posedge dummy_estrobe);
      for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 1)
	 begin
	    // $display ("Loop count is: %x\n",loop_count);
	    dummy_addr = loop_count;
	    for (vil = 0; vil < 3'b100; vil = vil + 3'b001)
	       begin
		  // $display ("vil is %x\n",vil);
		  mem_select = vil;
		  case (vil)
		    3'b000 : 
		       begin
			  dummy_word = temp_X_ram[loop_count];
			  dummy_data1 = dummy_word[16:0];
			  dummy_data2 = dummy_word[23:17];
			  #(`CLOCK_CYCLE/4 + 1) e_write = 1'b0;
			  // Inputs to vec file
			  $fstrobe(dumpfile,
				   $time, " %b %b %b x x %b %h %b %b %b %b %b %b %b x x ",
				   Clk, reset, mem_select, e_enable, 
				   eab_in, edb1_in, edb2_in, e_outen, e_write, test, 
				   scanin1, scanin2 );
			  #(`CLOCK_CYCLE - (`CLOCK_CYCLE/4) - 1) e_write = 1'b1;
			  // Inputs to vec file
			  $fstrobe(dumpfile,
				   $time, " %b %b %b x x %b %h %b %b %b %b %b %b %b x x ",
				   Clk, reset, mem_select, e_enable, 
				   eab_in, edb1_in, edb2_in, e_outen, e_write, test, 
				   scanin1, scanin2 );
		       end
		    3'b001 : 
		       begin
			  dummy_word = temp_Y_ram[loop_count];
			  dummy_data1 = dummy_word[16:0];
			  dummy_data2 = dummy_word[23:17];
			  #(`CLOCK_CYCLE/4 + 1) e_write = 1'b0;
			  // Inputs to vec file
			  $fstrobe(dumpfile,
				   $time, " %b %b %b x x %b %h %b %b %b %b %b %b %b x x ",
				   Clk, reset, mem_select, e_enable, 
				   eab_in, edb1_in, edb2_in, e_outen, e_write, test, 
				   scanin1, scanin2 );
			  #(`CLOCK_CYCLE - (`CLOCK_CYCLE/4) - 1) e_write = 1'b1;
			  // Inputs to vec file
			  $fstrobe(dumpfile,
				   $time, " %b %b %b x x %b %h %b %b %b %b %b %b %b x x ",
				   Clk, reset, mem_select, e_enable, 
				   eab_in, edb1_in, edb2_in, e_outen, e_write, test, 
				   scanin1, scanin2 );
		       end
		    3'b010 : 
		       begin
			  dummy_word = 256'b0;
			  e_write = 1'b1;
		       end
		    3'b011 : 
		       begin
			  dummy_word = temp_P_ram[loop_count];
			  dummy_data1 = dummy_word[16:0];
			  dummy_data2 = dummy_word[23:17];
			  #(`CLOCK_CYCLE/4 + 1) e_write = 1'b0;
			  // Inputs to vec file
			  $fstrobe(dumpfile,
				   $time, " %b %b %b x x %b %h %b %b %b %b %b %b %b x x ",
				   Clk, reset, mem_select, e_enable, 
				   eab_in, edb1_in, edb2_in, e_outen, e_write, test, 
				   scanin1, scanin2 );
			  #(`CLOCK_CYCLE - (`CLOCK_CYCLE/4) - 1) e_write = 1'b1;
			  // Inputs to vec file
			  $fstrobe(dumpfile,
				   $time, " %b %b %b x x %b %h %b %b %b %b %b %b %b x x ",
				   Clk, reset, mem_select, e_enable, 
				   eab_in, edb1_in, edb2_in, e_outen, e_write, test, 
				   scanin1, scanin2 );
		       end
		  endcase
	       end
	    @(posedge dummy_estrobe);
	 end
      
      
      /*
       * Leave test mode
       */
      @(posedge dummy_estrobe);
      e_write <= 1'b1;
      e_enable <= 0;
      wait (e_access == 1'b0);


      /*
       * simulate the program
       */
      $display ("Running the program (clock is %d) . . . \n",clock_count);
		#(`CLOCK_CYCLE * `PROG_CYCLES);


      /*
       * get the values from memory and write them to a file
       *
       */
      $display ("Dumping the memory (clock is %d) . . . \n",clock_count);
      e_enable <= 1'b1;
      e_write <= 1'b1;
      e_outen <= 1'b0;
      wait (e_access == 1'b1);
      for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 1)
	 begin
	    dummy_addr = loop_count;
	    for (vil = 0; vil < 3'b100; vil = vil + 1)
	       begin
		  case (vil)
		    2'b00:
		       begin
			  mem_select = vil;
			  #(`CLOCK_CYCLE - (`CLOCK_CYCLE/5)) dummy_word[16:0] = edb1;
			  dummy_word[23:17] = edb2;
			  temp_X_ram[loop_count] = dummy_word;
			  // Inputs to vec file
			  $fstrobe(dumpfile,
				   $time, " %b %b %b x x %b %h %b %b %b %b %b %b %b x x ",
				   Clk, reset, mem_select, e_enable, 
				   eab_in, edb1_in, edb2_in, e_outen, e_write, test, 
				   scanin1, scanin2 );
		       end
		    2'b01:
		       begin
			  mem_select = vil;
			  #(`CLOCK_CYCLE - (`CLOCK_CYCLE/5)) dummy_word[16:0] = edb1;
			  dummy_word[23:17] = edb2;
			  temp_Y_ram[loop_count] = dummy_word;
			  // Inputs to vec file
			  $fstrobe(dumpfile,
				   $time, " %b %b %b x x %b %h %b %b %b %b %b %b %b x x ",
				   Clk, reset, mem_select, e_enable, 
				   eab_in, edb1_in, edb2_in, e_outen, e_write, test, 
				   scanin1, scanin2 );
		       end
		    2'b10:
		       begin
			  e_outen <= 1'b1;
			  #(`CLOCK_CYCLE - (`CLOCK_CYCLE/5)) e_outen <= 1'b0;
		       end
		    2'b11:
		       begin
			  mem_select = vil;
			  #(`CLOCK_CYCLE - (`CLOCK_CYCLE/5)) dummy_word[16:0] = edb1;
			  // Inputs to vec file
			  $fstrobe(dumpfile,
				   $time, " %b %b %b x x %b %h %b %b %b %b %b %b %b x x ",
				   Clk, reset, mem_select, e_enable, 
				   eab_in, edb1_in, edb2_in, e_outen, e_write, test, 
				   scanin1, scanin2 );
			  dummy_word[23:17] = edb2;
			  temp_P_ram[loop_count] = dummy_word;
		       end
		  endcase
		  @(posedge dummy_estrobe);
	       end
	 end


         
      /*
       *
       * Dump the memories to a file
       *
       */
      mem_dump = $fopen ("verilog_xmem_file");
      for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 4)
         begin
            $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",loop_count,temp_X_ram[loop_count],temp_X_ram[loop_count+1],temp_X_ram[loop_count+2],temp_X_ram[loop_count+3]);
         end
      $fclose(mem_dump);

      mem_dump = $fopen ("verilog_ymem_file");
      for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 4)
         begin
            $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",loop_count,temp_Y_ram[loop_count],temp_Y_ram[loop_count+1],temp_Y_ram[loop_count+2],temp_Y_ram[loop_count+3]);
         end
      $fclose(mem_dump);

      mem_dump = $fopen ("verilog_pmem_file");
      for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 4)
         begin
            $fdisplay (mem_dump, "p:$%x=\t$%x\t$%x\t$%x\t$%x\n",loop_count,temp_P_ram[loop_count],temp_P_ram[loop_count+1],temp_P_ram[loop_count+2],temp_P_ram[loop_count+3]);
         end
      $fclose(mem_dump);
      


      $finish;

      $fclose(dumpfile);
      $shm_close;     /* close the dump file */

		
	end


/*****************************************************************************
 *build the clock signal
 *
 *****************************************************************************/
always
   begin
      // Output data checked 10 ns before rising clock edge
      #(`CLOCK_CYCLE/2 - 10) if (~Clk)
      $fstrobe(dumpfile,
	       $time, " %b %b %b %b %b %b %h %b %b %b %b %b %b %b %b %b ",
	       Clk, reset, mem_select, estrobe, e_access, e_enable, 
	       eab, edb1, edb2, e_outen, e_write, test, 
		  scanin1, scanin2, scanout1, scanout2 );
      #10 Clk <= 1;
      // Inputs to vec file
      $fstrobe(dumpfile,
	       $time, " %b %b %b x x %b %h %b %b %b %b %b %b %b x x ",
	       Clk, reset, mem_select, e_enable, 
	       eab_in, edb1_in, edb2_in, e_outen, e_write, test, 
	       scanin1, scanin2 );
      
      #(`CLOCK_CYCLE/2) Clk <= 0;
      // Inputs to vec file
      $fstrobe(dumpfile,
	       $time, " %b %b %b x x %b %h %b %b %b %b %b %b %b x x ",
	       Clk, reset, mem_select, e_enable, 
	       eab_in, edb1_in, edb2_in, e_outen, e_write, test, 
	       scanin1, scanin2 );
      clock_count = clock_count + 1;
      // $display ("Clock is now: %x\n",clock_count);
   end
			


endmodule	



/*****************************************************************************
 *****************************************************************************/
