/*	File:	cr_test.v	  					*/

/*	module name: top						*/

/*	Description: TEST FILE						*/

/*	Tested Module name:	core_ram.v				*/


/*=======================================================================*/

/*************************************************************************/
/*									*/
/*	module:	top							*/
/*									*/
/*************************************************************************/

module top;


/*============		internal nets	=================================*/

reg reset;

reg Clk;

integer dumpfile, dumpfile_and_stdout;
integer j, mem_dump;

/*=========================================================================*/



/*===========	Module Instantiations	 ===================================*/

core_ram_cascade coreR(
	.reset (reset),
	.Clk (Clk)
	);

/*========================================================================*/


/*======================================================================*/
/*	initialize							*/
/*======================================================================*/

initial
	begin
	
		/* generates a dump file for Cwaves to use */
// 	  	$shm_open("wavesCR.shm");		
		/* keep track of all signals in the hierarcy */
//		$shm_probe("AS");

		/* open text file for dumping results */
//		dumpfile = $fopen("verilogSim.dump");
		/* also send results to standard output (Palnitkar p. 180) */
//		dumpfile_and_stdout = $fopen("verilogSim.dump") | 1;
	        dumpfile_and_stdout =  1;
	
		
		/* load data into RAM modules 		*/
		/* start at RAMData[00]             */
		$readmemh( "pmem_file", coreR.PRAM.RAMData, 8'h00);
		$readmemh( "xmem_file", coreR.XRAM.RAMData, 12'h00);
		$readmemh( "ymem_file", coreR.YRAM.RAMData, 12'h00);
		

		/* Start simulation with reset signal high */
		Clk = 0;
		reset = 1;	
	
		/* end of reset cycle */
		#50 reset = 0;
		

		/*
		 *dump the input and output ram areas
		 */
		/* $writememh ("verilog_before_input_mem", coreR.XRAM.RAMData, 12'h100, 12'h200); */
		/* $writememh ("verilog_before_output_mem", coreR.YRAM.RAMData, 12'h100, 12'h200); */
		mem_dump = $fopen ("verilog_before_input_mem");
		for (j=256; j < 512; j = j + 4)
			$fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,coreR.XRAM.RAMData[j],coreR.XRAM.RAMData[j+1],coreR.XRAM.RAMData[j+2],coreR.XRAM.RAMData[j+3]);
		$fclose (mem_dump);

		mem_dump = $fopen ("verilog_before_output_mem");
		for (j=256; j < 512; j = j +4)
			$fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,coreR.YRAM.RAMData[j],coreR.YRAM.RAMData[j+1],coreR.YRAM.RAMData[j+2],coreR.YRAM.RAMData[j+3]);
		$fclose (mem_dump);


		/*
		 * should need 1802 cycles, but the Verilog model needs 2062
		 */
		 
		#103100;      /* run through the simulation 2062 cycles */


		/*
		 *dump the input and output ram areas
		 */
		/* $writememh ("verilog_after_input_mem", coreR.XRAM.RAMData, 12'h100, 12'h200); */
		/* $writememh ("verilog_after_output_mem", coreR.YRAM.RAMData, 12'h100, 12'h200); */
		mem_dump = $fopen ("verilog_after_input_mem");
		for (j=256; j < 512; j = j + 4)
			$fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,coreR.XRAM.RAMData[j],coreR.XRAM.RAMData[j+1],coreR.XRAM.RAMData[j+2],coreR.XRAM.RAMData[j+3]);
		$fclose (mem_dump);

		mem_dump = $fopen ("verilog_after_output_mem");
		for (j=256; j < 512; j = j + 4)
			$fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,coreR.YRAM.RAMData[j],coreR.YRAM.RAMData[j+1],coreR.YRAM.RAMData[j+2],coreR.YRAM.RAMData[j+3]);
		$fclose (mem_dump);

		$finish; /* end simulation */

//		$fclose(dumpfile);		

//		$shm_close;		 /* close the dump file */
	end



/*======================================================================*/
/*	clock	25 units High, 25 units Low				*/
/* 		on each cycle, dump register contents			*/
/*======================================================================*/

always
	begin
		#25 Clk = 1;
		#25 Clk = 0;

		/* dump contents of architected registers */
		/* to send only to a file, use dumpfile instead of */
		/* dumpfile_and_stdout */
/*
		$fdisplay(dumpfile_and_stdout,
			"\nCore signals: \n\t PDB = %h \t XDB = %h \t YDB = %h \n",
			 coreR.PDB, coreR.XDB, coreR.YDB, 

			"PCU Registers: \n",
			"\t PC = %h \n",
			coreR.CORE.PCU.PC, 	

			"ALU Registers: \n",
			"\t x = %h  \ty = %h\n",
			 {coreR.CORE.ALU.x1, coreR.CORE.ALU.x0},
			 {coreR.CORE.ALU.y1, coreR.CORE.ALU.y0},
			"\t A = %h \t B = %h \n",
			 {coreR.CORE.ALU.a2, coreR.CORE.ALU.a1, coreR.CORE.ALU.a0},
			 {coreR.CORE.ALU.b2, coreR.CORE.ALU.b1, coreR.CORE.ALU.b0},

			"AGU Registers; \n",
			"\t r7 = %h \t n7 = %h \t m7 = %h \n",
			coreR.CORE.agu.agu_datapath.R_H.regout4, coreR.CORE.agu.agu_datapath.N_H.regout4, coreR.CORE.agu.agu_datapath.M_H.regout4, 
			"\t r6 = %h \t n6 = %h \t m6 = %h \n",
			coreR.CORE.agu.agu_datapath.R_H.regout3, coreR.CORE.agu.agu_datapath.N_H.regout3, coreR.CORE.agu.agu_datapath.M_H.regout3, 
			"\t r5 = %h \t n5 = %h \t m5 = %h \n",
			coreR.CORE.agu.agu_datapath.R_H.regout2, coreR.CORE.agu.agu_datapath.N_H.regout2, coreR.CORE.agu.agu_datapath.M_H.regout2, 
			"\t r4 = %h \t n4 = %h \t m4 = %h \n",    
			coreR.CORE.agu.agu_datapath.N_H.regout1, coreR.CORE.agu.agu_datapath.N_H.regout1, coreR.CORE.agu.agu_datapath.M_H.regout1,
			"\t r3 = %h \t n3 = %h \t m3 = %h \n",
			coreR.CORE.agu.agu_datapath.R_L.regout4, coreR.CORE.agu.agu_datapath.N_L.regout4, coreR.CORE.agu.agu_datapath.M_L.regout4, 
			"\t r2 = %h \t n2 = %h \t m2 = %h \n",
			coreR.CORE.agu.agu_datapath.R_L.regout3, coreR.CORE.agu.agu_datapath.N_L.regout3, coreR.CORE.agu.agu_datapath.M_L.regout3, 
			"\t r1 = %h \t n1 = %h \t m1 = %h \n",
			coreR.CORE.agu.agu_datapath.R_L.regout2, coreR.CORE.agu.agu_datapath.N_L.regout2, coreR.CORE.agu.agu_datapath.M_L.regout2,
			"\t r0 = %h \t n0 = %h \t m0 = %h \n",
			coreR.CORE.agu.agu_datapath.R_L.regout1, coreR.CORE.agu.agu_datapath.N_L.regout1, coreR.CORE.agu.agu_datapath.M_L.regout1, 

			"next",$time,
			"\n\n"
			);
*/

	end
	
endmodule	/* end module top */
