/*	File:	test_scan_pads.v  					*/

/*	module name: top						*/

/*	Description: TEST FILE	for scan chain					*/

/*	Tested Module name:	top.v				*/


/*=======================================================================*/

/*************************************************************************/
/*									*/
/*	module:	top							*/
/*									*/
/*************************************************************************/

module top;



/*============		internal nets	=================================*/

   reg		Clk;
   reg		reset;
   reg		e_enable;
   reg [1:0]	mem_select;
   reg		e_outen;
   reg		e_write;
   wire [15:0]	eab;
   wire [23:0]	edb;
   reg		scanin1;
   reg		scanin2;
   wire		scanout1;
   wire		scanout2;
   wire		estrobe;
   wire		e_access;
   reg		test;

   integer	dumpfile, dumpfile_and_stdout;
   integer	j, mem_dump;
   integer      i,k;
   reg scanin;   /* 1 when time to scan in the stuff we just scanned out */
                 /* otherwise it's 0 */
   reg [600:0] scan_output1; 
   reg [600:0] scan_output2;

/*=========================================================================*/



/*===========	Module Instantiations	 ===================================*/

pads fullchip (Clk, reset, mem_select, estrobe, e_access, e_enable, eab,
	       edb, e_outen, e_write, test, scanin1, scanin2, scanout1,
	       scanout2, pwr0, pwr1, pwr2, pwr3, pwr4, pwr5, pwr6, pwr7, 
	       pwr8, pwr9, gnd0, gnd1, gnd2, gnd3, gnd4, gnd5, gnd6, gnd7, 
	       gnd8, gnd9);

/*========================================================================*/


/*======================================================================*/
/*	initialize							*/
/*======================================================================*/

initial
   begin
      
      /* generates a dump file for Cwaves to use */
//      $shm_open("scan.shm");		
      /* keep track of all signals in the hierarcy */
//      $shm_probe("AS");
      
      /* open text file for dumping results */
      dumpfile = $fopen("verilogSim.dump");
      /* also send results to standard output (Palnitkar p. 180) */
      //		dumpfile_and_stdout = $fopen("verilogSim.dump") | 1;
      dumpfile_and_stdout =  1;
      
      
      /* load data into RAM modules 		*/
      /* start at RAMData[00]             */
      $readmemh( "pmem_file", 
		 fullchip.processor.memory.pram_cell.ram.inst1.mem, 8'h00);
      $readmemh( "xmem_file", 
		 fullchip.processor.memory.xram_cell.ram.inst1.mem, 12'h00);
      $readmemh( "ymem_file", 
		 fullchip.processor.memory.yram_cell.ram.inst1.mem, 12'h00);
      
      
      /* Start simulation with reset signal high */
      Clk = 0;
      reset = 1;	
      e_enable = 0;
      mem_select = 2'b00;
      e_outen = 1;
      e_write = 1;
//      eab = 16'hzzz;
//      edb1 = 17'hzzzz;
//      edb2 = 7'hzz;

      scanin2 = 0;
      test = 0;	
      scanin = 0;
      /* end of reset cycle */
      #100 reset = 0;

//      #100 e_enable = 0;

//      #100 reset = 1;

//      #100 reset = 0;
      
      

      /*
       *dump the input and output ram areas
       */
      /* $writememh ("verilog_before_input_mem", coreR.XRAM.RAMData, 12'h100, 12'h200); */
      /* $writememh ("verilog_before_output_mem", coreR.YRAM.RAMData, 12'h100, 12'h200); */
      mem_dump = $fopen ("verilog_before_input_mem");
      for (j=256; j < 512; j = j + 4)
	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,fullchip.processor.memory.xram_cell.ram.inst1.mem[j],fullchip.processor.memory.xram_cell.ram.inst1.mem[j+1],fullchip.processor.memory.xram_cell.ram.inst1.mem[j+2],fullchip.processor.memory.xram_cell.ram.inst1.mem[j+3]);
      $fclose (mem_dump);
      
      mem_dump = $fopen ("verilog_before_output_mem");
      for (j=256; j < 512; j = j +4)
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,fullchip.processor.memory.yram_cell.ram.inst1.mem[j],fullchip.processor.memory.yram_cell.ram.inst1.mem[j+1],fullchip.processor.memory.yram_cell.ram.inst1.mem[j+2],fullchip.processor.memory.yram_cell.ram.inst1.mem[j+3]);
      $fclose (mem_dump);
      
      
       /*
        * let it run for a while first 
        */
        #30050;
	/*
	 * now let's scan out contents of scannable registers
	 */
        test = 1;
        scanin2 = 1;  /* since we're scanning OUT stuff */
        i = 0;  /* reset array counters */
        k = 0;

          #28000;    /* it takes 560 cycles to scan out the entire chain*/
        
	
	/* print out contents of array buffers , check what we've just scanned
	   out */
      for (j=0; j<559; j = j+1)
	 $fdisplay(dumpfile,"scan_output2 at %d is %x\n", j
		   , scan_output2[j]);
      
      i = 0;
      k = 0;
      test = 1;
      scanin = 1;
      
      #28000;   /* 560 cycles to scan back in */
      test = 0;
      scanin = 0;
      scanin2 = 0;
      
      #72000;       /* run through the simulation 2062 cycles */


   
      
      
      /*
       *dump the input and output ram areas
       */
      /* $writememh ("verilog_after_input_mem", coreR.XRAM.RAMData, 12'h100, 12'h200); */
      /* $writememh ("verilog_after_output_mem", coreR.YRAM.RAMData, 12'h100, 12'h200); */
      mem_dump = $fopen ("verilog_after_input_mem");
      for (j=256; j < 512; j = j + 4)
	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,fullchip.processor.memory.xram_cell.ram.inst1.mem[j],fullchip.processor.memory.xram_cell.ram.inst1.mem[j+1],fullchip.processor.memory.xram_cell.ram.inst1.mem[j+2],fullchip.processor.memory.xram_cell.ram.inst1.mem[j+3]);
      $fclose (mem_dump);
      
      mem_dump = $fopen ("verilog_after_output_mem");
      for (j=256; j < 512; j = j + 4)
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,fullchip.processor.memory.yram_cell.ram.inst1.mem[j],fullchip.processor.memory.yram_cell.ram.inst1.mem[j+1],fullchip.processor.memory.yram_cell.ram.inst1.mem[j+2],fullchip.processor.memory.yram_cell.ram.inst1.mem[j+3]);
      $fclose (mem_dump);
      
      $finish; /* end simulation */
      
      $fclose(dumpfile);		
      
//      		$shm_close;		 /* close the dump file */
   end


/*======================================================================*/
/*      clock   25 units High, 25 units Low                             */
/*              on each cycle, dump register contents                   */
/*======================================================================*/

always
        begin
                #25 Clk = 1;
                /* rising edge */
                if (test == 1'b1 && scanin == 1'b0)     
                  begin
                        scan_output2[i] = scanout2;
                        i = i+1;
                  end
                else if (test == 1'b1 && scanin == 1'b1)
                  begin
                        scanin2 = scan_output2[i];
                        i = i+1;
                  end
                #25 Clk = 0;
	end
/*======================================================================*/
/*	clock	25 units High, 25 units Low				*/
/* 		on each cycle, dump register contents			*/
/*======================================================================*/

always
   begin
      #25 Clk = 1;
      #25 Clk = 0;

      /* dump contents of architected registers */
      /* to send only to a file, use dumpfile instead of */
      /* dumpfile_and_stdout */
/*
      $fdisplay(dumpfile,
		"\nCore signals: \n\t PDB = %h \t XDB = %h \t YDB = %h \n",
		fullchip.processor.C_PDB, 
		fullchip.processor.C_XDB, 
		fullchip.processor.C_YDB, 
		
		"PCU Registers: \n",
		"\t PC = %h \n",
		fullchip.processor.dsp.PCU.PC, 	
		
		"ALU Registers: \n",
		"\t x = %h  \ty = %h\n",
		{fullchip.processor.dsp.ALU.x1, 
		 fullchip.processor.dsp.ALU.x0},
		{fullchip.processor.dsp.ALU.y1, 
		 fullchip.processor.dsp.ALU.y0},
		"\t A = %h \t B = %h \n",
		{fullchip.processor.dsp.ALU.a2, 
		 fullchip.processor.dsp.ALU.a1, 
		 fullchip.processor.dsp.ALU.a0},
		{fullchip.processor.dsp.ALU.b2, 
		 fullchip.processor.dsp.ALU.b1, 
		 fullchip.processor.dsp.ALU.b0},
		
		"next",$time,
		"\n\n"
		);
*/
   end
	
endmodule	/* end module top */
