//	File:	add_sub_16bit.v	  					
//	module name: add_sub_16bit					
//	Description:	A generic adder/subtractor with 2 inputs of 16 bit each. 
//                      This file is a structural description of the module
//	Author: Jonathan Tong			


//==============================================================================


//******************************************************************************
//*	module:	add_sub_16bit						       *
//******************************************************************************

module add_sub_16bit (
	a,
	b,
	control,
	out
	);


//==============================================================================
//============	I/O direction	================================================
//==============================================================================

input [15:0] a;
input [15:0] b;
input control;

output [15:0] out;


//==============================================================================
//============		I/O type	========================================
//==============================================================================

wire [15:0] a;
wire [15:0] b;
wire control;			// (control = 0) => (out = a + b)		
				// (control = 1) => (out = a - b)		

wire [15:0] out;


//==============================================================================
//============		Internal Nets	============================================
//==============================================================================

//wire [15:0] bcomp;		// ~b 

wire [15:0] bin;		// b input into the adder


parameter DPATH = "add_sub_16bit";
//==============================================================================
//============		Instantiated Module		====================================
//==============================================================================

xor2 #(16,1,DPATH) addsub_xor(
	b, {16{control}}, bin);

addhs 
	#(16,1,DPATH) addhs1 (
	.A (a),
	.B (bin),
	.CIN (control),
	.COUT (),
	.SUM (out)
	);




endmodule	// end module add_sub_16bit 
