// This file adds an 8-bit register which tells the address arithmetic
// for each of the 8 M registers. Each bit corresponds to one M register
// If the bit is 0, then the corresponding M reg specifies linear or
// bit_reverse arithmetic. Else it specifies modulo address arithmetic
//
// Can be used in cycle 2 by the decoding logic to determine latch
// enable signals for the adders in the address ALUs.
// ===================================================================
// Author : Jonathan Tong                  9/24/97
// ===================================================================

module addr_modifier (
	Clk,
	reset,
	Min1_L,
	Min1_H,
	Mwaddr1_L,
	Mwaddr1_H,
	Min2_L,
	Min2_H,
	Mwaddr2_L,
	Mwaddr2_H,
	Mwrite1_L,
	Mwrite2_L,
	Mwrite1_H,
	Mwrite2_H,
	addr_modifier_out
	);

input Clk;
input reset;
input [15:0] Min1_L;
input [15:0] Min1_H;
input [1:0] Mwaddr1_L;
input [1:0] Mwaddr1_H;
input [15:0] Min2_L;
input [15:0] Min2_H;
input [1:0] Mwaddr2_L;
input [1:0] Mwaddr2_H;
input Mwrite1_L;
input Mwrite2_L;
input Mwrite1_H;
input Mwrite2_H;

output [7:0] addr_modifier_out;

//=======================================================

reg [7:0] addr_modifier_out;


always @(posedge Clk or posedge reset)
begin
  if (reset)
	addr_modifier_out = 8'b0;
  else 
     begin
	
    	if ( Min1_L != 16'b0 && Min1_L != 16'hffff && Mwrite1_L) 	   
		addr_modifier_out[Mwaddr1_L] = 1'b1;
    	else if (Mwrite1_L) 				   
		addr_modifier_out[Mwaddr1_L] = 1'b0;  

	if ( Min2_L != 16'b0 && Min2_L != 16'hffff && Mwrite2_L) 	   
		addr_modifier_out[Mwaddr2_L] = 1'b1;
    	else if (Mwrite2_L) 				   
		addr_modifier_out[Mwaddr2_L] = 1'b0;  

	if ( Min1_H != 16'b0 && Min1_H != 16'hffff && Mwrite1_H) 	   
		addr_modifier_out[{1'b1,Mwaddr1_H}] = 1'b1;
    	else if (Mwrite1_H) 				   
		addr_modifier_out[{1'b1,Mwaddr1_H}] = 1'b0;  

	if ( Min2_H != 16'b0 && Min2_H != 16'hffff && Mwrite2_H) 	   
		addr_modifier_out[{1'b1,Mwaddr2_H}]= 1'b1;
    	else if (Mwrite2_H) 				   
		addr_modifier_out[{1'b1,Mwaddr2_H}] = 1'b0;  

    		
     end
end


endmodule 