/*	File:	addsub_c.v	  					*/

/*	module name: addsub_c						*/

/*	Description: adder/subtractor module for ALU			*/
/*					using Cascade module		*/

/*				Calculates in1 +/- in2			*/


/*======================================================================*/





   `include "./verilog/globals.v"



/************************************************************************/
/*									*/
/*	module:								*/
/*									*/
/************************************************************************/

module addsub_c (
	in1,
	in2,
	naddsub,
	sum,
	overflow,
	carry,
	zero,
	negative
	);


/*============	I/O direction	=========================================*/

input [`acc]	 in1, in2;
input 		naddsub;    /* 0 if add, 1 if subtract */
output [`acc]	sum;
output		carry, overflow, zero, negative;
					/* goes to status bits */

/*========================================================================*/


/*============		I/O type	==================================*/

wire [`acc]	sum;
wire		carry, overflow, zero, negative;	
					/* goes to status bits */

/*============		internal nets	==================================*/

wire [`acc]	in2_sign;	/* in2, negated if subtract */
wire		cin;		/* carry in, 1 if subtract */
wire		cout;		/* carry in, 1 if subtract */

/*========================================================================*/


/*======================================================================*/
/*	processes							*/
/*======================================================================*/

/* if this is subtract (naddsub = 1), we need to negate in2 */
/* this is done by XORing each bit of in2 with naddsub and setting cin to naddsub */
/* should I DPFLAG be set to 0? */


   assign in2_sign = in2 ^ {56{naddsub}};




assign cin = naddsub;

/*=================== module instanitations =============================*/

/* use cascade carry look ahead adder */


   DW01_add #(56) ADDER(in1, in2_sign, cin, sum, cout);




   
assign carry = cout;

/* if cout is not the same as the sign bit, the adder overflowed */

   assign overflow = cout ^ sum[55];




   
/* check for zero */

   assign zero = (sum == 56'b0) ? 1'b1 : 1'b0 ;




   
assign negative = sum[55];


endmodule	/* end module */
