/*	File:	control_p.v	  						*/

/*	module name: control_p						*/

/*	Description: 		 						*/

/*		control logic for parallel move				*/

/*		all signals latched			*/


/*==============================================================================*/




   `include "./verilog/globals.v"


/********************************************************************************/
/*																				*/
/*	module:																		*/
/*																				*/
/********************************************************************************/

module control_p (
		  Clk, reset,
		  pdb,
		  immediate,
		  x0_to_XDB,
		  x1_to_XDB,
		  y0_to_YDB,
		  y1_to_YDB,
		  l0_to_YDB,
		  l1_to_YDB,
		  b0_to_YDB,
		  l0_to_XDB,
		  l1_to_XDB,
		  a0_to_XDB,
		  x0_write, x0_sel,
		  x1_write, x1_sel,
		  y0_write, y0_sel,
		  y1_write, y1_sel,
//		  limit0_lsb,
//		  limit1_lsb,
		  limit0_pass,
		  limit1_pass,
		  db2a1_sel,
		  areg_sel,	
		  a2_write,
		  a1_write,
		  a0_write,
		  db2b1_sel,
		  breg_sel,
		  b2_write,
		  b1_write,
		  b0_write,
		
		  scan_in, test, scan_out

		  );

	
/*========================= I/O direction ============================*/

   input	    Clk, reset;
   input [`databus] pdb;	/* instruction word */
   input	    immediate;		/* flag for immediate data on pdb */
   output	    x0_write, x1_write, y0_write, y1_write;
   output [1:0]	    x0_sel, x1_sel, y0_sel, y1_sel;
//   output		 limit0_lsb, limit1_lsb;	/* limiter used for lsb in long move */
   output	    limit0_pass, limit1_pass;	/* do not limit if 1 */   
   output	    x0_to_XDB, x1_to_XDB, l0_to_XDB, l1_to_XDB;
   output	    y0_to_YDB, y1_to_YDB, l0_to_YDB, l1_to_YDB;
   output	    b0_to_YDB, a0_to_XDB;
   output	    db2a1_sel, areg_sel;	
   output	    a2_write, a1_write, a0_write;
   output	    db2b1_sel, breg_sel;
   output	    b2_write, b1_write, b0_write;
/*------------- scan chain -----------------*/
   input	    scan_in, test;
   output	    scan_out;

/*========================= I/O type ============================*/


   wire		    x1_write, x0_write;	/* write enable signal for x registers */
   reg [1:0]	    x1_sel, x0_sel;		/* select signal for x registers */
   wire		    y1_write, y0_write;	/* write enable signal for y registers */
   reg [1:0]	    y1_sel, y0_sel;		/* select signal for y registers */
//   reg		 limit0_lsb, limit1_lsb;
   reg		    limit0_pass, limit1_pass;
   wire		    x0_to_XDB, x1_to_XDB, l0_to_XDB, l1_to_XDB;
   wire		    y0_to_YDB, y1_to_YDB, l0_to_YDB, l1_to_YDB;
   wire		    b0_to_YDB, a0_to_XDB;
   reg		    db2a1_sel, areg_sel;	
   wire		    a2_write, a1_write, a0_write;
   reg		    db2b1_sel, breg_sel;
   wire		    b2_write, b1_write, b0_write;


/*========================= Internal Nets ============================*/

/* register inputs for control signals that get latched */
   reg		    x1_write_d, x0_write_d;	/* write enable signal for x registers */
   reg [1:0]	    x1_sel_d, x0_sel_d;	/* select signal for x registers */
   reg [1:0]	    y1_sel_d, y0_sel_d;	/* select signal for y registers */
   reg		    y1_write_d, y0_write_d;	/* write enable signal for y registers */
//   reg		 limit0_lsb_d, limit1_lsb_d;
   reg		    limit0_pass_d, limit1_pass_d;
   reg		    x0_to_XDB_d, x1_to_XDB_d, l0_to_XDB_d, l1_to_XDB_d;
   reg		    y0_to_YDB_d, y1_to_YDB_d, l0_to_YDB_d, l1_to_YDB_d;
   reg		    b0_to_YDB_d, a0_to_XDB_d;
   reg		    db2a1_sel_d, areg_sel_d;	
   reg		    a2_write_d, a1_write_d, a0_write_d;
   reg		    db2b1_sel_d, breg_sel_d;
   reg		    b2_write_d, b1_write_d, b0_write_d;
   
/* register output for signals that are disabled during scan mode */
   reg		    x1_write_t, x0_write_t;	/* write enable signal for x registers */
   reg		    y1_write_t, y0_write_t;	/* write enable signal for y registers */
   reg		    x0_to_XDB_t, x1_to_XDB_t, l0_to_XDB_t, l1_to_XDB_t;
   reg		    y0_to_YDB_t, y1_to_YDB_t, l0_to_YDB_t, l1_to_YDB_t;
   reg		    b0_to_YDB_t, a0_to_XDB_t;
   reg		    a2_write_t, a1_write_t, a0_write_t;
    reg		    b2_write_t, b1_write_t, b0_write_t;
   
/*---------------- scan chain -------------------*/
   reg		    scan_out;


/*======================================================================*/
/*	processes							*/
/*======================================================================*/

   always @(pdb or immediate)
      begin
	 /* set defaults */
	 /* signals that effect stage 3 and are latched */
	 x0_to_XDB_d = 0;
	 x1_to_XDB_d = 0;
	 y0_to_YDB_d = 0;
	 y1_to_YDB_d = 0;
	 l0_to_YDB_d = 0;	/* limit0 tied to A */
	 l1_to_YDB_d = 0;	/* limit1 tied to B */
	 l0_to_XDB_d = 0;	/* limit0 tied to A */
	 l1_to_XDB_d = 0;	/* limit1 tied to B */
	 a0_to_XDB_d = 0;	
	 b0_to_YDB_d = 0;      
	 x0_write_d = 0;
	 x1_write_d = 0;
	 y0_write_d = 0;
	 y1_write_d = 0;
	 x0_sel_d = 2'b00;	/* default to DB bus */
	 x1_sel_d = 2'b00;	/* default to DB bus */
	 y0_sel_d = 2'b00;	/* default to DB bus */
	 y1_sel_d = 2'b00;	/* default to DB bus */
//	 limit0_lsb_d = 0;
//	 limit1_lsb_d = 0;
	 limit0_pass_d = 0;
	 limit1_pass_d = 0;
	 db2a1_sel_d = 0;        /* default to XDB */
	 areg_sel_d  = 0;	/* defailt to Data Path */
	 a2_write_d = 0;	a1_write_d = 0;	a0_write_d = 0;
	 db2b1_sel_d = 1;        /* default to YDB */
	 breg_sel_d  = 0;	/* defailt to Data Path */
	 b2_write_d = 0;	b1_write_d = 0;	b0_write_d = 0;
	 
	 if (!immediate) begin	/* make sure pdb is not immediate data */
	    casex (pdb[23:8])	
	      /*--------------------------------------*/
	      /*	No parallel move		*/
	      /*					*/
	      /* this goes first to prevent aliasing	*/
	      /*--------------------------------------*/
	      16'b0010000000000000 : ;	/* do nothing */
	      /*--------------------------------------*/
	      /*	MOVE X: Y:			*/
	      /*					*/
	      /*--------------------------------------*/
	      16'b1xxxxxxxxxxxxxxx : begin
		 $display($time, " MOVE X: Y: activated.  PDB = %h", pdb);		
		 /* X: portion */
		 if (!pdb[15])		/* read from registers */
		    begin
		       case (pdb[19:18])	/* source */
			 2'b00: x0_to_XDB_d = 1;
			 2'b01: x1_to_XDB_d = 1;
			 2'b10: l0_to_XDB_d = 1;	/* limit0 tied to A */
			 2'b11: l1_to_XDB_d = 1;	/* limit1 tied to B */
		       endcase
		    end
		 else begin		/* write to registers  */
		    case (pdb[19:18])	/* destination */
		      2'b00: x0_write_d = 1;
		      2'b01: x1_write_d = 1;
		      2'b10: begin /* A  */
			 db2a1_sel_d = 0;
			 areg_sel_d  = 1;
			 a2_write_d = 1;	a1_write_d = 1;	a0_write_d = 1;
		      end		
		      2'b11: begin /* B  */
			 db2b1_sel_d = 0;
			 breg_sel_d  = 1;
			 b2_write_d = 1;	b1_write_d = 1;	b0_write_d = 1;
		      end		
		    endcase
		 end
		 /* Y: portion */
		 if (!pdb[22])		/* write to Y memory */
		    begin
		       case (pdb[17:16])	/* source */
			 2'b00: y0_to_YDB_d = 1;
			 2'b01: y1_to_YDB_d = 1;
			 2'b10: l0_to_YDB_d = 1;	/* limit0 tied to A */
			 2'b11: l1_to_YDB_d = 1;	/* limit1 tied to B */
		       endcase
		    end
		 else begin		/* read from Y memory */
		    case (pdb[17:16])	/* destination */
		      2'b00: y0_write_d = 1;
		      2'b01: y1_write_d = 1;
		      2'b10: begin /* A  */
			 db2a1_sel_d = 1;
			 areg_sel_d  = 1;
			 a2_write_d = 1;	a1_write_d = 1;	a0_write_d = 1;
		      end		
		      2'b11: begin /* B  */
			 db2b1_sel_d = 1;
			 breg_sel_d  = 1;
			 b2_write_d = 1;	b1_write_d = 1;	b0_write_d = 1;
		      end		
		    endcase
		 end
	      end /* MOVE X: Y: */
	      /*--------------------------------------*/
	      /*		MOVE X:			*/
	      /*					*/
	      /* can be aliased by MOVE L:		*/
	      /*--------------------------------------*/
	      16'b010x0xxxxxxxxxxx : begin	
		 $display($time, " MOVE X: activated.  PDB = %h", pdb);		
		 if (!pdb[15])		/* read from registers  */
		    begin
		       case ({pdb[20],pdb[18:16]})	/* source register */
			 4'b0100: x0_to_XDB_d = 1;
			 4'b0101: x1_to_XDB_d = 1;
			 4'b0110: y0_to_YDB_d = 1;	/* use bus switch */
			 4'b0111: y1_to_YDB_d = 1;	/* use bus switch */
				4'b1100: begin /* A1 */
				   l0_to_XDB_d = 1;	/* limit0 tied to A */
				   limit0_pass_d = 1;
				end // case: 5'b01100
				4'b1101: begin /* B1 */
				   l1_to_YDB_d = 1;	/* limit1 tied to B */
				   limit1_pass_d = 1;
				end // case: 5'b01101
			 4'b1110: l0_to_XDB_d = 1;	/* limit0 tied to A */
			 4'b1111: l1_to_XDB_d = 1;	/* limit1 tied to B */
			 default: $display ("ERROR (control_p): Instruction not supported.");
		       endcase
		    end	/* if (!pdb[15]) */
		 else begin		/* write to registers */
		    case ({pdb[20],pdb[18:16]})	/* destination */
		      4'b0100: begin
			 x0_write_d = 1;
			 if (pdb[13:8] == 6'b110100)
			    x0_sel_d = 2'b10;
		      end
		      4'b0101:  begin
			 x1_write_d = 1;
			 if (pdb[13:8] == 6'b110100)
			    x1_sel_d = 2'b10;
		      end
		      4'b0110:  begin
			 y0_write_d = 1;
			 if (pdb[13:8] == 6'b110100)
			    y0_sel_d = 2'b10;
		      end
		      4'b0111:  begin
			 y1_write_d = 1;
			 if (pdb[13:8] == 6'b110100)
			    y1_sel_d = 2'b10;
		      end
		      4'b1100: begin /* A1  */
			 if (pdb[13:8] == 6'b110100)
			    $display ("ERROR (control_p): Instruction not supported.");
			 else begin
			    db2a1_sel_d = 0;
			    areg_sel_d  = 1;
			    a2_write_d = 0;	a1_write_d = 1;	a0_write_d = 0;
			 end // else: !if(pdb[13:8] == 6'b110100)
		      end		
		      4'b1101: begin /* B1  */
			 if (pdb[13:8] == 6'b110100)
			    $display ("ERROR (control_p): Instruction not supported.");
			 else begin
			    db2b1_sel_d = 0;
			    breg_sel_d  = 1;
			    b2_write_d = 0;	b1_write_d = 1;	b0_write_d = 0;
			 end // else: !if(pdb[13:8] == 6'b110100)
		      end
		      4'b1110: begin /* A  */			
			 if (pdb[13:8] == 6'b110100)
			    $display ("ERROR (control_p): Instruction not supported.");
			 else begin
			    db2a1_sel_d = 0;
			    areg_sel_d  = 1;
			    a2_write_d = 1;	a1_write_d = 1;	a0_write_d = 1;
			 end
		      end		
		      4'b1111: begin /* B  */
			 if (pdb[13:8] == 6'b110100)
			    $display ("ERROR (control_p): Instruction not supported.");
			 else begin
			    db2b1_sel_d = 0;
			    breg_sel_d  = 1;
			    b2_write_d = 1;	b1_write_d = 1;	b0_write_d = 1;
			 end
		      end
		      default:$display ("ERROR (control_p): Instruction not supported."); 
		    endcase
		 end
	      end /* MOVE X: */
	      /*--------------------------------------*/
	      /*		MOVE Y:			*/
	      /*					*/
	      /* can be aliased by MOVE L:		*/
	      /*--------------------------------------*/
	      16'b010x1xxxxxxxxxxx : begin	
		 $display($time, " MOVE Y: activated.  PDB = %h", pdb);		
		if (!pdb[15])		/* read from registers  */
		   begin
		      case ({pdb[20],pdb[18:16]})	/* source register */
			4'b0100: x0_to_XDB_d = 1;	/* use bus switch */
			4'b0101: x1_to_XDB_d = 1;	/* use bus switch */
			4'b0110: y0_to_YDB_d = 1;
			4'b0111: y0_to_YDB_d = 1;
			4'b1100: begin /* A1 */
			   l0_to_XDB_d = 1;	/* limit0 tied to A */
			   limit0_pass_d = 1;
			end // case: 5'b01100
			4'b1101: begin /* B1 */
			   l1_to_YDB_d = 1;	/* limit1 tied to B */
			   limit1_pass_d = 1;
			end // case: 5'b01101
			4'b1110: l0_to_YDB_d = 1;	/* limit0 tied to A */
			4'b1111: l1_to_YDB_d = 1;	/* limit1 tied to B */
			default: $display ("ERROR (control_p): Instruction not supported.");
		      endcase
		   end	/* if (!pdb[15]) */
		else begin		/* write to registers */
		   case ({pdb[20],pdb[18:16]})	/* destination */
		     4'b0100: begin
			x0_write_d = 1;
			if (pdb[13:8] == 6'b110100)
			   x0_sel_d = 2'b10;
		     end
		     4'b0101: begin
			x1_write_d = 1;
			if (pdb[13:8] == 6'b110100)
			   x1_sel_d = 2'b10;
		     end
		     4'b0110: begin
			y0_write_d = 1;
			if (pdb[13:8] == 6'b110100)
			   y0_sel_d = 2'b10;
		     end
		     4'b0111: begin
			y1_write_d = 1;
			if (pdb[13:8] == 6'b110100)
			   y1_sel_d = 2'b10;
		     end
		     4'b1100: begin /* A1  */
			if (pdb[13:8] == 6'b110100)
			   $display ("ERROR (control_p): Instruction not supported.");
			else begin
			   db2a1_sel_d = 1;
			   areg_sel_d  = 1;
			   a2_write_d = 0;	a1_write_d = 1;	a0_write_d = 0;
			end // else: !if(pdb[13:8] == 6'b110100)
		     end		
		     4'b1101: begin /* B1  */
			if (pdb[13:8] == 6'b110100)
			   $display ("ERROR (control_p): Instruction not supported.");
			else begin
			   db2b1_sel_d = 1;
			   breg_sel_d  = 1;
			   b2_write_d = 0;	b1_write_d = 1;	b0_write_d = 0;
			end
		     end
		     4'b1110: begin /* A  */
			 if (pdb[13:8] == 6'b110100)
			    $display ("ERROR (control_p): Instruction not supported.");
			 else begin
			    db2a1_sel_d = 1;
			    areg_sel_d  = 1;
			    a2_write_d = 1;	a1_write_d = 1;	a0_write_d = 1;
			 end // else: !if(pdb[13:8] == 6'b110100)
		     end		
		     4'b1111: begin /* B  */
			if (pdb[13:8] == 6'b110100)
			   $display ("ERROR (control_p): Instruction not supported.");
			else begin
			   db2b1_sel_d = 1;
			   breg_sel_d  = 1;
			   b2_write_d = 1;	b1_write_d = 1;	b0_write_d = 1;
			end
		     end
		     default:$display ("ERROR (control_p): Instruction not supported."); 
		   endcase
		end
	      end /* MOVE Y: */
	      /*--------------------------------------*/
	      /*		MOVE U			*/
	      /*					*/
	      /* 					*/
	      /*--------------------------------------*/
	      /* Do nothing, but put above MOVE R  */
	      /* to avoid aliasing */
	      16'b00100000010xxxxx : begin
	      end /* move U */
	      /*--------------------------------------*/
	      /*		MOVE R			*/
	      /*					*/
	      /* 					*/
	      /*--------------------------------------*/
	      /* if move from N or R register, assume */
	      /* bus switch will direct to X or Y bus */
	      16'b001000xxxxxxxxxx : begin	
		 $display($time, " MOVE R activated.  PDB = %h.", pdb);
		 casex (pdb[12:8])	/* destination */
		   5'b00100: begin
		      casex (pdb[17:13])
			5'b1xxxx:  x0_write_d = 1;    /* from AGU */
			5'b001xx:  x0_write_d = 1;    /* from x0, x1, y0, y1 */
			5'b011xx:  x0_write_d = 1;    /* from a, b, a1, b1 */
			5'b0100x:  x0_write_d = 1;    /* from a0, b0 */
			default: $display ("ERROR (control_p): Instruction not supported."); 
		      endcase
		   end
		   5'b00101: begin
		      casex (pdb[17:13])
			5'b1xxxx:  x1_write_d = 1;
			5'b001xx:  x1_write_d = 1;    /* from x0, x1, y0, y1 */
			5'b011xx:  x1_write_d = 1;    /* from a, b, a1, b1 */
			5'b0100x:  x1_write_d = 1;    /* from a0, b0 */
			default: $display ("ERROR (control_p): Instruction not supported."); 
		      endcase
		   end
		   5'b00110: begin
		      casex (pdb[17:13])
			5'b1xxxx:  y0_write_d = 1;
			5'b001xx:  y0_write_d = 1;    /* from x0, x1, y0, y1 */
			5'b011xx:  y0_write_d = 1;    /* from a, b, a1, b1 */
			5'b0100x:  y0_write_d = 1;    /* from a0, b0 */
			default: $display ("ERROR (control_p): Instruction not supported."); 
		      endcase
		   end
		   5'b00111: begin
		      casex (pdb[17:13])
			5'b1xxxx:  y1_write_d = 1;
			5'b001xx:  y1_write_d = 1;    /* from x0, x1, y0, y1 */
			5'b011xx:  y1_write_d = 1;    /* from a, b, a1, b1 */
			5'b0100x:  y1_write_d = 1;    /* from a0, b0 */
			default: $display ("ERROR (control_p): Instruction not supported."); 
		      endcase
		   end // case: 5'b00111
		   5'b01100: begin /* A1 */
		      casex (pdb[17:13])
			5'b1xxxx, 5'b001xx, 5'b011xx, 5'b0100x:  begin 
			   areg_sel_d  = 1;
			   a2_write_d = 0;	a1_write_d = 1;	a0_write_d = 0;
			end		
			default: $display ("ERROR (control_p): Instruction not supported."); 
		      endcase
		   end
		   5'b01101, 5'b001xx, 5'b011xx, 5'b0100x: begin  /* B1 */
		      casex (pdb[17:13])
			5'b1xxxx:  begin 
			   breg_sel_d  = 1;
			   b2_write_d = 0;	b1_write_d = 1;	b0_write_d = 0;
			end		
			default: $display ("ERROR (control_p): Instruction not supported."); 
		      endcase
		   end
		   5'b01110, 5'b001xx, 5'b011xx, 5'b0100x: begin /* A */
		      casex (pdb[17:13])
			5'b1xxxx:  begin 
			   areg_sel_d  = 1;
			   a2_write_d = 1;	a1_write_d = 1;	a0_write_d = 1;
			end		
			default: $display ("ERROR (control_p): Instruction not supported."); 
		      endcase
		   end
		   5'b01111, 5'b001xx, 5'b011xx: begin  /* B */
		      casex (pdb[17:13])
			5'b1xxxx, 5'b001xx, 5'b011xx, 5'b0100x:  begin 
			   breg_sel_d  = 1;
			   b2_write_d = 1;	b1_write_d = 1;	b0_write_d = 1;
			end		
			default: $display ("8ERROR (control_p): Instruction not supported."); 
		      endcase
		   end // casex: 5'b01111
		   5'b1xxxx: ;  /* N register or R register */
		   default:$display ("ERROR (control_p): Instruction not supported."); /* A, B not implemented */
		 endcase // casex (pdb[12:8])
		 casex (pdb[12:8])  /* possible desinatetions from ALU */
		   5'b1xxxx,  /* N register or R register */
		   5'b001xx,  /* from x0, x1, y0, y1 */
		   5'b011xx,  /* from a, b, a1, b1 */
		   5'b0100x:     /* from a0, b0 */
		      casex (pdb[17:13]) /* source register */
			5'b1xxxx: ; /* Source and Destination in AGU, do nothing */
			5'b00100: x0_to_XDB_d = 1;
			5'b00101: x1_to_XDB_d = 1;
			5'b00110: y0_to_YDB_d = 1;
			5'b00111: y0_to_YDB_d = 1;
			5'b01100: begin /* A1 */
			   l0_to_XDB_d = 1;	/* limit0 tied to A */
			   limit0_pass_d = 1;
			end // case: 5'b01100
			5'b01101: begin /* B1 */
			   l1_to_YDB_d = 1;	/* limit0 tied to A */
			   limit1_pass_d = 1;
			end // case: 5'b01101
			5'b01110: l0_to_XDB_d = 1;	/* limit0 tied to A */
			5'b01111: l1_to_YDB_d = 1;	/* limit1 tied to B */
			5'b01000: a0_to_XDB_d = 1;	/* limit0 tied to A */
			5'b01001: b0_to_YDB_d = 1;	/* limit1 tied to B */
			5'b00010:  ;	/* This is Move U, do nothing */
			default: $display ("ERROR (control_p): Instruction not supported.");
		      endcase
		   default:$display ("ERROR (control_p): Instruction not supported."); /* A, B not implemented */
		 endcase
	      end /* MOVE I */
	      /*--------------------------------------*/
	      /*		MOVE I			*/
	      /*					*/
	      /* 					*/
	      /*--------------------------------------*/
	      16'b0010xxxxxxxxxxxx : begin	
		 $display($time, " MOVE I activated.  PDB = %h.", pdb);	
		 casex ({pdb[20],pdb[18:16]})	/* destination */
		   4'b0100: begin
		      x0_write_d = 1;
		      x0_sel_d = 2'b01;
		   end
		   4'b0101: begin
		      x1_write_d = 1;
		      x1_sel_d = 2'b01;
		   end
		   4'b0110: begin
		      y0_write_d = 1;
		      y0_sel_d = 2'b01;
		   end
		   4'b0111: begin
		      y1_write_d = 1;
		      y1_sel_d = 2'b01;
		   end
		   4'b0000:  ;	/* This is Move U, do nothing */
		   4'b00xx:  ;	/* This is Move R, do nothing */
		   default:$display ("ERROR (control_p): Instruction not supported."); /* A, B not implemented */
		 endcase
	      end /* MOVE I */
	      default: ;	/* other moves NOT SUPPORTED */
	    endcase /* casex (pdb[23:8]) */
	    
	    casex (pdb[23:0])	
	      /*-----------------------------------------*/
	      /*	           No op		*/
	      /*					*/
	      /* this goes first to prevent aliasing	*/
	      /*----------------------------------------*/
	      24'b001000000000000000000000 : ;	/* do nothing */
	      /*--------------------------------------*/
	      /*		REP			*/
	      /*					*/
	      /* 					*/
	      /*--------------------------------------*/
	      24'b0000110_1100xxxx_0010000 : begin	
		 $display($time, " REP activated.  PDB = %h.", pdb);
		 casex (pdb[17:13]) /* source register */
		   4'b0100: x0_to_XDB_d = 1;
		   4'b0101: x1_to_XDB_d = 1;
		   4'b0110: y0_to_YDB_d = 1;
		   4'b0111: y0_to_YDB_d = 1;
		   4'b1100: begin /* A1 */
		      l0_to_XDB_d = 1;	/* limit0 tied to A */
		      limit0_pass_d = 1;
		   end // case: 5'b01100
		   4'b1101: begin /* B1 */
		      l1_to_YDB_d = 1;	/* limit0 tied to A */
		      limit1_pass_d = 1;
		   end // case: 5'b01101
		   4'b1110: l0_to_XDB_d = 1;	/* limit0 tied to A */
		   4'b1111: l1_to_YDB_d = 1;	/* limit1 tied to B */
		   default: $display ("ERROR (control_p): Instruction not supported.");
		 endcase
	      end /* REP */
	    endcase /* casex (pdb[23:0]) */
	 end /* if (!immediate) */
      end /* always */
   
/*===========	Latch control signals		=================================*/
   
   always @(posedge Clk)
      begin
	 if (reset)
	    begin
	       x0_to_XDB_t <= 0;
	       x1_to_XDB_t <= 0;
	       y0_to_YDB_t <= 0;
	       y1_to_YDB_t <= 0;
	       l0_to_YDB_t <= 0;	/* limit0 tied to A */
	       l1_to_YDB_t <= 0;	/* limit1 tied to B */
	       l0_to_XDB_t <= 0;	/* limit0 tied to A */
	       l1_to_XDB_t <= 0;	/* limit1 tied to B */
	       a0_to_XDB_t <= 0;
	       b0_to_YDB_t <= 0;
	       x0_write_t  <= 0;
	       x1_write_t  <= 0;
	       y0_write_t  <= 0;
	       y1_write_t  <= 0;
	       x0_sel <= 0;
	       x1_sel <= 0;
	       y0_sel <= 0;
	       y1_sel <= 0;
	       limit0_pass <= 0;
	       limit1_pass <= 0;
	       db2a1_sel <= 0;
	       areg_sel  <= 0;	
	       a2_write_t <= 0;
	       a1_write_t <= 0;
	       a0_write_t <= 0;
	       db2b1_sel <= 0;
	       breg_sel  <= 0;
	       b2_write_t <= 0;
	       b1_write_t <= 0;
	       b0_write_t <= 0;
	    end
	 else begin
	    x0_to_XDB_t <= x0_to_XDB_d;
	    x1_to_XDB_t <= x1_to_XDB_d;
	    y0_to_YDB_t <= y0_to_YDB_d;
	    y1_to_YDB_t <= y1_to_YDB_d;
	    l0_to_YDB_t <= l0_to_YDB_d;	/* limit0 tied to A */
	    l1_to_YDB_t <= l1_to_YDB_d;	/* limit1 tied to B */
	    l0_to_XDB_t <= l0_to_XDB_d;	/* limit0 tied to A */
	    l1_to_XDB_t <= l1_to_XDB_d;	/* limit1 tied to B */
	    a0_to_XDB_t <= a0_to_XDB_d;
	    b0_to_YDB_t <= b0_to_YDB_d;
	    x0_write_t <= x0_write_d;
	    x1_write_t <= x1_write_d;
	    y0_write_t <= y0_write_d;
	    y1_write_t <= y1_write_d;
	    x0_sel <= x0_sel_d;
	    x1_sel <= x1_sel_d;
	    y0_sel <= y0_sel_d;
	    y1_sel <= y1_sel_d;
//	    limit0_lsb <= limit0_lsb_d;
//	    limit1_lsb <= limit1_lsb_d;
	    limit0_pass <= limit0_pass_d;
	    limit1_pass <= limit1_pass_d;
	    db2a1_sel <= db2a1_sel_d;
	    areg_sel  <= areg_sel_d;	
	    a2_write_t <= a2_write_d;
	    a1_write_t <= a1_write_d;
	    a0_write_t <= a2_write_d;
	    db2b1_sel <= db2b1_sel_d;
	    breg_sel  <= breg_sel_d;
	    b2_write_t <= b2_write_d;
	    b1_write_t <= b1_write_d;
	    b0_write_t <= b0_write_d;
	 end
      end
   /* end always */
   
      /* add gates to disable register enables when test (scan signal) is high */
   assign x0_to_XDB = x0_to_XDB_t & !test;
   assign x1_to_XDB = x1_to_XDB_t & !test;
   assign y0_to_YDB = y0_to_YDB_t & !test;
   assign y1_to_YDB = y1_to_YDB_t & !test;
   assign l0_to_YDB = l0_to_YDB_t & !test;	/* limit0 tied to A */
   assign l1_to_YDB = l1_to_YDB_t & !test;	/* limit1 tied to B */
   assign l0_to_XDB = l0_to_XDB_t & !test;	/* limit0 tied to A */
   assign l1_to_XDB = l1_to_XDB_t & !test;	/* limit1 tied to B */
   assign a0_to_XDB = a0_to_XDB_t & !test;
   assign b0_to_YDB = b0_to_YDB_t & !test;
   assign x0_write = x0_write_t & !test;
   assign x1_write = x1_write_t & !test;
   assign y0_write = y0_write_t & !test;
   assign y1_write = y1_write_t & !test;
   assign a2_write = a2_write_t & !test;
   assign a1_write = a1_write_t & !test;
   assign a0_write = a2_write_t & !test;
   assign b2_write = b2_write_t & !test;
   assign b1_write = b1_write_t & !test;
   assign b0_write = b0_write_t & !test;
   
endmodule
