/*	File:	extcheck_c.v	  						*/

/*	module name: extcheck_c						*/

/*	Description: Checks exension and MSB to see if value */
/*				 is represented without extension, i.e.	 */
/*				 are the msb and ext bits all the same value */


/*======================================================================*/


/************************************************************************/
/*									*/
/*	module:								*/
/*									*/
/************************************************************************/

module extcheck_c (
	in,
	S1, S0,
	not_equal
	);


/*============	Parameters	========================================*/

parameter dpname = "extcheck";

/*============	I/O direction	========================================*/


input [9:0] 	in;
input 	S1, S0;					/* status bits */
output	not_equal;					/* all bits are the the same */

/*=======================================================================*/


/*============		I/O type	==================================*/

wire not_equal;

/*============	internal nets	=========================================*/

wire [1:0]	in_inv;				/* all values of in inverted */
wire all_zero8, all_zero9, all_zero10;
wire all_one8, all_one8a, all_one8b, all_one9, all_one10;
   reg all_zero, all_one;

   
/*=====================================================================*/


/*======================================================================*/
/*	processes							*/
/*======================================================================*/

/*=================== module instanitations ============================*/

   assign in_inv = ~in[1:0];


/* Check for all zero */
   assign all_zero8 = (in[9:2] == 0);
   assign all_zero9 = (all_zero8 & in_inv[1]);
   assign all_zero10 = (all_zero9 & in_inv[0]);

   always @(S1 or S0 or all_zero8 or all_zero9 or all_zero10)
     case ({S1,S0})
	2'b00: all_zero = all_zero9;
	2'b01: all_zero = all_zero8;
	2'b10: all_zero = all_zero10;
	default: all_zero = 0;
     endcase // case{S0,S1}

   assign all_one8a = in[9] & in[8] & in[7] & in[6];
   assign all_one8b = in[5] & in[4] & in[3] & in[2];
   assign all_one8 = all_one8a & all_one8b;
   assign all_one9 = all_one8 & in[1];
   assign all_one10 = all_one9 & in[0];
   
   always @(S1 or S0 or all_one8 or all_one9 or all_one10)
     case ({S1,S0})
       2'b00: all_one = all_one9;
       2'b01: all_one = all_one8;
       2'b10: all_one = all_one10;
       default: all_one = 0;
     endcase // case{S0,S1}
   
   assign not_equal = ~(all_zero | all_one);

endmodule // extcheck_c

