/*	File:	multi_wrap_address.v					*/

/*	module name: multi_wrap_address					*/

/*	Description: Generates the new register value for multi-wrap-around modulo addressing mode.	*/


/* Author: Nitzan Weinberg */


/*==============================================================================*/


/********************************************************************************/
/*																				*/
/*	module:	multi_wrap_address													*/
/*																				*/
/********************************************************************************/

module multi_wrap_address (
	Rn,
	Mn,
	linear,
	multi_wrap_out
	);


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

input [15:0] Rn;
input [15:0] Mn;
input [15:0] linear;

output [15:0] multi_wrap_out;



/*==============================================================================*/
/*============		I/O type	================================================*/
/*==============================================================================*/

wire [15:0] Rn;
wire [15:0] Mn;
wire [15:0] linear;

reg [15:0] multi_wrap_out;



/*==============================================================================*/
/*===========	Internal Nets	================================================*/
/*==============================================================================*/



/*==============================================================================*/
/*	Processes																	*/
/*==============================================================================*/

always @(	Rn or
			Mn or
			linear
		)
	begin	/*----------------------*/
			/* multiple wrap around */
			/*----------------------*/

		if ( Mn == 16'h8001 )
			multi_wrap_out = {Rn[15:1], linear[0]};

		else if ( Mn == 16'h8003 )
			multi_wrap_out = {Rn[15:2], linear[1:0]};

		else if ( Mn == 16'h8007 )
			multi_wrap_out = {Rn[15:3], linear[2:0]};

		else if ( Mn == 16'h800f )
			multi_wrap_out = {Rn[15:4], linear[3:0]};

		else if ( Mn == 16'h801f )
			multi_wrap_out = {Rn[15:5], linear[4:0]};

		else if ( Mn == 16'h803f )
			multi_wrap_out = {Rn[15:6], linear[5:0]};

		else if ( Mn == 16'h807f )
			multi_wrap_out = {Rn[15:7], linear[6:0]};

		else if ( Mn == 16'h80ff )
			multi_wrap_out = {Rn[15:8], linear[7:0]};

		else if ( Mn == 16'h81ff )
			multi_wrap_out = {Rn[15:9], linear[8:0]};

		else if ( Mn == 16'h83ff )
			multi_wrap_out = {Rn[15:10], linear[9:0]};

		else if ( Mn == 16'h87ff )
			multi_wrap_out = {Rn[15:11], linear[10:0]};

		else if ( Mn == 16'h8fff )
			multi_wrap_out = {Rn[15:12], linear[11:0]};

		else if ( Mn == 16'h9fff )
			multi_wrap_out = {Rn[15:13], linear[12:0]};

		else if ( Mn == 16'hbfff )
			multi_wrap_out = {Rn[15:14], linear[13:0]};
			
		else
			multi_wrap_out = 16'hxxxx;

	end		/* always */




endmodule		/* end module multi_wrap_address */
