/*	File:	mux8_16bit.v	  					*/

/*	module name: mux8_16bit					*/

/*	Description:	A generic selector between 14 inputs of 16 bit each, towards a single output of 16 bit. */

/*	Author: Nitzan Weinberg			*/


/*==============================================================================*/


/*==============================================================================*/


/********************************************************************************/
/*																				*/
/*	module:	mux8_16bit												*/
/*																				*/
/********************************************************************************/

module mux8_16bit (
	in0,
	in1,
	in2,
	in3,
	in4,
	in5,
	in6,
	in7,
	control,
	out
	);


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

input [15:0] in0;
input [15:0] in1;
input [15:0] in2;
input [15:0] in3;
input [15:0] in4;
input [15:0] in5;
input [15:0] in6;
input [15:0] in7;
input [2:0] control;

output [15:0] out;


/*==============================================================================*/
/*============		Internal Nets	============================================*/
/*==============================================================================*/

wire [15:0] temp0;		/* output from m0 */
wire [15:0] temp1;		/* output from m1 */



/*==============================================================================*/
/*============		Instantiated Modules	====================================*/
/*==============================================================================*/

/* choose among in0-3 */

mux4
	#(16,0,"AUTO","1") m0 (
	.IN0 (in0),
	.IN1 (in1),
	.IN2 (in2),
	.IN3 (in3),
	.S0 (control [0]),
	.S1 (control [1]),
	.Y (temp0)
	);


/* choose among in4-7 */

mux4
	#(16,0,"AUTO","1") m1 (
	.IN0 (in4),
	.IN1 (in5),
	.IN2 (in6),
	.IN3 (in7),
	.S0 (control [0]),
	.S1 (control [1]),
	.Y (temp1)
	);



/* choose among temp0-1 */

mux2
	#(16,0,"AUTO","1") m2 (
	.IN0 (temp0),
	.IN1 (temp1),
	.S0 (control [2]),
	.Y (out)
	);



endmodule	/* end module mux8_16bit */
