/*	File:	mux_mult.v	 			*/

/*	Description: input muxes for multiplier 	*/
/*			uses cascade modules		*/
/*                   Includes guard latches             */

/*========================================================================*/


/* `include "/afs/ece.cmu.edu/usr/lowpower/dsp/alu/globals.v" */




   `include "./verilog/globals.v"



/***************************************************************************/
/*																			*/
/*	module:																	*/
/*																			*/
/***************************************************************************/

module mux_mult (
	x0, x1, y0, y1,
	decode_out,
	mult_aa_ctl, mult_bb_ctl,
	mult_a_ctl,
	mult_ena,
	aa, a, bb, b
	);

/*============	I/O direction	===========================================*/

   input [`databus] x0, x1, y0, y1;
   input [`databus] decode_out;
   input [1:0]	    mult_aa_ctl, mult_bb_ctl;
   input	    mult_a_ctl;
   input	    mult_ena;
   
   output [`databus] aa;    // feeds adder input
   output [`databus] bb;    // feeds adder input
   output [`databus] a;
   output [`databus] b;

/*=========================================================================*/


/*==================== I/O type ===========================================*/

   wire [`databus]   aa;

   reg [`databus]    a, b;



/*===================== internal =======================*/

   wire [`databus]   mux_aa_out;
   wire [`databus]   mux_a_out, mux_b_out;
   

/*==========================================================================*/


/*==========================================================================*/
/*	processes							*/
/*==========================================================================*/


/* this is a cascade module */

/* first level: select among x0, x1, y0, y1 */

   DW01_mux_any #(96, 2, 24) MUX_AA(
       {y1, x1, y0, x0}, mult_aa_ctl, mux_aa_out);
   DW01_mux_any #(96, 2, 24) MUX_BB(
       {x1, y1, y0, x0}, mult_bb_ctl, mux_b_out);







   
assign aa = mux_aa_out;
assign bb = mux_b_out;
   

/* second level, a: choose among decode or mult_aa */

   DW01_mux_any #(48, 1, 24) MUX_A(
       {mux_aa_out, decode_out}, mult_a_ctl, mux_a_out);




   
/* Guard latches */

   always @(mult_ena or mux_a_out or mux_b_out)
    if (mult_ena)
     begin
	a <= mux_a_out;
	b <= mux_b_out;
     end 








endmodule	/* end module */



