/* $Id: ram_enable_control.v,v 2.1 1998/02/05 22:20:35 benk Exp $ */
/*****************************************************************************
 *ram_enable_control
 *
 *selects the control signals and generates the right looking signals for the
 *internal SRAMS
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: benk $
 *Date: $Date: 1998/02/05 22:20:35 $
 *Revision: $Revision: 2.1 $
 *
 *
 *Log: $Log: ram_enable_control.v,v $
 *Log: Revision 2.1  1998/02/05 22:20:35  benk
 *Log: Changed assignment operators from "<=" to "="
 *Log:
# Revision 2.0  1997/12/16  21:06:36  inacio
# up the version to 2.0
#
# Revision 1.6  1997/12/16  21:06:10  inacio
# a new version of the ram enable control lines, with the logic Designed By
# Commitee  (TM) Jon Herman and Ben.
#
# Revision 1.5  1997/12/03  00:23:56  inacio
# corrected the write logic from the core.
#
# Revision 1.4  1997/11/15  07:08:53  inacio
# reformatted the source and changed the logic to deal with the fact the
# cpu_stio is active low, not active high
#
# Revision 1.3  1997/11/09  08:51:46  inacio
# I changed the tabbing in the files to be spaces, now it looks the same in
# less as it does in my editor
#
# Revision 1.2  1997/10/21  17:21:42  inacio
# fixed some sizing problems and formatting
#
# Revision 1.1  1997/10/14  20:19:29  inacio
# moved the file into the before synth subdir
#
# Revision 1.1  1997/10/14  19:56:14  inacio
# Logic to control the output enable and write enable signals to the SRAM
#
 *****************************************************************************/


module ram_enable_control (cpu_stop, mem_slct, clk, CX_WRITE, CX_READ,
        CY_READ, CY_WRITE, E_OUTEN, E_WRITE, EP_OUTEN, EP_WRITE, EX_OUTEN,
        EX_WRITE, EY_OUTEN, EY_WRITE, c_xab_ov, c_pab_ov, c_yab_ov);


input    cpu_stop;
input    [1:0]mem_slct;
input    clk;
input    CX_WRITE;
input    CX_READ;
input    CY_READ;
input    CY_WRITE;
input    E_OUTEN;
input    E_WRITE;
input    c_xab_ov;
input    c_pab_ov;
input    c_yab_ov;
output   EP_OUTEN;
output   EP_WRITE;
output   EX_OUTEN;
output   EX_WRITE;
output   EY_OUTEN;
output   EY_WRITE;


reg      EP_OUTEN;
reg      EP_WRITE;
reg      EX_OUTEN;
reg      EX_WRITE;
reg      EY_OUTEN;
reg      EY_WRITE;




always @(cpu_stop or mem_slct or CX_WRITE or CX_READ or CY_READ or CY_WRITE or E_OUTEN or E_WRITE or clk or c_xab_ov or c_yab_ov or c_pab_ov)
   begin

      /*
       * cpu_stop is active low
       */
      if (cpu_stop == 1'b0)
         begin
            case (mem_slct)
               2'b00:
                  begin
                     if (E_OUTEN == 1'b0 && E_WRITE == 1'b1)
                        begin
                           EX_OUTEN = 1'b0;
                        end
                     else
                        begin
                           EX_OUTEN = 1'b1;
                        end

                     if (E_OUTEN == 1'b1 && E_WRITE == 1'b0)
                        begin
                           EX_WRITE = 1'b0;
                        end
                     else
                        begin
                           EX_WRITE = 1'b1;
                        end

                     EY_OUTEN = 1'b1;
                     EY_WRITE = 1'b1;
                     EP_OUTEN = 1'b1;
                     EP_WRITE = 1'b1;
                  end
               2'b01:
                  begin
                     if (E_OUTEN == 1'b0 && E_WRITE == 1'b1)
                        begin
                           EY_OUTEN = 1'b0;
                        end
                     else
                        begin
                           EY_OUTEN = 1'b1;
                        end

                     if (E_OUTEN == 1'b1 && E_WRITE == 1'b0)
                        begin
                           EY_WRITE = 1'b0;
                        end
                     else
                        begin
                           EY_WRITE = 1'b1;
                        end

                     EX_OUTEN = 1'b1;
                     EX_WRITE = 1'b1;
                     EP_OUTEN = 1'b1;
                     EP_WRITE = 1'b1;
                  end
               2'b10:
                  begin
                     EX_OUTEN = 1'b1;
                     EX_WRITE = 1'b1;
                     EY_OUTEN = 1'b1;
                     EY_WRITE = 1'b1;
                     EP_OUTEN = 1'b1;
                     EP_WRITE = 1'b1;
                  end
               2'b11:
                  begin
                     if (E_OUTEN == 1'b0 && E_WRITE == 1'b1)
                        begin
                           EP_OUTEN = 1'b0;
                        end
                     else
                        begin
                           EP_OUTEN = 1'b1;
                        end

                     if (E_OUTEN == 1'b1 && E_WRITE == 1'b0)
                        begin
                           EP_WRITE = 1'b0;
                        end
                     else
                        begin
                           EP_WRITE = 1'b1;
                        end


                     EX_OUTEN = 1'b1;
                     EX_WRITE = 1'b1;
                     EY_OUTEN = 1'b1;
                     EY_WRITE = 1'b1;
                  end
            endcase
         end
      else /* core is in control ... */
         begin
            if (c_xab_ov == 1'b1)
               begin
                  EX_OUTEN = 1'b1;
                  EX_WRITE = 1'b1;
               end
            else
               begin
                  EX_OUTEN = ~CX_READ;
                  EX_WRITE = ~CX_WRITE | clk;
               end
   
            if (c_yab_ov == 1'b1)
               begin
                  EY_OUTEN = 1'b1;
                  EY_WRITE = 1'b1;
               end
            else
               begin
                  EY_OUTEN = ~CY_READ;
                  EY_WRITE = ~CY_WRITE | clk;
               end
   
            if (c_pab_ov == 1'b1)
               begin
                  EP_OUTEN = 1'b1;
                  EP_WRITE = 1'b1;
               end
            else
               begin
                  EP_OUTEN = 1'b0;
                  EP_WRITE = 1'b1;
               end
         end
   end
endmodule


