/*	File:	rfn.v	  					*/

/*	module name: rfn						*/

/*	Description:	A generic register file of 4 16-bit registers, 3 read and 2 write ports.	*/
/*					It has a synchronized reset control that sets all registers to $0000.		*/


/* Author: Jonathan Tong*/


/*==============================================================================*/


/*==============================================================================*/


// epo set_attribute FIXEDBLOCK = 1

/********************************************************************************/
/*																				*/
/*	module:	rfn																	*/
/*																				*/
/********************************************************************************/


module rfn (
	reset,
	in1,
	raddr1,
	raddr2,
	waddr1,
	write1,
	out1,	
	out2,	
	Clk
	);

/*===================    PARAMETERS   ================================*/

parameter dpname = "rfn_dp";

/*============	I/O direction	================================================*/

input reset;
input [15:0] in1;
input [1:0] raddr1;
input [1:0] raddr2;
input [1:0] waddr1;
input write1;

output [15:0] out1;
output [15:0] out2;


input Clk;





/*============		I/O type	================================================*/

wire reset;				/* synchronized reset */
wire reset_inv;				/* inversion of reset signal */

wire [15:0] in1;		/* input data bus port 1 */

wire [1:0] raddr1;		/* read address port 1 */
wire [1:0] raddr2;		/* read address port 2 */

wire [1:0] waddr1;		/* write address port 1 */

wire write1;			/* synchronized write enable signal for incoming data on port 1 */

wire [15:0] regout1;              /* outtput of register 1 */
wire [15:0] regout2;              /* output of register 2 */
wire [15:0] regout3;              /* input to register 3 */
wire [15:0] regout4;	          /* input to register 4 */

wire we1;              /* write enable to register 1 */
wire we2;              /* write enable to register 2 */
wire we3;              /* write enable to register 3 */
wire we4;	       /* write enable to register 4 */

wire we1_inv;              /* inversion of write enable to register 1 */
wire we2_inv;              /* inversion of write enable to register 2 */
wire we3_inv;              /* inversion of write enable to register 3 */
wire we4_inv;	           /* inversion of write enable to register 4 */

wire [3:0] decodeout1;   /* output of the first decoder */

wire [15:0] out1;		/* output data bus port 1 */
wire [15:0] out2;		/* output data bus port 2 */



wire Clk;

/*=======    Cascade module instantiation      ===============================*/


   assign reset_inv = ~reset;
















   

   DW03_reg_s_pl #(16, 0) reg1 (in1, Clk, reset_inv, we1, regout1);   









   

   DW03_reg_s_pl #(16, 0) reg2 (in1, Clk, reset_inv, we2, regout2);   











   DW03_reg_s_pl #(16, 0) reg3 (in1, Clk, reset_inv, we3, regout3);   











   DW03_reg_s_pl #(16, 0) reg4 (in1, Clk, reset_inv, we4, regout4);   











   DW01_mux_any #(64, 2, 16) outmux1
       ({regout4, regout3, regout2, regout1}, raddr1, out1);  













   DW01_mux_any #(64, 2, 16) outmux2
       ({regout4, regout3, regout2, regout1}, raddr2, out2);  











   
mydecoder decoder1 (
	.SEL(waddr1),
	.Y(decodeout1)
	);

assign we1 = (write1 & decodeout1[0]);
assign we2 = (write1 & decodeout1[1]);
assign we3 = (write1 & decodeout1[2]);
assign we4 = (write1 & decodeout1[3]);


endmodule	/* end module rf */


