/*	File:	round.v	  						*/

/*	module name: round						*/

/*	Description: round */

/*		Rounds input using convergent rounding	*/
/*		Do NOT round if {s1, s0} = 11		*/

/* Note: scaling mode bits not supported. */


/*=========================================================================*/




   `include "./verilog/globals.v"


/************************************************************************/
/*									*/
/*	module:								*/
/*									*/
/************************************************************************/

module round_c (
	s0, s1,
	in,
	out
	);
// rem epoch pre_compiled round_c
// rem epoch set_attribute FIXEDBLOCK = 1

/*============	I/O direction	==========================================*/

   input	 s0, s1;	
   input [`acc]	 in;
   output [`acc] out;

/*=========================================================================*/



/*============		I/O type	===================================*/

   wire [`acc]	 out;

/*=========================================================================*/


/*==================	Internal Nets	===================================*/

   wire		 lsb_all_zero;
   wire		 lsb_inv;       /* inverted lsb_all_zero */
   wire [24:22]	 ai_s1, ai_s0;	/* intermediate logic for add_in */
   wire [55:22]	 add_in;	/* number to add for rounding */
//   wire [24:22]	 add_out;	/* result of rounding addition */
//   wire		 inc_in;	/* carry out of adder triggers incrementer */
   wire [55:22]	 inc_out;	/* result of incrementer */
   wire [25:23]	 conv, conv_or;	// bits flipped for convergent rounding
   wire		 trunc_sel;   
   

/*======================================================================*/
/*	processes							*/
/*======================================================================*/


/* Check lsb (21:0) for all zero */

   assign lsb_all_zero = (in[21:0] == 22'b0) ? 1 : 0;




   
/* Based on scaling mode bits, decide which bit gets added */
/*  S1  S0  |	add_in */
/*   0   0  |  	010 */
/*   0   1  |  	100 */
/*   1   0  |  	001 */
/*   1   1  |  	000 (don't round)*/

assign ai_s0[24] = s0;   
assign ai_s1[22] = s1;
   

   assign ai_s1[24] = ~s1;
   assign ai_s1[23] = ~s1;
   assign ai_s0[23] = ~s0;
   assign ai_s0[22] = ~s0;
   assign add_in[24:22] = ai_s1 & ai_s0;












//assign add_in[22] = (~s1 & s0);
//assign add_in[23] = (~s1 & ~s0);
//assign add_in[24] = (s1 & ~s0);


assign   add_in[55:25] = 31'b0;
   
/* Increment msb (55:25) to round, depending on cout of adder */

   DW01_add #(34) INC (.A(add_in), .B(in[55:22]), .CI(1'b0), .SUM(inc_out), .CO(tc)); 




/* Convergent rounding is a pain			 */
/*  S1  S0  |	add_in 	out[25:23]			*/
/*   0   0  |  	010 	inc_out[25], conv, 0 		*/
/*   0   1  |  	100 	conv, 00			*/
/*   1   0  |  	001 	inc_out[25], add_out[24], conv	*/
/*   1   1  |  	000 	inc_out[25], add_out[24:23]	*/

/* convergent rounding means that if out[23:0] = 0, make out[24] = 0 */

   assign lsb_inv = ~lsb_all_zero;
   assign conv_or =  {3{lsb_inv}} | {3{inc_out[22]}} | {inc_out[23],inc_out[23],1'b0} | {inc_out[24],2'b0};  
   assign conv = conv_or & {inc_out[25],inc_out[24:23]};











//assign conv[23] = (!lsb_all_zero | add_out[22]) & add_out[23];
//assign conv[24] = (!lsb_all_zero | add_out[23] | add_out[22]) & add_out[24];
//assign conv[25] = (!lsb_all_zero | add_out[24] | add_out[23] | add_out[22]) & inc_out[25];


   DW01_mux_any #(12, 2, 3) CONV_MUX(
	{inc_out[25:23], {inc_out[25:24], conv[23]}, {conv[25], 2'b00},  {inc_out[25], conv[24], 1'b0} }, {s1, s0}, out[25:23]);






assign out[55:26] = inc_out[55:26];

/* truncate unless s1, s0 = 11 */

   assign trunc_sel = ~(s0 & s1);
   DW01_mux_any #(46, 1, 23) TRUNC({23'h000000, in[22:0]}, trunc_sel, out[22:0]);







   
endmodule	/* end module */
