
/*----------------------------------------------------------------------*/
/*									*/
/*	module:	right_shifter						*/
/*									*/
/*	Right shift a 24-bit signed number and sign extend it.		*/
/*	The source is initially places in MSB (47:24) and then shifted.	*/
/*	Can shift between 1 and 23 bits to the right.			*/
/*	The functions return a 56-bit signed number.			*/
/*									*/
/*----------------------------------------------------------------------*/

module right_shifter (data_in, shift, rs_out);

input [`databus] data_in;		/* source to be shifted */
input [4:0] shift;			/* the number of bits to shift */
output [`acc] rs_out;			/* right shifted output */

reg [47:0] longdata;			/* the extended source to be shifted */
reg [`acc] rs_out;


	always @(data_in or shift)
	begin
		longdata = { data_in, 24'h000000 };		/* the source is put at MSB position in the long word */
		
		case (shift)
			5'b00001	:	rs_out = {{  9 {longdata[47]}}, longdata[47:1]  };
			5'b00010	:	rs_out = {{ 10 {longdata[47]}}, longdata[47:2]  };
			5'b00011	:	rs_out = {{ 11 {longdata[47]}}, longdata[47:3]  };
			5'b00100	:	rs_out = {{ 12 {longdata[47]}}, longdata[47:4]  };
			5'b00101	:	rs_out = {{ 13 {longdata[47]}}, longdata[47:5]  };
			5'b00110	:	rs_out = {{ 14 {longdata[47]}}, longdata[47:6]  };
			5'b00111	:	rs_out = {{ 15 {longdata[47]}}, longdata[47:7]  };
			5'b01000	:	rs_out = {{ 16 {longdata[47]}}, longdata[47:8]  };
			5'b01001	:	rs_out = {{ 17 {longdata[47]}}, longdata[47:9]  };
			5'b01010	:	rs_out = {{ 18 {longdata[47]}}, longdata[47:10] };
			5'b01011	:	rs_out = {{ 19 {longdata[47]}}, longdata[47:11] };
			5'b01100	:	rs_out = {{ 20 {longdata[47]}}, longdata[47:12] };
			5'b01101	:	rs_out = {{ 21 {longdata[47]}}, longdata[47:13] };
			5'b01110	:	rs_out = {{ 22 {longdata[47]}}, longdata[47:14] };
			5'b01111	:	rs_out = {{ 23 {longdata[47]}}, longdata[47:15] };
			5'b10000	:	rs_out = {{ 24 {longdata[47]}}, longdata[47:16] };
			5'b10001	:	rs_out = {{ 25 {longdata[47]}}, longdata[47:17] };
			5'b10010	:	rs_out = {{ 26 {longdata[47]}}, longdata[47:18] };
			5'b10011	:	rs_out = {{ 27 {longdata[47]}}, longdata[47:19] };
			5'b10100	:	rs_out = {{ 28 {longdata[47]}}, longdata[47:20] };
			5'b10101	:	rs_out = {{ 29 {longdata[47]}}, longdata[47:21] };
			5'b10110	:	rs_out = {{ 30 {longdata[47]}}, longdata[47:22] };
			5'b10111	:	rs_out = {{ 31 {longdata[47]}}, longdata[47:23] };
			default		:	;	/* No Action */
		endcase		/* shift */
	end


endmodule		/* right_shifter */




