/* $Id: thingy_control.v,v 1.10 1998/02/19 20:03:50 benk Exp $ */
/*****************************************************************************
 *thingy_control.v
 *
 *logic to control the bus thingy's.  Mux/tribuf/demultiplexers...
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: benk $
 *Date: $Date: 1998/02/19 20:03:50 $
 *Revision: $Revision: 1.10 $
 *
 *
 *Log: $Log: thingy_control.v,v $
 *Log: Revision 1.10  1998/02/19 20:03:50  benk
 *Log: Removed x overflow detect from port of thingy_control
 *Log:
# Revision 1.9  1998/01/27  18:55:44  benk
# Changed EDB tristates
#
# Revision 1.8  1998/1/24  15:50:28  benk
# Redid logic for memory mode
#
# Revision 1.8  1997/12/15  15:50:28  benk
# Fixed logic for dir signals to avoid implied latch from Synopsys
#
# Revision 1.7  1997/12/14  23:21:09  benk
# Rebuilt control for thingy.  All logic in thingy_control.
# Added X_READ, Y_READ to enable signals.
#
# Revision 1.6  1997/11/09  09:58:28  inacio
# reformatted the file, added out-of-range detection to turn off the RAMs is
# you try to read past them. added some comments and fixed a couple of bugs
#
# Revision 1.5  1997/11/09  08:51:47  inacio
# I changed the tabbing in the files to be spaces, now it looks the same in
# less as it does in my editor
#
# Revision 1.4  1997/11/04  08:33:38  inacio
# updated the sensitivity list
#
# Revision 1.3  1997/10/29  22:50:34  inacio
# added in support for the test signal for the scan chain
#
# Revision 1.2  1997/10/21  17:21:52  inacio
# added some comments and fixed some stuff
#
# Revision 1.1  1997/10/14  20:19:32  inacio
# moved the file into the before synth subdir
#
# Revision 1.4  1997/10/13  21:44:42  inacio
# updated the sensitivity list
#
# Revision 1.2  1997/10/10  20:32:19  inacio
# control logic for the thingies, now relies on the clock_stop signal instead
# of EENABLE
#
# Revision 1.1  1997/10/08  23:40:23  inacio
# The control logic for the thingy module.  Eventually the mem_slct lines will
# have to be latched and clocked, making them obsolete in here, but I javen't
# done that yet...
#
 *****************************************************************************/

module thingy_control (cpu_stop, MEM_SLCT, 
			CX_WRITE, CY_WRITE,CX_READ, CY_READ, 
                     	E_WRITE, E_OUTEN,
			nC_to_xram, nC_to_yram, nC_to_pram, 
			nRS_to_xram, nRS_to_yram, nRS_to_pram, 
			nxram_to_C, nyram_to_C, npram_to_C, 
			nxram_to_RS, nyram_to_RS, npram_to_RS, 
			test);


   input       cpu_stop;     // cpu is stopped, external bus is enabled
   input [1:0] MEM_SLCT;    // selects which mem bank to access
           // from external bus
   input       CX_WRITE;     // memory write control signals (C's from
   input       CY_WRITE;     // core, E from external)
   input       E_WRITE;
   input       CX_READ;     // memory read control signals (C's from
   input       CY_READ;     // core, E from external)
   input       E_OUTEN;

/* All output signals are active low */
   output      nC_to_xram;	// from core to memory
   output      nC_to_yram;
   output      nC_to_pram;
   output      nRS_to_xram;	// from pins to memory
   output      nRS_to_yram;
   output      nRS_to_pram;
   output      nxram_to_C;	// from memory to core
   output      nyram_to_C;
   output      npram_to_C;
   output      nxram_to_RS;
   output      nyram_to_RS;
   output      npram_to_RS;

   input       test;       // test signal, scan chains are ective

/*
 * some definitions to make Synopsys happy...
 */
   reg	       nC_to_xram;	// from core to memory
   reg	       nC_to_yram;
   reg	       nC_to_pram;
   reg	       nRS_to_xram;	// from pins to memory
   reg	       nRS_to_yram;
   reg	       nRS_to_pram;
   reg	       nxram_to_C;	// from memory to core
   reg	       nyram_to_C;
   reg	       npram_to_C;
   reg	       nxram_to_RS;
   reg	       nyram_to_RS;
   reg	       npram_to_RS;





always @(cpu_stop or MEM_SLCT or E_WRITE or E_OUTEN or 
	 CX_WRITE or CY_WRITE or CX_READ or CY_READ or
	 test)
   begin
      
   	/* set defaults */
      
      nC_to_xram = 1;	// from core to memory
      nC_to_yram = 1;
      nC_to_pram = 1;
      nxram_to_C = 1;	// from memory to core
      nyram_to_C = 1;
      npram_to_C = 1;
      nRS_to_xram = 1;	// from pins to memory
      nRS_to_yram = 1;
      nRS_to_pram = 1;
      nxram_to_RS = 1;	// from memory to pins
      nyram_to_RS = 1;
      npram_to_RS = 1;
      
      /*
       *
       * thingy enable control signals....  (cpu_stop is active low)
       * and all signals connecting ram to core
       *
       */
      if (test==1'b1)
	 begin
	 	/* use defaults */
	 end
      else
	 if (cpu_stop==1'b1)
	    begin
	       /*
		* cpu active (DSP mode)
		*/
	       nC_to_xram = !CX_WRITE;
	       nC_to_yram = !CY_WRITE;
	       nxram_to_C = !CX_READ;
	       nyram_to_C = !CY_READ;
	       npram_to_C = 1'b0;
	       
	    end
	 else
	       /*
		* cpu inactive (memory mode)
		*/
	      /*
	      	e_outen | e_write | nRS_to_mem | nmem_to_RS | direction
	      	--------+---------+------------+------------+----------
	      	   0  	|   0	  |       1    |      1     |  no connect
	      	   0  	|   1	  |       1    |      0     |  from memory
	      	   1  	|   0	  |       0    |      1     |  to memory
	      	   1  	|   1	  |       0    |      1     |  to memory
	      	   */
	    case (MEM_SLCT)
	      2'b00 :  /* X memory */
		 case ({E_OUTEN, E_WRITE})
		   2'b00: ; /* illegal, use default */
		   2'b01:   /* Read memory */
		     nxram_to_RS = 0;
		   2'b10:   /* Write Memory */
		     nRS_to_xram = 0;
		   2'b11:   /* idle, or part of write cycle */
		     nRS_to_xram = 0;
		 endcase
	      2'b01 :  /* Y memory */
		 case ({E_OUTEN, E_WRITE})
		   2'b00: ; /* illegal, use default */
		   2'b01:   /* Read memory */
		     nyram_to_RS = 0;
		   2'b10:   /* Write Memory */
		     nRS_to_yram = 0;
		   2'b11:   /* idle, or part of write cycle */
		     nRS_to_yram = 0;
		 endcase
	      2'b10 :; /* illegal, use default */
	      2'b11 :  /* P memory */
		 case ({E_OUTEN, E_WRITE})
		   2'b00: ; /* illegal, use default */
		   2'b01:   /* Read memory */
		     npram_to_RS = 0;
		   2'b10:   /* Write Memory */
		     nRS_to_pram = 0;
		   2'b11:   /* idle, or part of write cycle */
		     nRS_to_pram = 0;
		 endcase
	    endcase
      
      end
endmodule

