/*=======================================================================*
 *	File:	test.v
 *
 *	module name: top
 *
 *	Description: TEST FILE
 *
 *	
 * $Revision: 1.19 $
 * $Date: 1999/06/04 13:28:27 $
 *
 *=======================================================================*/

/*************************************************************************/
/*									*/
/*	module:	top							*/
/*									*/
/*************************************************************************/


`define CLOCK_CYCLE 100
`define SCAN_CHAIN_LENGTH 616
`define N_LENGTH 0
module top;


/*============		internal nets	=================================*/

   reg		Clk;
   reg		reset;
   reg		e_enable;
   reg [1:0]	mem_select;
   reg		e_outen;
   reg		e_write;
   wire [15:0]	eab;
   wire [23:0]	edb;
   reg		scanin;
   wire		scanout;
   wire		estrobe;
   wire		e_access;
   reg		test;

   /* Wires used for testing interface */
   reg [15:0]	dummy_addr;    /* address for EAB */
   reg [23:0]	dummy_data;    /* data for write to EDB */
   reg		eab_input, edb_input;
   /*
    * fake memories to load up the real Cascade memories and store results
    */
   reg [23:0]	temp_P_ram[1023:0];
   reg [23:0]	temp_X_ram[1023:0];
   reg [23:0]	temp_Y_ram[1023:0];
   /*
    * Memory and variables for testing the scan chain
    */
   integer	i,k;
   reg		control_scanin;   /* 1 when time to scan in the stuff 
			   * we just scanned out 
			   * otherwise it's 0 */
   reg [(`SCAN_CHAIN_LENGTH+`N_LENGTH):0]	scan_output; 

   /*
    * some other variables used in the tes bench
    */
   integer	dumpfile, mem_dump;  // file handles
   integer	j, vil;              // loop variables

/*======================== tasks used for interface =======================*/

/***************************************************************************
 * write one memory location
 *
 ***************************************************************************/
task write_mem ;
   input [15:0] address;
   input [24:0] data;
   begin
      dummy_addr = address;
      dummy_data = data[23:0];
      #(`CLOCK_CYCLE/4) e_write = 1'b0;
      #(`CLOCK_CYCLE/2) e_write = 1'b1;
      #(`CLOCK_CYCLE/4) ;
   end
endtask // write_mem

/***************************************************************************
 * read one memory location
 *
 ***************************************************************************/
task read_mem ;
   input [15:0] address;
   begin
      #(`CLOCK_CYCLE/4) ;
      dummy_addr = address;
      #(`CLOCK_CYCLE*3/4) ;
   end
endtask // write_mem

/*****************************************************************************
 *build the assign statements to drive the busses
 *
 *****************************************************************************/
assign
	eab = (eab_input) ? dummy_addr : 16'bz,
        edb = (edb_input) ? dummy_data : 24'bz;
   

/*===========	Module Instantiations	 ===================================*/
`ifdef CORE_ONLY
   core_ram_struc coreR(
	.reset (reset),
	.Clk (Clk)
	);

`endif // ifdef CORE_ONLY
   
`ifdef PADS
pads fullchip (Clk, reset, mem_select, estrobe, e_access, e_enable, eab,
	       edb, e_outen, e_write, test, scanin, scanout, pwr0, pwr1, pwr2,
	       pwr3, pwr4, pwr5, pwr6, pwr7, pwr8, pwr9, gnd0, gnd1, gnd2, gnd3,
	       gnd4, gnd5, gnd6, gnd7, gnd8, gnd9);
`endif // ifdef PADS
   
`ifdef POST_LAYOUT
pads fullchip (Clk, reset, mem_select, estrobe, e_access, e_enable, eab,
	       edb, e_outen, e_write, test, scanin, scanout, pwr0, pwr1, pwr2,
	       pwr3, pwr4, pwr5, pwr6, pwr7, pwr8, pwr9, gnd0, gnd1, gnd2, gnd3,
	       gnd4, gnd5, gnd6, gnd7, gnd8, gnd9);
`endif // ifdef POST_LAYOUT
   

/*========================================================================*/


/*======================================================================*/
/*	initialize							*/
/*======================================================================*/

initial
  begin
     /*
      $monitor ($time, "\t****\tClk = %b   RAM_wr = %b   Clk = %b & C_YWRITE = %b = core_y_write = %b",
		Clk,
		top.fullchip.pads_core.processor_memory_yram_cell.WR,
		top.fullchip.pads_core.ESTROBE,
		top.fullchip.pads_core.processor_C_YWRITE,
		top.fullchip.pads_core.processor_bus_control_re_control_core_y_write);
      */


`ifdef SCAN
   `ifdef POST_LAYOUT
      $monitor ("monitor: time %0d PAB_reg_out %b, PABwrite_out %b, PAB2 (agu) %b", $time, pads.pads_core.processor_dsp_PCU_PAB_reg_out, pads.pads_core.SCAN_OUT, pads.pads_core.processor_dsp_agu.pdb2);
   `endif
`endif   


      
`ifdef SHM      
      /* generates a dump file for Cwaves to use */
      $shm_open("waves.shm");
      /* currently no dumps for CORE_ONLY */
`ifdef CORE_ONLY
      $shm_probe("AS");
      
`endif // ifdef CORE_ONLY

`ifdef PADS
//      $shm_probe(fullchip,"A",fullchip.processor,"A"
//		 );
      $shm_probe("AS");
`endif // ifdef SHM
`ifdef POST_LAYOUT
//      $shm_probe(top,"A", fullchip,"A",fullchip.pads_core,"A");
//      		 fullchip.pads_core.processor_dsp_ALU, "AS");
      $shm_probe (fullchip.pads_core, "AS");
`endif // ifdef POST_LAYOUT
`endif // ifdef SHM
            

      /* open text file for dumping results */
      /* always open dump file, but only write data if CYCLE_DUMP */
      dumpfile = $fopen("verilogSim.dump");


      /* Start simulation with reset signal high */
      Clk = 0;
      reset = 0;	
      e_enable = 0;
      mem_select = 2'b10;
      e_outen = 1;
      e_write = 1;
      scanin = 0;
      test = 0;
      control_scanin = 0;
      eab_input = 0;
      edb_input = 0;
      dummy_addr = 16'b0;
      dummy_data = 24'b0;
      
      /* give inputs a chance to settle */
      #(`CLOCK_CYCLE) ;
            
      /* load data into RAM modules 		*/
      /* start at RAMData[00]             */
`ifdef INTERFACE
      /*
       * read from files into the fake memories
       */
      $readmemh(`pmem_file, temp_P_ram, 16'h0000);
      $readmemh(`xmem_file, temp_X_ram, 16'h0000);
      $readmemh(`ymem_file, temp_Y_ram, 16'h0000);
      
      /*
       * Enter memory mode, wait two cycles, then drive busses
       */
      e_enable <= 1;
      #(`CLOCK_CYCLE * 2) ;
      edb_input = 1;
      eab_input = 1;
      

      /*
       * write the values into the memory
       *
       */
      $display ("Loading the memory . . .\n");
      for (j = 0; j < 1024; j = j + 1)
	 begin
	    //  $display ("Loop count is: %x\n",j);
	    dummy_addr = j;
	    for (vil = 0; vil < 3'b100; vil = vil + 2'b01)
	       begin
		  // $display ("vil is %x\n",vil);
		  mem_select = vil;
		  case (vil)
		    2'b00 : 
		       begin
			  write_mem(dummy_addr, temp_X_ram[j]);
		       end
		    2'b01 : 
		       begin
			  write_mem(dummy_addr, temp_Y_ram[j]);
		       end
		    2'b11 : 
		       begin
			  write_mem(dummy_addr, temp_P_ram[j]);
		       end
		    default : 
		       begin
			  e_write = 1'b1;
		       end
		  endcase
	       end
	 end // for (j = 0; ...
      
      /*
       * Leave memory mode, wait two cycles
       */
      e_write <= 1'b1;
      e_enable <= 0;
      edb_input = 0;
      eab_input = 0;
      #(`CLOCK_CYCLE * 2) ;
`else // !ifdef INTERFACE
`ifdef CORE_ONLY
      $readmemh( `pmem_file, coreR.PRAM.RAMData, 8'h00);
      $readmemh( `xmem_file, coreR.XRAM.RAMData, 12'h00);
      $readmemh( `ymem_file, coreR.YRAM.RAMData, 12'h00);
`endif // ifdef CORE_ONLY
      
`ifdef PADS
      $readmemh( `pmem_file, 
		 fullchip.processor.memory.pram_cell.mem, 8'h00);
      $readmemh( `xmem_file, 
		 fullchip.processor.memory.xram_cell.mem, 8'h00);
      $readmemh( `ymem_file, 
		 fullchip.processor.memory.yram_cell.mem, 8'h00);
`endif // ifdef PADS
`ifdef POST_LAYOUT
      $readmemh( `pmem_file, 
		 fullchip.pads_core.processor_memory_pram_cell.ram.inst1.mem, 
		 8'h00);
      $readmemh( `xmem_file,
		 fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem, 
		 8'h00);
      $readmemh( `ymem_file, 
		 fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem, 
		 8'h00);
`endif // ifdef POST_LAYOUT
`endif // !ifdef INTERFACE
            
      /* Start reset cycle */
      reset = 1;

      /* End of reset cycle */
      #(`CLOCK_CYCLE);
      #(`CLOCK_CYCLE);
      #(`CLOCK_CYCLE/2);
      #(`CLOCK_CYCLE/2);
      reset = 0;

//      $monitor ($time, "\t%b  %b\n", 
//	fullchip.pads_core.processor_C_PAB, fullchip.pads_core.processor_C_PDB);
//	fullchip.pads_core.processor_C_XAB, fullchip.pads_core.processor_C_XDB);
//	fullchip.pads_core.processor_C_XAB, fullchip.pads_core.processor_C_XDB);

      /*
       *dump the input and output ram areas before simulation
       */
      $display ("Dumping pre-simulation memory...");
      mem_dump = $fopen ("verilog_input_dump");
      for (j=0; j <= 1024; j = j + 4)
	 
`ifdef CORE_ONLY
	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    coreR.XRAM.RAMData[j],coreR.XRAM.RAMData[j+1],
		    coreR.XRAM.RAMData[j+2],coreR.XRAM.RAMData[j+3]);
`endif // ifdef CORE_ONLY
      
`ifdef PADS
      $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		 fullchip.processor.memory.xram_cell.mem[j],
		 fullchip.processor.memory.xram_cell.mem[j+1],
		 fullchip.processor.memory.xram_cell.mem[j+2],
		 fullchip.processor.memory.xram_cell.mem[j+3]);
`endif // ifdef PADS

`ifdef POST_LAYOUT
      $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		 fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j],
		 fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+1],
		 fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+2],
		 fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+3]);
`endif // ifdef POST_LAYOUT

      for (j=0; j <= 1024; j = j + 4)
`ifdef CORE_ONLY
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    coreR.YRAM.RAMData[j],coreR.YRAM.RAMData[j+1],
		    coreR.YRAM.RAMData[j+2],coreR.YRAM.RAMData[j+3]);
`endif // ifdef CORE_ONLY
      
`ifdef PADS	 
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    fullchip.processor.memory.yram_cell.mem[j],
		    fullchip.processor.memory.yram_cell.mem[j+1],
		    fullchip.processor.memory.yram_cell.mem[j+2],
		    fullchip.processor.memory.yram_cell.mem[j+3]);
`endif // ifdef PADS

`ifdef POST_LAYOUT
      $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		 fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j],
		 fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+1],
		 fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+2],
		 fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+3]);
`endif // ifdef POST_LAYOUT
      $fclose (mem_dump);
      
      
      /*
       * Simulte the program
       */
      $display ("Running the program...");


`ifdef SCAN
      /*
       * let it run for a while first 
       */
//      #(`CLOCK_CYCLE * 200 - 2);  Huh, minus 2???
      #(`CLOCK_CYCLE * 200);

/*
      $fdisplay (dumpfile, "LF is %b, DM is %b, unbuf_S1 is %b, unbuf_S0 is %b, I1 is %b, I0 is %b",
               pads.pads_core.processor_dsp_PCU_LF,
               pads.pads_core.processor_dsp_PCU_DM,
               pads.pads_core.processor_dsp_PCU_unbuf_S1,
               pads.pads_core.processor_dsp_PCU_unbuf_S0,
               pads.pads_core.processor_dsp_PCU_I1,
               pads.pads_core.processor_dsp_PCU_I0);
      $fdisplay (dumpfile, "OMR is %b",pads.pads_core.processor_dsp_PCU_OMR);
*/

      $fdisplay (dumpfile, "End of the scan chain");
      $fdisplay (dumpfile, "PABreg_out is %b, PABwrite_out is %b\n",
               pads.pads_core.processor_dsp_PCU_PAB_reg_out,
               /* pads.pads_core.processor_dsp_PCU_PABwrite_out);  */
               pads.pads_core.SCAN_OUT); /* */

      $fdisplay (dumpfile, "position 107 in the scan chain");
      $fdisplay (dumpfile, "PDB2 in the AGU is %b\n",pads.pads_core.processor_dsp_agu.pdb2);

      $fdisplay (dumpfile, "Beginning of the scan chain, starting at position 0");
      $fdisplay (dumpfile, "immediate_checker %b, acw_xab_select_ltch %b, acw_yab_select_ltch %b, acw_PAB_2_select_ltch %b, acw_xwrite_ltch %b, acw_xread_ltch %b, acw_ywrite_ltch %b, acw_yread_ltch %b, acw_PABwrite_2_ltch %b, acw_gdb_out_select_ltch %b, acw_gdb_write %b, acw_addr_alu_H_N_or_1 %b, acw_addr_alu_H_add_sub %b, acw_addr_alu_L_N_or_1 %b, acw_addr_alu_L_add_sub %b\n",
               pads.pads_core.processor_dsp_agu.agu_control_immediate_checker_n1459,
               pads.pads_core.processor_dsp_agu.xab_select,
               pads.pads_core.processor_dsp_agu.yab_select,
               pads.pads_core.processor_dsp_agu.PAB_2_select,
               pads.pads_core.processor_dsp_agu.xwrite,
               pads.pads_core.processor_dsp_agu.xread,
               pads.pads_core.processor_dsp_agu.ywrite,
               pads.pads_core.processor_dsp_agu.yread,
               pads.pads_core.processor_dsp_agu.PABwrite_2,
               pads.pads_core.processor_dsp_agu.gdb_out_select,
               pads.pads_core.processor_dsp_agu.gdb_write,
               pads.pads_core.processor_dsp_agu.addr_alu_H_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_H_add_sub,
               pads.pads_core.processor_dsp_agu.addr_alu_L_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_L_add_sub);



      #(`CLOCK_CYCLE);
      $fdisplay (dumpfile, "End of the scan chain");
      $fdisplay (dumpfile, "PABreg_out is %b, PABwrite_out is %b\n",
               pads.pads_core.processor_dsp_PCU_PAB_reg_out,
               /* pads.pads_core.processor_dsp_PCU_PABwrite_out);  */
               pads.pads_core.SCAN_OUT); /* */

      $fdisplay (dumpfile, "position 107 in the scan chain");
      $fdisplay (dumpfile, "PDB2 in the AGU is %b\n",pads.pads_core.processor_dsp_agu.pdb2);

      $fdisplay (dumpfile, "Beginning of the scan chain, starting at position 0");
      $fdisplay (dumpfile, "immediate_checker %b, acw_xab_select_ltch %b, acw_yab_select_ltch %b, acw_PAB_2_select_ltch %b, acw_xwrite_ltch %b, acw_xread_ltch %b, acw_ywrite_ltch %b, acw_yread_ltch %b, acw_PABwrite_2_ltch %b, acw_gdb_out_select_ltch %b, acw_gdb_write %b, acw_addr_alu_H_N_or_1 %b, acw_addr_alu_H_add_sub %b, acw_addr_alu_L_N_or_1 %b, acw_addr_alu_L_add_sub %b\n",
               pads.pads_core.processor_dsp_agu.agu_control_immediate_checker_n1459,
               pads.pads_core.processor_dsp_agu.xab_select,
               pads.pads_core.processor_dsp_agu.yab_select,
               pads.pads_core.processor_dsp_agu.PAB_2_select,
               pads.pads_core.processor_dsp_agu.xwrite,
               pads.pads_core.processor_dsp_agu.xread,
               pads.pads_core.processor_dsp_agu.ywrite,
               pads.pads_core.processor_dsp_agu.yread,
               pads.pads_core.processor_dsp_agu.PABwrite_2,
               pads.pads_core.processor_dsp_agu.gdb_out_select,
               pads.pads_core.processor_dsp_agu.gdb_write,
               pads.pads_core.processor_dsp_agu.addr_alu_H_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_H_add_sub,
               pads.pads_core.processor_dsp_agu.addr_alu_L_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_L_add_sub);


      #(`CLOCK_CYCLE);
      $fdisplay (dumpfile, "End of the scan chain");
      $fdisplay (dumpfile, "PABreg_out is %b, PABwrite_out is %b\n",
               pads.pads_core.processor_dsp_PCU_PAB_reg_out,
               /* pads.pads_core.processor_dsp_PCU_PABwrite_out);  */
               pads.pads_core.SCAN_OUT); /* */

      $fdisplay (dumpfile, "position 107 in the scan chain");
      $fdisplay (dumpfile, "PDB2 in the AGU is %b\n",pads.pads_core.processor_dsp_agu.pdb2);

      $fdisplay (dumpfile, "Beginning of the scan chain, starting at position 0");
      $fdisplay (dumpfile, "immediate_checker %b, acw_xab_select_ltch %b, acw_yab_select_ltch %b, acw_PAB_2_select_ltch %b, acw_xwrite_ltch %b, acw_xread_ltch %b, acw_ywrite_ltch %b, acw_yread_ltch %b, acw_PABwrite_2_ltch %b, acw_gdb_out_select_ltch %b, acw_gdb_write %b, acw_addr_alu_H_N_or_1 %b, acw_addr_alu_H_add_sub %b, acw_addr_alu_L_N_or_1 %b, acw_addr_alu_L_add_sub %b",
               pads.pads_core.processor_dsp_agu.agu_control_immediate_checker_n1459,
               pads.pads_core.processor_dsp_agu.xab_select,
               pads.pads_core.processor_dsp_agu.yab_select,
               pads.pads_core.processor_dsp_agu.PAB_2_select,
               pads.pads_core.processor_dsp_agu.xwrite,
               pads.pads_core.processor_dsp_agu.xread,
               pads.pads_core.processor_dsp_agu.ywrite,
               pads.pads_core.processor_dsp_agu.yread,
               pads.pads_core.processor_dsp_agu.PABwrite_2,
               pads.pads_core.processor_dsp_agu.gdb_out_select,
               pads.pads_core.processor_dsp_agu.gdb_write,
               pads.pads_core.processor_dsp_agu.addr_alu_H_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_H_add_sub,
               pads.pads_core.processor_dsp_agu.addr_alu_L_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_L_add_sub);
      $fdisplay (dumpfile, "%b %b %b %b %b %b %b %b %b %b %b %b %b %b %b\n",
               pads.pads_core.processor_dsp_agu.agu_control_immediate_checker_n1459,
               pads.pads_core.processor_dsp_agu.xab_select,
               pads.pads_core.processor_dsp_agu.yab_select,
               pads.pads_core.processor_dsp_agu.PAB_2_select,
               pads.pads_core.processor_dsp_agu.xwrite,
               pads.pads_core.processor_dsp_agu.xread,
               pads.pads_core.processor_dsp_agu.ywrite,
               pads.pads_core.processor_dsp_agu.yread,
               pads.pads_core.processor_dsp_agu.PABwrite_2,
               pads.pads_core.processor_dsp_agu.gdb_out_select,
               pads.pads_core.processor_dsp_agu.gdb_write,
               pads.pads_core.processor_dsp_agu.addr_alu_H_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_H_add_sub,
               pads.pads_core.processor_dsp_agu.addr_alu_L_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_L_add_sub);


      #(`CLOCK_CYCLE / 2);
      
      /*
       * now let's scan out contents of scannable registers
       * put a message in the log that we can out at time whatever
       * toggle scan mode on the chip; initialize the counters to store
       * the scan results; and save the first item from the scan chain
       * before we clock the chip
       *
       * control_scanin = 0 for scanout
       */
      $display ("Pause: scan out register contents...(%0d)", $time);
      scanin <= 1'b1;
      test <= 1;
      control_scanin <= 0;  
      i = `SCAN_CHAIN_LENGTH+`N_LENGTH-1;  
      scan_output[`SCAN_CHAIN_LENGTH+`N_LENGTH] = scanout;


      /* 
       * it takes SCAN_CHAIN_LENGTH cycles to scan out the entire chain
       */
      #(`CLOCK_CYCLE * (`SCAN_CHAIN_LENGTH + `N_LENGTH));

      
      /* 
	    * print out contents of array buffers , check what we've just scanned
	    * out 
	    */
      for (j=0; j<=(`SCAN_CHAIN_LENGTH + `N_LENGTH); j = j+1)
	      $fdisplay(dumpfile,"scan_output at %0d is %0h", j, scan_output[j]);
  
      /*
       * now let's scan the data back into the scannable registers
       */
      $display ("Pause: scan in register contents...(%0d)",$time);
      scanin <= scan_output[`SCAN_CHAIN_LENGTH+`N_LENGTH];
      i = `SCAN_CHAIN_LENGTH+`N_LENGTH-1;
      test <= 1;
      control_scanin <= 1;
      
      /* 
       * SCAN_CHAIN_LENGTH cycles to scan back in 
       */
      #(`CLOCK_CYCLE * (`SCAN_CHAIN_LENGTH + `N_LENGTH)); 
      test <= 0;
      control_scanin <= 0;



      /*
       * display state of the beginning middle and end of the scan chain
       * after we've scanned the info back in
       */
      $fdisplay (dumpfile, "End of the scan chain");
      $fdisplay (dumpfile, "PABreg_out is %b, PABwrite_out is %b\n",
               pads.pads_core.processor_dsp_PCU_PAB_reg_out,
               /* pads.pads_core.processor_dsp_PCU_PABwrite_out);  */
               pads.pads_core.SCAN_OUT); /* */

      $fdisplay (dumpfile, "position 107 in the scan chain");
      $fdisplay (dumpfile, "PDB2 in the AGU is %b\n",pads.pads_core.processor_dsp_agu.pdb2);

      $fdisplay (dumpfile, "Beginning of the scan chain, starting at position 0");
      $fdisplay (dumpfile, "immediate_checker %b, acw_xab_select_ltch %b, acw_yab_select_ltch %b, acw_PAB_2_select_ltch %b, acw_xwrite_ltch %b, acw_xread_ltch %b, acw_ywrite_ltch %b, acw_yread_ltch %b, acw_PABwrite_2_ltch %b, acw_gdb_out_select_ltch %b, acw_gdb_write %b, acw_addr_alu_H_N_or_1 %b, acw_addr_alu_H_add_sub %b, acw_addr_alu_L_N_or_1 %b, acw_addr_alu_L_add_sub %b",
               pads.pads_core.processor_dsp_agu.agu_control_immediate_checker_n1459,
               pads.pads_core.processor_dsp_agu.xab_select,
               pads.pads_core.processor_dsp_agu.yab_select,
               pads.pads_core.processor_dsp_agu.PAB_2_select,
               pads.pads_core.processor_dsp_agu.xwrite,
               pads.pads_core.processor_dsp_agu.xread,
               pads.pads_core.processor_dsp_agu.ywrite,
               pads.pads_core.processor_dsp_agu.yread,
               pads.pads_core.processor_dsp_agu.PABwrite_2,
               pads.pads_core.processor_dsp_agu.gdb_out_select,
               pads.pads_core.processor_dsp_agu.gdb_write,
               pads.pads_core.processor_dsp_agu.addr_alu_H_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_H_add_sub,
               pads.pads_core.processor_dsp_agu.addr_alu_L_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_L_add_sub);
      $fdisplay (dumpfile, "%b %b %b %b %b %b %b %b %b %b %b %b %b %b %b\n",
               pads.pads_core.processor_dsp_agu.agu_control_immediate_checker_n1459,
               pads.pads_core.processor_dsp_agu.xab_select,
               pads.pads_core.processor_dsp_agu.yab_select,
               pads.pads_core.processor_dsp_agu.PAB_2_select,
               pads.pads_core.processor_dsp_agu.xwrite,
               pads.pads_core.processor_dsp_agu.xread,
               pads.pads_core.processor_dsp_agu.ywrite,
               pads.pads_core.processor_dsp_agu.yread,
               pads.pads_core.processor_dsp_agu.PABwrite_2,
               pads.pads_core.processor_dsp_agu.gdb_out_select,
               pads.pads_core.processor_dsp_agu.gdb_write,
               pads.pads_core.processor_dsp_agu.addr_alu_H_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_H_add_sub,
               pads.pads_core.processor_dsp_agu.addr_alu_L_N_or_1,
               pads.pads_core.processor_dsp_agu.addr_alu_L_add_sub);





      /*
       * finish simulating the program
       */
      #(`CLOCK_CYCLE/2);
      $display ("Continuing the program... (%0d)",$time);
      #(`CLOCK_CYCLE * (`PROG_CYCLES - 200));
      
`else // !ifdef SCAN
       
      #(`CLOCK_CYCLE * `PROG_CYCLES);

`endif // !ifdef SCAN
      
           
      
      /*
       *dump the input and output ram areas after simulation
       */
      $display ("Dumping post-simulation memory...");
      mem_dump = $fopen ("verilog_output_dump");
      for (j=0; j <= 1024; j = j + 4)
`ifdef CORE_ONLY
	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    coreR.XRAM.RAMData[j],coreR.XRAM.RAMData[j+1],
		    coreR.XRAM.RAMData[j+2],coreR.XRAM.RAMData[j+3]);
`endif // ifdef CORE_ONLY
      
`ifdef PADS
	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    fullchip.processor.memory.xram_cell.mem[j],
		    fullchip.processor.memory.xram_cell.mem[j+1],
		    fullchip.processor.memory.xram_cell.mem[j+2],
		    fullchip.processor.memory.xram_cell.mem[j+3]);
`endif // ifdef PADS

`ifdef POST_LAYOUT
      	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+1],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+2],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+3]);
`endif // ifdef POST_LAYOUT

      for (j=0; j <= 1024; j = j + 4)
`ifdef CORE_ONLY
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    coreR.YRAM.RAMData[j],coreR.YRAM.RAMData[j+1],
		    coreR.YRAM.RAMData[j+2],coreR.YRAM.RAMData[j+3]);
`endif // ifdef CORE_ONLY
      
`ifdef PADS
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    fullchip.processor.memory.yram_cell.mem[j],
		    fullchip.processor.memory.yram_cell.mem[j+1],
		    fullchip.processor.memory.yram_cell.mem[j+2],
		    fullchip.processor.memory.yram_cell.mem[j+3]);
`endif // ifdef PADS

`ifdef POST_LAYOUT
      	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+1],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+2],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+3]);
`endif // ifdef POST_LAYOUT
      $fclose (mem_dump);

      /*
       * If using interface, unload memory and overwrite the files
       */
`ifdef INTERFACE
      $display ("Reading out the memory . . .\n");
      e_enable <= 1'b1;
      e_write <= 1'b1;
      e_outen <= 1'b0;
      edb_input = 0;
      eab_input = 1;
      #(`CLOCK_CYCLE * 2) ;
      for (j = 0; j <= 1024; j = j + 1)
         begin
            for (vil = 0; vil < 3'b100; vil = vil + 1)
               begin
                  case (vil)
                     2'b00:
                        begin
                           mem_select = vil;
			   read_mem(j);
			   temp_X_ram[j] = edb;
                        end
                     2'b01:
                        begin
                           mem_select = vil;
			   read_mem(j);
			   temp_Y_ram[j] = edb;
                        end
                     2'b11:
                        begin
                           mem_select = vil;
			   read_mem(j);
			   temp_P_ram[j] = edb;
                        end
		    default: ;
		  endcase
	       end // for (vil = 0; vil < 3'b100; vil = vil + 1)
	 end // for (j = 0; ...
               
      /*
       * Dump the memories to a file
       */
      $display ("Dumping results of memory read...");
      mem_dump = $fopen ("verilog_output_dump");
      for (j=0; j <= 1024; j = j + 4)
	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    temp_X_ram[j],temp_X_ram[j+1],
		    temp_X_ram[j+2],temp_X_ram[j+3]);
      
      for (j=0; j <= 1024; j = j + 4)
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    temp_Y_ram[j],temp_Y_ram[j+1],
		    temp_Y_ram[j+2],temp_Y_ram[j+3]);
      $fclose (mem_dump);
`endif // ifdef INTERFACE
      
      

      
      $fclose(dumpfile);		

`ifdef SHM      
      $shm_close;		 /* close the dump file */
`endif // ifdef SHM
      
      $finish; /* end simulation */
      
   end



/*======================================================================*/
/*	clock	25 units High, 25 units Low				*/
/* 		on each cycle, dump register contents			*/
/*======================================================================*/

  task test_contention;
    input [5:0] id;
    input [31:0] bus;
    input in0, in1, in2, in3, in4, in5, in6, in7;

    begin
      if       ((~in0 & ~in1) |
                (~in0 & ~in2) |
                (~in0 & ~in3) |
                (~in0 & ~in4) |
                (~in0 & ~in5) |
                (~in0 & ~in6) |
                (~in0 & ~in7) |
                (~in1 & ~in2) |
                (~in1 & ~in3) |
                (~in1 & ~in4) |
                (~in1 & ~in5) |
                (~in1 & ~in6) |
                (~in1 & ~in7) |
                (~in2 & ~in3) |
                (~in2 & ~in4) |
                (~in2 & ~in5) |
                (~in2 & ~in6) |
                (~in2 & ~in7) |
                (~in3 & ~in4) |
                (~in3 & ~in5) |
                (~in3 & ~in6) |
                (~in3 & ~in7) |
                (~in4 & ~in5) |
                (~in4 & ~in6) |
                (~in4 & ~in7) |
                (~in5 & ~in6) |
                (~in5 & ~in7) |
                (~in6 & ~in7))
	$display("Conflict ID #%d, Bus: %h, Control: %b, %b, %b, %b, %b, %b, %b, %b",
		id, bus, in0, in1, in2, in3, in4, in5, in6, in7);
     end
  endtask

`ifdef PADS
  always @(posedge Clk) begin
    test_contention(1, fullchip.processor.EAB,		// EAB
	fullchip.processor.bus_control.ebus_interface.xbus_enable,
    	fullchip.processor.bus_control.ebus_interface.ybus_enable,
    	fullchip.processor.bus_control.ebus_interface.pbus_enable,
    	1, 1, 1, 1, 1);
    test_contention(2, fullchip.processor.EDB,		// EDB
	fullchip.processor.bus_control.ebus_interface.xbus_enable,
    	fullchip.processor.bus_control.ebus_interface.ybus_enable,
    	fullchip.processor.bus_control.ebus_interface.pbus_enable,
    	fullchip.processor.bus_control.npram_to_RS,
    	fullchip.processor.bus_control.nxram_to_RS,
    	fullchip.processor.bus_control.nyram_to_RS,
        1, 1);
    test_contention(3, fullchip.processor.dsp.GDB,	// GDB
	fullchip.processor.dsp.bus_switch.xdb_to_gdb_inv,
	fullchip.processor.dsp.bus_switch.ydb_to_gdb_inv,
	fullchip.processor.dsp.bus_switch.pdb_to_gdb_inv,
	~fullchip.processor.dsp.agu.gdb_write,
	1, 1, 1, 1);
    test_contention(4, fullchip.processor.C_PAB,	// PAB
	~fullchip.processor.dsp.PCU.PABwrite,
	~fullchip.processor.dsp.agu.PABwrite_2,
	1, 1, 1, 1, 1, 1);
    test_contention(5, fullchip.processor.C_XDB,	// XDB
	fullchip.processor.dsp.bus_switch.ydb_to_xdb_inv,
	fullchip.processor.dsp.bus_switch.gdb_to_xdb_inv,
	fullchip.processor.dsp.ALU.x1_to_XDB_inv,
	fullchip.processor.dsp.ALU.x0_to_XDB_inv,
	fullchip.processor.dsp.ALU.l1_to_XDB_inv,
	fullchip.processor.dsp.ALU.l0_to_XDB_inv,
	fullchip.processor.dsp.ALU.a0_to_XDB_inv,
	fullchip.processor.bus_control.nxram_to_C);
    test_contention(6, fullchip.processor.C_YDB,	// YDB
	fullchip.processor.dsp.bus_switch.xdb_to_ydb_inv,
	fullchip.processor.dsp.bus_switch.gdb_to_ydb_inv,
	fullchip.processor.dsp.ALU.y1_to_YDB_inv,
	fullchip.processor.dsp.ALU.y0_to_YDB_inv,
	fullchip.processor.dsp.ALU.l1_to_YDB_inv,
	fullchip.processor.dsp.ALU.l0_to_YDB_inv,
	fullchip.processor.dsp.ALU.b0_to_YDB_inv,
	fullchip.processor.bus_control.nyram_to_C);
    #1;
  end
`endif // ifdef PADS

always
   begin
      #(`CLOCK_CYCLE/2) ;
      
      /* dump contents of architected registers */
      /* to send only to a file, use dumpfile instead of */
      /* dumpfile_and_stdout */
`ifdef CYCLE_DUMP
      if (e_access == 1) 
`ifdef PADS
      $fdisplay(dumpfile,
		"\nCore signals: \n\t PAB = %h \t XAB = %h \t YAB = %h \n",
		fullchip.processor.C_PAB, 
		fullchip.processor.C_XAB, 
		fullchip.processor.C_YAB, 
		"\t PDB = %h \t XDB = %h \t YDB = %h \n",
		fullchip.processor.C_PDB, 
		fullchip.processor.C_XDB, 
		fullchip.processor.C_YDB, 
		
		"PCU Registers: \n",
		"\t PC = %h \n",
		fullchip.processor.dsp.PCU.PC,
		"\t LC = %h \n",
		fullchip.processor.dsp.PCU.LC,
		
		"ALU Registers: \n",
		"\t x = %h  \ty = %h\n",
		{fullchip.processor.dsp.ALU.x1, 
		 fullchip.processor.dsp.ALU.x0},
		{fullchip.processor.dsp.ALU.y1, 
		 fullchip.processor.dsp.ALU.y0},
		"\t A = %h \t B = %h \n",
		{fullchip.processor.dsp.ALU.a2, 
		 fullchip.processor.dsp.ALU.a1, 
		 fullchip.processor.dsp.ALU.a0},
		{fullchip.processor.dsp.ALU.b2, 
		 fullchip.processor.dsp.ALU.b1, 
		 fullchip.processor.dsp.ALU.b0},
		"next",$time,
		"\n\n"
		);
`endif // ifdef PADS
`ifdef POST_LAYOUT
            $fdisplay(dumpfile,
		"\nCore signals: \n\t PAB = %h \t XAB = %h \t YAB = %h \n",
		fullchip.pads_core.processor_C_PAB, 
		fullchip.pads_core.processor_C_XAB, 
		fullchip.pads_core.processor_C_YAB, 
		"\t PDB = %h \t XDB = %h \t YDB = %h \n",
		fullchip.pads_core.processor_C_PDB, 
		fullchip.pads_core.processor_C_XDB, 
		fullchip.pads_core.processor_C_YDB, 
		
		"PCU Registers: \n",
		"\t PC = %h \n",
		fullchip.pads_core.processor_dsp_PCU_PC,
		"\t LC = %h \n",
		fullchip.pads_code.processor_dsp_PCU_LC,
		
		
		"ALU Registers: \n",
		"\t x = %h  \ty = %h\n",
		{fullchip.pads_core.processor_dsp_ALU.x1, 
		 fullchip.pads_core.processor_dsp_ALU.x0},
		{fullchip.pads_core.processor_dsp_ALU.y1, 
		 fullchip.pads_core.processor_dsp_ALU.y0},
		"\t A = %h \t B = %h \n",
		{fullchip.pads_core.processor_dsp_ALU.a2, 
		 fullchip.pads_core.processor_dsp_ALU.a1, 
		 fullchip.pads_core.processor_dsp_ALU.a0},
		{fullchip.pads_core.processor_dsp_ALU.b2, 
		 fullchip.pads_core.processor_dsp_ALU.b1, 
		 fullchip.pads_core.processor_dsp_ALU.b0},
		"next",$time,
		"\n\n"
		);
`endif // ifdef POST_LAYOUT
`endif // ifdef CYCLE_DUMP

      
      
      Clk = 1;
      #(`CLOCK_CYCLE/2) Clk = 0;


`ifdef SCAN
      if (test == 1'b1 && control_scanin == 1'b0)     
	      begin
	         scan_output[i] = scanout;
            i = i-1;
	      end
      else if (test == 1'b1 && control_scanin == 1'b1)
	      begin
	         scanin = scan_output[i];
	         i = i-1;
	      end

      /*
	   if (test == 1'b1)
	      begin
	         $fdisplay (dumpfile, "scan cycle: %d\tAGU scanout %h\tALU scanout %h\tBus Switch scanout %h",
	               i,
	               pads.pads_core.processor_dsp_agu_scanout, 
	               pads.pads_core.processor_dsp_alu_scanout, 
	               pads.pads_core.processor_dsp_bus_switch_scanout);
	         $fdisplay (dumpfile, "ALU scan chain: pdb2_dff %h\t pdb3_dff %h\t control_p_scanout %h\t control_scanout %h\t x0 %h\t a2_dff %h\t b2_dff %h\t b0_dff %h\t a0_dff %h",
	               pads.pads_core.processor_dsp_ALU.pdb2[23],
	               pads.pads_core.processor_dsp_ALU.pdb3[23],
	               pads.pads_core.processor_dsp_ALU.control_p_scanout,
	               pads.pads_core.processor_dsp_ALU.control_scanout,
	               pads.pads_core.processor_dsp_ALU.x0[23],
	               pads.pads_core.processor_dsp_ALU.a2[7],
	               pads.pads_core.processor_dsp_ALU.b2[7],
	               pads.pads_core.processor_dsp_ALU.b0[23],
	               pads.pads_core.processor_dsp_ALU.a0[23]);
	      end
	   */

`endif // ifdef SCAN


   end
   
endmodule	/* end module top */
